@@ -868,7 +868,12 @@ static int iax_xmit_frame(struct iax_frame *f)
 										f->datalen - sizeof(struct ast_iax2_full_hdr));
 	}
 #endif
-	/* Send the frame raw */
+    if (!f->session) {
+        IAXERROR "No session in iax_xmit_frame");
+        return -1;
+    }
+
+    /* Send the frame raw */
 	res = f->session->sendto(netfd, (const char *) f->data, f->datalen,
 			IAX_SOCKOPTS, f->transfer ?
 			(struct sockaddr *)&(f->session->transfer) :
@@ -1348,10 +1353,8 @@ static int send_command_samples(struct iax_session *i, char type, int command, u
 	return __send_command(i, type, command, ts, data, datalen, seqno, 0, 0, 0, 0, samples);
 }
 
-
 int iax_transfer(struct iax_session *session, const char *number)
 {
-	static int res;				//Return Code
 	struct iax_ie_data ied;			//IE Data Structure (Stuff To Send)
 
 	// Clear The Memory Used For IE Buffer
@@ -1361,7 +1364,7 @@ int iax_transfer(struct iax_session *session, const char *number)
 	iax_ie_append_str(&ied, IAX_IE_CALLED_NUMBER, number);
 
 	// Send The Transfer Command - Asterisk Will Handle The Rest!
-	res = send_command(session, AST_FRAME_IAX, IAX_COMMAND_TRANSFER, 0, ied.buf, ied.pos, -1);
+	send_command(session, AST_FRAME_IAX, IAX_COMMAND_TRANSFER, 0, ied.buf, ied.pos, -1);
 
 	// Return Success
 	return 0;
@@ -1749,14 +1752,14 @@ int iax_video_bypass_jitter(struct iax_session *s, int mode)
 int iax_register(struct iax_session *session, const char *server, const char *peer, const char *secret, int refresh)
 {
 	/* Send a registration request */
-	char tmp[256];
+	char tmp[200];
 	char *p;
 	int res;
 	int portno = IAX_DEFAULT_PORTNO;
 	struct iax_ie_data ied;
 	struct hostent *hp;
 
-	tmp[255] = '\0';
+	tmp[199] = '\0';
 	strncpy(tmp, server, sizeof(tmp) - 1);
 	p = strchr(tmp, ':');
 	if (p) {
@@ -1775,9 +1778,11 @@ int iax_register(struct iax_session *session, const char *server, const char *pe
 	/* Connect first */
 	hp = gethostbyname(tmp);
 	if (!hp) {
-		snprintf(iax_errstr, sizeof(iax_errstr), "Invalid hostname: %s", tmp);
+		snprintf(iax_errstr, sizeof(iax_errstr) - 1, "Invalid hostname: %s", tmp);
+		iax_errstr[255] = '\0';
 		return -1;
 	}
+
 	memcpy(&session->peeraddr.sin_addr, hp->h_addr, sizeof(session->peeraddr.sin_addr));
 	session->peeraddr.sin_port = htons(portno);
 	session->peeraddr.sin_family = AF_INET;
@@ -1792,13 +1797,13 @@ int iax_register(struct iax_session *session, const char *server, const char *pe
 int iax_unregister(struct iax_session *session, const char *server, const char *peer, const char *secret, const char *reason)
 {
 	/* Send an unregistration request */
-	char tmp[256];
+	char tmp[200];
 	char *p;
 	int portno = IAX_DEFAULT_PORTNO;
 	struct iax_ie_data ied;
 	struct hostent *hp;
 
-	tmp[255] = '\0';
+	tmp[199] = '\0';
 	strncpy(tmp, server, sizeof(tmp) - 1);
 	p = strchr(tmp, ':');
 	if (p) {
@@ -1820,9 +1825,11 @@ int iax_unregister(struct iax_session *session, const char *server, const char *
 	/* Connect first */
 	hp = gethostbyname(tmp);
 	if (!hp) {
-		snprintf(iax_errstr, sizeof(iax_errstr), "Invalid hostname: %s", tmp);
+		snprintf(iax_errstr, sizeof(iax_errstr) - 1, "Invalid hostname: %s", tmp);
+		iax_errstr[255] = '\0';
 		return -1;
 	}
+
 	memcpy(&session->peeraddr.sin_addr, hp->h_addr, sizeof(session->peeraddr.sin_addr));
 	session->peeraddr.sin_port = htons(portno);
 	session->peeraddr.sin_family = AF_INET;
@@ -2530,7 +2537,7 @@ static void iax_handle_vnak(struct iax_session *session, struct ast_iax2_full_hd
 			{
 				/*
 				 * We cannot retransmit immediately, since the frames are ordered by retransmit time
-				 * We need to collect them and orrange them in ascending order of their oseqno
+				 * We need to collect them and arrange them in ascending order of their oseqno
 				 */
 				tmp = (struct iax_sched *)calloc(1, sizeof(struct iax_sched));
 				tmp->frame = sch->frame;
@@ -2763,7 +2770,8 @@ static struct iax_event *iax_header_to_event(struct iax_session *session, struct
 				if (e->ies.codec_prefs) {
 					strncpy(session->codec_order,
 							e->ies.codec_prefs,
-							sizeof(session->codec_order));
+							sizeof(session->codec_order) - 1);
+					session->codec_order[31] = '\0';
 					session->codec_order_len =
 						(int)strlen(session->codec_order);
 				}
