@@ -1,24 +1,10 @@
-// FGAIBase - abstract base class for AI objects
-// Written by David Culp, started Nov 2003, based on
-// David Luff's FGAIEntity class.
-// - davidculp2@comcast.net
-//
-// With additions by Mathias Froehlich & Vivian Meazza 2004 -2007
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
+/*
+ * SPDX-FileName: AIBase.cxx
+ * SPDX-FileComment: abstract base class for AI objects, based on David Luff's FGAIEntity class.
+ * SPDX-FileCopyrightText: Written by David Culp, started Nov 2003 - davidculp2@comcast.net
+ * SPDX-FileContributor: With additions by Mathias Froehlich & Vivian Meazza 2004-2007
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include <config.h>
 
@@ -60,7 +46,7 @@ using namespace simgear;
 
 class FGAIModelData : public simgear::SGModelData {
 public:
-    FGAIModelData(SGPropertyNode *root = nullptr)
+    explicit FGAIModelData(SGPropertyNode *root = nullptr)
       : _root (root)
     {
     }
@@ -119,7 +105,7 @@ public:
      * Not thread-safe. Call from main thread only. */
     void init(void) { _initialized = true; }
 
-    bool needInitilization(void) { return _ready && !_initialized;}
+    bool needInitialization(void) { return _ready && !_initialized;}
     bool isInitialized(void) { return _initialized;}
     inline std::string& get_sound_path() { return _fxpath;}
 
@@ -144,18 +130,18 @@ private:
     ErrorContext _errorContext;
 };
 
-FGAIBase::FGAIBase(object_type ot, bool enableHot) :
-    replay_time(fgGetNode("sim/replay/time", true)),
-    model_removed( fgGetNode("/ai/models/model-removed", true) ),
-    _impact_lat(0),
-    _impact_lon(0),
-    _impact_elev(0),
-    _impact_hdg(0),
-    _impact_pitch(0),
-    _impact_roll(0),
-    _impact_speed(0),
-    _refID( _newAIModelID() ),
-    _otype(ot)
+FGAIBase::FGAIBase(object_type ot, bool enableHot) : replay_time(fgGetNode("sim/replay/time", true)),
+                                                     model_removed(fgGetNode("/ai/models/model-removed", true)),
+                                                     pos(SGGeod::fromDeg(0.0, 0.0)),
+                                                     _impact_lat(0),
+                                                     _impact_lon(0),
+                                                     _impact_elev(0),
+                                                     _impact_hdg(0),
+                                                     _impact_pitch(0),
+                                                     _impact_roll(0),
+                                                     _impact_speed(0),
+                                                     _refID(_newAIModelID()),
+                                                     _otype(ot)
 {
     tgt_heading = hdg = tgt_altitude_ft = tgt_speed = 0.0;
     tgt_roll = roll = tgt_pitch = tgt_yaw = tgt_vs = vs_fps = pitch = 0.0;
@@ -178,7 +164,6 @@ FGAIBase::FGAIBase(object_type ot, bool enableHot) :
     _roll_offset = 0;
     _yaw_offset = 0;
 
-    pos = SGGeod::fromDeg(0, 0);
     speed = 0;
     altitude_ft = 0;
     speed_north_deg_sec = 0;
@@ -194,12 +179,6 @@ FGAIBase::FGAIBase(object_type ot, bool enableHot) :
 
     serviceable = false;
 
-    rho = 1;
-    T = 280;
-    p = 1e5;
-    a = 340;
-    Mach = 0;
-
     // explicitly disable HOT for (most) AI models
     if (!enableHot)
         aip.getSceneGraph()->setNodeMask(~SG_NODEMASK_TERRAIN_BIT);
@@ -237,6 +216,9 @@ FGAIBase::removeModel()
         aip.clear();
         _modeldata = nullptr;
         _model = nullptr;
+        _interior = nullptr;
+        _high_res = nullptr;
+        _low_res = nullptr;
 
         // pass it on to the pager, to be be deleted in the pager thread
         pSceneryManager->getPager()->queueDeleteRequest(temp);
@@ -260,7 +242,7 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
         return;
 
     setPath(scFileNode->getStringValue("model",
-            fgGetString("/sim/multiplay/default-model", default_model)));
+            fgGetString("/sim/multiplay/default-model", default_model).c_str()).c_str());
 
     setFallbackModelIndex(scFileNode->getIntValue("fallback-model-index", 0));
 
@@ -271,6 +253,8 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
     setLatitude(scFileNode->getDoubleValue("latitude", 0.0));
     setBank(scFileNode->getDoubleValue("roll", 0.0));
     setPitch(scFileNode->getDoubleValue("pitch", 0.0));
+    setCollisionHeight(scFileNode->getDoubleValue("collision-height", 0.0));
+    setCollisionLength(scFileNode->getDoubleValue("collision-length", 0.0));
 
     SGPropertyNode* submodels = scFileNode->getChild("submodels");
 
@@ -282,14 +266,13 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
     string searchOrder = scFileNode->getStringValue("search-order", "");
     if (!searchOrder.empty()) {
         if (searchOrder == "DATA_ONLY") {
-            _searchOrder = DATA_ONLY;
+            _searchOrder = ModelSearchOrder::DATA_ONLY;
         } else if (searchOrder == "PREFER_AI") {
-            _searchOrder = PREFER_AI;
+            _searchOrder = ModelSearchOrder::PREFER_AI;
         } else if (searchOrder == "PREFER_DATA") {
-            _searchOrder = PREFER_DATA;
+            _searchOrder = ModelSearchOrder::PREFER_DATA;
         } else
             SG_LOG(SG_AI, SG_WARN, "invalid model search order " << searchOrder << ". Use either DATA_ONLY, PREFER_AI or PREFER_DATA");
-
     }
 
     const string modelLowres = scFileNode->getStringValue("model-lowres", "");
@@ -303,16 +286,13 @@ void FGAIBase::update(double dt) {
     SG_UNUSED(dt);
     if (replay_time->getDoubleValue() > 0)
         return;
-    if (_otype == otStatic)
+    if (_otype == object_type::otStatic)
         return;
 
-    if (_otype == otBallistic)
-        CalculateMach();
-
     ft_per_deg_lat = 366468.96 - 3717.12 * cos(pos.getLatitudeRad());
     ft_per_deg_lon = 365228.16 * cos(pos.getLatitudeRad());
 
-    if ((_modeldata)&&(_modeldata->needInitilization()))
+    if ((_modeldata)&&(_modeldata->needInitialization()))
     {
         // process deferred nasal initialization,
         // which must be done in main thread
@@ -334,6 +314,8 @@ void FGAIBase::update(double dt) {
             const string& fxpath = _modeldata->get_sound_path();
             if (fxpath != "")
             {
+                removeSoundFx();
+
                 simgear::ErrorReportContext ec("ai-model", _name);
                 if (!_scenarioPath.empty()) {
                     ec.add("scenario-path", _scenarioPath);
@@ -341,9 +323,6 @@ void FGAIBase::update(double dt) {
 
                 props->setStringValue("sim/sound/path", fxpath.c_str());
 
-                // Remove any existing sound FX (e.g. from another model)
-                removeSoundFx();
-
                 // initialize the sound configuration
                 std::stringstream name;
                 name <<  "aifx:";
@@ -378,6 +357,7 @@ void FGAIBase::updateInterior()
 
     if (!_modeldata->getInteriorLoaded()) { // interior is not yet load
         _interior = SGModelLib::loadPagedModel(_modeldata->getInteriorPath(), props, _modeldata);
+        _group->addChild(_interior);
         if (_interior.valid()) {
             bool pixel_mode = !fgGetBool("/sim/rendering/static-lod/aimp-range-mode-distance", false);
             if (pixel_mode) {
@@ -388,7 +368,6 @@ void FGAIBase::updateInterior()
                 _interior->setRangeMode(osg::LOD::DISTANCE_FROM_EYE_POINT);
                 _interior->setRange(0, 0.0, _maxRangeInterior);
             }
-            aip.add(_interior.get());
             _modeldata->setInteriorLoaded(true);
             SG_LOG(SG_AI, SG_INFO, "AIBase: Loaded interior model " << _interior->getName());
         }
@@ -416,23 +395,27 @@ void FGAIBase::updateLOD()
         {
             // High detail model (only)
             // - disables the low detail detail model by setting its visibility from 0 to 0
-            if (_model->getNumFileNames() == 2) {
+            if (_high_res.valid()) {
                 _model->setRange(modelHighDetailIndex, 0.0, FLT_MAX); // all ranges.
                 _model->setRange(modelLowDetailIndex, 0.0, 0.0); // turn it off
             }
             else {
-                _model->setRange(0, 0.0, FLT_MAX); // only one model.
+                // only having low-res model
+                _model->setRange(modelLowDetailIndex, 0.0, FLT_MAX); // only having low-res model.
+                _model->setRange(modelHighDetailIndex, 0.0, 0.0); // only having low-res model.
             }
         }
         else if ((int)maxRangeBare == (int)maxRangeDetail)
         {
             // low detail model  (only)
-            if (_model->getNumFileNames() == 2) {
+            if (_low_res.valid()) {
                 _model->setRange(modelHighDetailIndex, 0, 0); // turn it off
                 _model->setRange(modelLowDetailIndex, 0, FLT_MAX);
             }
             else {
-                _model->setRange(0, 0, FLT_MAX);
+                // Only having high_res model
+                _model->setRange(modelHighDetailIndex, 0, FLT_MAX);
+                _model->setRange(modelLowDetailIndex, 0, 0.0);
             }
         }
         else
@@ -459,7 +442,7 @@ void FGAIBase::updateLOD()
                   );
                 }
 
-                if (_model->getNumFileNames() == 2) {
+                if (_low_res.valid() && _high_res.valid()) {
                     /*if (_model->getRadius() < 0)
                     {
                         osg::BoundingSphere bs = _model->computeBound();
@@ -470,9 +453,14 @@ void FGAIBase::updateLOD()
                     }*/
                   _model->setRange(modelHighDetailIndex , maxRangeDetail, FLT_MAX); // most detailed
                   _model->setRange(modelLowDetailIndex , maxRangeBare, maxRangeDetail); // least detailed
-                } else {
-                  // we have only one model it obviously will have to be displayed from the smallest value
-                  _model->setRange(0, min(maxRangeBare, maxRangeDetail), FLT_MAX );
+                } else if (_low_res.valid() && !_high_res.valid()) {
+                  // we have only low_res_model model it obviously will have to be displayed from the smallest value
+                  _model->setRange(modelLowDetailIndex, std::min(maxRangeBare, maxRangeDetail), FLT_MAX );
+                  _model->setRange(modelHighDetailIndex, 0,0);
+                } else if (!_low_res.valid() && _high_res.valid()) {
+                    // we have only high_res model it obviously will have to be displayed from the smallest value
+                    _model->setRange(modelHighDetailIndex, std::min(maxRangeBare, maxRangeDetail), FLT_MAX );
+                    _model->setRange(modelLowDetailIndex, 0,0);
                 }
             } else {
                 /* In non-pixel range mode we're dealing with straight distance.
@@ -491,11 +479,15 @@ void FGAIBase::updateLOD()
                 }
 
 
-                if (_model->getNumFileNames() == 2) {
+                if (_low_res.valid() && _high_res.valid()) {
                   _model->setRange(modelHighDetailIndex , 0, maxRangeDetail); // most detailed
                   _model->setRange(modelLowDetailIndex , maxRangeDetail, maxRangeDetail+maxRangeBare); // least detailed
-                } else {
-                  _model->setRange(0, 0, maxRangeBare + maxRangeDetail); // only one model, so display from 0 to the highest value in meters
+                } else if (_low_res.valid() && !_high_res.valid()) {
+                  _model->setRange(modelLowDetailIndex, 0, maxRangeBare + maxRangeDetail); // only low_res, so display from 0 to the highest value in meters
+                  _model->setRange(modelHighDetailIndex, 0, 0);
+                } else if (!_low_res.valid() && _high_res.valid()) {
+                    _model->setRange(modelHighDetailIndex, 0, maxRangeBare + maxRangeDetail); // only high_res, so display from 0 to the highest value in meters
+                    _model->setRange(modelLowDetailIndex, 0, 0);
                 }
             }
         }
@@ -542,7 +534,7 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
 {
     string_list path_list;
 
-    if (searchOrder == DATA_ONLY) {
+    if (searchOrder == ModelSearchOrder::DATA_ONLY) {
         SG_LOG(SG_AI, SG_DEBUG, "Resolving model path:  DATA only");
         auto p = simgear::SGModelLib::findDataFile(model_path);
         if (!p.empty()) {
@@ -598,7 +590,7 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
             }
         }
 
-        if ((searchOrder == PREFER_AI) && !path_list.empty()) {
+        if ((searchOrder == ModelSearchOrder::PREFER_AI) && !path_list.empty()) {
             // if we prefer AI, and we've got a valid AI path from above, then use it, we're done
             _installed = true;
             return path_list;
@@ -641,12 +633,48 @@ bool FGAIBase::init(ModelSearchOrder searchOrder)
     // set by FGAISchedule::createAIAircraft
     _modeldata->captureErrorContext("traffic-aircraft-callsign");
 
-    if (_otype == otMultiplayer) {
+    if (_otype == object_type::otMultiplayer) {
         _modeldata->addErrorContext("multiplayer", getCallSign());
     }
 
-    vector<string> model_list = resolveModelPath(searchOrder);
-    _model= SGModelLib::loadPagedModel(model_list, props, _modeldata);
+    // Load models
+    _model = new osg::LOD();
+    std::vector<string> model_list = resolveModelPath(searchOrder);
+    if(model_list.size() == 1 && _modeldata && _modeldata->hasInteriorPath()) {
+        // Only one model and interior available (expecting this to be a high_res model)
+        _low_res = new osg::PagedLOD(); // Dummy node to keep LOD node happy
+        _model->addChild(_low_res);
+        _high_res = SGModelLib::loadPagedModel(model_list[0], props, _modeldata);
+        _group = osg::ref_ptr<osg::Group>(new osg::Group());
+        _group->addChild(_high_res);
+        _model->addChild(_group);
+    } else if (model_list.size() == 1) {
+        // low_res model only (as we do not have any interior)
+        _low_res = SGModelLib::loadPagedModel(model_list[0], props, _modeldata);
+        _model->addChild(_low_res);
+        _group = osg::ref_ptr<osg::Group>(new osg::Group()); // Dummy node to keep LOD node happy
+        _model->addChild(_group);
+    } else {
+        // high and low-res model
+        assert(model_list.size() == 2);
+        _low_res = SGModelLib::loadPagedModel(model_list[0], props, _modeldata);
+        _model->addChild(_low_res);
+        _high_res = SGModelLib::loadPagedModel(model_list[1], props, _modeldata);
+        _group = osg::ref_ptr<osg::Group>(new osg::Group());
+        _group->addChild(_high_res);
+        _model->addChild(_group);
+    }
+
+    // Set PagedLODs to MAX Range. The visibility is controlled with the top-level LOD node
+    if(_high_res.valid()) {
+        _high_res->setRangeMode(osg::LOD::DISTANCE_FROM_EYE_POINT);
+        _high_res->setRange(0, 0, FLT_MAX);
+    }
+    if(_low_res.valid()) {
+        _low_res->setRangeMode(osg::LOD::DISTANCE_FROM_EYE_POINT);
+        _low_res->setRange(0, 0, FLT_MAX);
+    }
+
     _model->setName("AI-model range animation node");
     _model->setRadius(getDefaultModelRadius());
 
@@ -702,7 +730,6 @@ bool FGAIBase::isa( object_type otype ) {
     return otype == _otype;
 }
 
-
 void FGAIBase::bind() {
     _tiedProperties.setRoot(props);
     tie("id", SGRawValueMethods<FGAIBase,int>(*this,
@@ -761,11 +788,11 @@ void FGAIBase::bind() {
     tie("radar/rotation", SGRawValuePointer<double>(&rotation));
     tie("radar/ht-diff-ft", SGRawValuePointer<double>(&ht_diff));
     tie("subID", SGRawValuePointer<int>(&_subID));
-    tie("controls/lighting/nav-lights", SGRawValueFunctions<bool>(_isNight));
 
+    props->setStringValue("sim/model/path", model_path);
 
-    props->setBoolValue("controls/lighting/beacon", true);
-    props->setBoolValue("controls/lighting/strobe", true);
+    // note: AIAircraft creates real SGPropertyNodes for these, we don't do
+    // that here because it would bloat AIBase slightly
     props->setBoolValue("controls/glide-path", true);
 
     props->setStringValue("controls/flight/lateral-mode", "roll");
@@ -777,7 +804,7 @@ void FGAIBase::bind() {
     // The property above was incorrectly labelled 'longitude-mode' up until
     // FG 2018.4, so create an alias in case anyone is relying on the old name
     auto node = props->getNode("controls/flight/longitude-mode", true);
-    node->alias(props->getNode("controls/flight/vertical-mode"));
+    node->alias(props->getNode("controls/flight/vertical-mode"), false);
 
     props->setDoubleValue("controls/flight/target-alt", altitude_ft);
     props->setDoubleValue("controls/flight/target-pitch", pitch);
@@ -798,14 +825,11 @@ void FGAIBase::unbind() {
     removeSoundFx();
 }
 
-void FGAIBase::removeSoundFx() {
-    // drop reference to sound effects now
-    if (_fx)
-    {
-        // must remove explicitly - since the sound manager also keeps a reference
-        _fx->unbind();
-        // now drop last reference - kill the object
-        _fx = 0;
+void FGAIBase::removeSoundFx()
+{
+    if (_fx) {
+        _fx->shutdown();
+        _fx.clear();
     }
 }
 
@@ -900,6 +924,22 @@ bool FGAIBase::getGroundElevationM(const SGGeod& pos, double& elev,
                                                    _model.get());
 }
 
+SGPropertyNode* FGAIBase::getPositionFromNode(SGPropertyNode* scFileNode, const std::string &key, SGVec3d &position) {
+    SGPropertyNode* positionNode = scFileNode->getChild(key);
+    if (positionNode) {
+        // Transform to the right coordinate frame, configuration is done in
+        // the usual x-back, y-right, z-up coordinates, computations
+        // in the simulation usual body x-forward, y-right, z-down coordinates
+        position(0) = -positionNode->getDoubleValue("x-offset-m", 0);
+        position(1) = positionNode->getDoubleValue("y-offset-m", 0);
+        position(2) = -positionNode->getDoubleValue("z-offset-m", 0);
+        return positionNode;
+    }
+    else
+        position = SGVec3d::zeros();
+    return nullptr;
+}
+
 double FGAIBase::_getCartPosX() const {
     SGVec3d cartPos = getCartPos();
     return cartPos.x();
@@ -928,10 +968,8 @@ void FGAIBase::_setSubID( int s ) {
 }
 
 bool FGAIBase::setParentNode() {
-
     if (_parent == ""){
-       SG_LOG(SG_AI, SG_ALERT, "AIBase: " << _name
-            << " parent not set ");
+        SG_LOG(SG_AI, SG_ALERT, "AIBase: " << _name << " parent not set ");
        return false;
     }
 
@@ -944,33 +982,30 @@ bool FGAIBase::setParentNode() {
             model = _selected_ac;
         } else {
             model = ai->getChild(i);
-            //const string& path = ai->getPath();
             const string name = model->getStringValue("name");
 
             if (!model->nChildren()){
                 continue;
             }
+
             if (name == _parent) {
                 _selected_ac = model;  // save selected model for last iteration
                 break;
             }
-
         }
+
         if (!model)
             continue;
-
     }// end for loop
 
     if (_selected_ac != 0){
-        const string name = _selected_ac->getStringValue("name");
+        // DEADCODE: const string name = _selected_ac->getStringValue("name");
         return true;
     } else {
-        SG_LOG(SG_AI, SG_ALERT, "AIBase: " << _name
-            << " parent not found: dying ");
+        SG_LOG(SG_AI, SG_ALERT, "AIBase: " << _name << " parent not found: dying ");
         setDie(true);
         return false;
     }
-
 }
 
 double FGAIBase::_getLongitude() const {
@@ -1131,38 +1166,6 @@ int FGAIBase::_getFallbackModelIndex() const {
     return _fallback_model_index;
 }
 
-void FGAIBase::CalculateMach() {
-    // Calculate rho at altitude, using standard atmosphere
-    // For the temperature T and the pressure p,
-    double altitude = altitude_ft;
-
-    if (altitude < 36152) {		// curve fits for the troposphere
-        T = 59 - 0.00356 * altitude;
-        p = 2116 * pow( ((T + 459.7) / 518.6) , 5.256);
-    } else if ( 36152 < altitude && altitude < 82345 ) {    // lower stratosphere
-        T = -70;
-        p = 473.1 * pow( e , 1.73 - (0.000048 * altitude) );
-    } else {                                    //  upper stratosphere
-        T = -205.05 + (0.00164 * altitude);
-        p = 51.97 * pow( ((T + 459.7) / 389.98) , -11.388);
-    }
-
-    rho = p / (1718 * (T + 459.7));
-
-    // calculate the speed of sound at altitude
-    // a = sqrt ( g * R * (T + 459.7))
-    // where:
-    // a = speed of sound [ft/s]
-    // g = specific heat ratio, which is usually equal to 1.4
-    // R = specific gas constant, which equals 1716 ft-lb/slug/R
-    a = sqrt ( 1.4 * 1716 * (T + 459.7));
-
-    // calculate Mach number
-    Mach = speed/a;
-
-    // cout  << "Speed(ft/s) "<< speed <<" Altitude(ft) "<< altitude << " Mach " << Mach << endl;
-}
-
 int FGAIBase::_newAIModelID() {
     static int id = 0;
 
@@ -1183,11 +1186,20 @@ bool FGAIBase::isValid() const
 	return !fp || fp->isValidPlan();
 }
 
-osg::PagedLOD* FGAIBase::getSceneBranch() const
+osg::LOD* FGAIBase::getSceneBranch() const
 {
     return _model;
 }
 
+bool FGAIBase::modelLoaded() const
+{
+    if(_low_res.valid())
+        return _low_res->getNumChildren() >= 1;
+    else if (_high_res.valid())
+        return _high_res->getNumChildren() >= 1;
+    return false;
+}
+
 SGGeod FGAIBase::getGeodPos() const
 {
     return pos;
