@@ -23,26 +23,16 @@
 #endif
 
 #include <Main/fg_props.hxx>
-#include <Main/globals.hxx>
-#include <Scenery/scenery.hxx>
-#include <string>
-#include <cmath>
-
-using std::string;
 
 #include "AIStatic.hxx"
 
 
-FGAIStatic::FGAIStatic() : FGAIBase(otStatic, false) {
-    _searchOrder = PREFER_AI;
+FGAIStatic::FGAIStatic() : FGAIBase(object_type::otStatic, false) {
+    _searchOrder = ModelSearchOrder::DATA_ONLY;
 }
 
 
-FGAIStatic::~FGAIStatic() {
-}
-
 void FGAIStatic::update(double dt) {
    FGAIBase::update(dt);
    Transform();
 }
-
