@@ -45,7 +45,7 @@ string ATCSpeech::getSpokenNumber( string number )
 {
   string result;
   for( string::iterator it = number.begin(); it != number.end(); ++it ) {
-    if( false == result.empty() )
+    if( !result.empty() )
       result.SPACE;
     result.append( getSpokenDigit( (*it) - '0' ));
   }
@@ -79,8 +79,8 @@ string ATCSpeech::getSpokenNumber( int number, bool leadingZero, int digits )
     result.append( globals->get_locale()->getLocalizedString("minus", "atc", "minus" ) );
   }
 
-  while( false == spokenDigits.empty() ) {
-    if( false == result.empty() )
+  while( !spokenDigits.empty() ) {
+    if( !result.empty() )
       result.SPACE;
 
     result.append( spokenDigits.back() );
@@ -94,7 +94,7 @@ string ATCSpeech::getSpokenAltitude( int altitude )
 {
   string result;
   int thousands = altitude / 1000;
-  int hundrets = (altitude % 1000) / 100;
+  int hundreds = (altitude % 1000) / 100;
 
   if( thousands > 0 ) {
     result.append( getSpokenNumber(thousands) );
@@ -102,8 +102,8 @@ string ATCSpeech::getSpokenAltitude( int altitude )
     result.append( getSpokenDigit(1000) );
     result.SPACE;
   }
-  if( hundrets > 0 )
-      result.append( getSpokenNumber(hundrets) )
+  if( hundreds > 0 )
+      result.append( getSpokenNumber(hundreds) )
             .SPACE
             .append( getSpokenDigit(100) );
 
@@ -157,7 +157,7 @@ SGPropertyNode_ptr findAtisTemplate( const std::string & stationId, SGPropertyNo
   for( PropertyList::iterator asit = schemaNodes.begin(); asit != schemaNodes.end(); ++asit ) {
     SGPropertyNode_ptr ppp = (*asit)->getNode("station-starts-with", false );
     atisTemplate = (*asit)->getNode("atis", false );
-    if( false == atisTemplate.valid() ) continue; // no <atis> node - ignore entry
+    if( !atisTemplate.valid() ) continue; // no <atis> node - ignore entry
 
     PropertyList startsWithNodes = (*asit)->getChildren("station-starts-with");
     for( PropertyList::iterator swit = startsWithNodes.begin(); swit != startsWithNodes.end(); ++swit ) {
@@ -176,10 +176,10 @@ string ATISEncoder::encodeATIS( ATISInformationProvider * atisInformation )
 {
   using simgear::strutils::lowercase;
 
-  if( false == atisInformation->isValid() ) return NO_ATIS;
+  if( !atisInformation->isValid() ) return NO_ATIS;
 
   airport = FGAirport::getByIdent( atisInformation->airportId() );
-  if( false == airport.valid() ) {
+  if( !airport.valid() ) {
     SG_LOG( SG_ATC, SG_WARN, "ATISEncoder: unknown airport id " << atisInformation->airportId() );
     return NO_ATIS;
   }
@@ -187,7 +187,7 @@ string ATISEncoder::encodeATIS( ATISInformationProvider * atisInformation )
   _atis = atisInformation;
 
   // lazily load the schema file on the first call
-  if( false == atisSchemaNode.valid() ) {
+  if( !atisSchemaNode.valid() ) {
     atisSchemaNode = new SGPropertyNode();
     try
     {
@@ -204,7 +204,7 @@ string ATISEncoder::encodeATIS( ATISInformationProvider * atisInformation )
   string stationId = lowercase( airport->ident() );
   
   SGPropertyNode_ptr atisTemplate = findAtisTemplate( stationId, atisSchemaNode );;
-  if( false == atisTemplate.valid() ) {
+  if( !atisTemplate.valid() ) {
     SG_LOG(SG_ATC, SG_WARN, "no matching atis template for station " << stationId  );
     return NO_ATIS; // no template for this station!?
   }
@@ -225,9 +225,9 @@ string ATISEncoder::processTokens( SGPropertyNode_ptr node )
 
 string ATISEncoder::processToken( SGPropertyNode_ptr token ) 
 {
-  HandlerMap::iterator it = handlerMap.find( token->getName());
+  HandlerMap::iterator it = handlerMap.find( token->getNameString());
   if( it == handlerMap.end() ) {
-    SG_LOG(SG_ATC, SG_WARN, "ATISEncoder: unknown token: " << token->getName() );
+    SG_LOG(SG_ATC, SG_WARN, "ATISEncoder: unknown token: " << token->getNameString() );
     return EMPTY;
   }
   handler_t h = it->second;
@@ -248,8 +248,6 @@ string ATISEncoder::processTokenToken( SGPropertyNode_ptr token )
   }
   handler_t h = it->second;
   return (this->*h)( token );
-
-  token->getStringValue();
 }
 
 string ATISEncoder::processIfToken( SGPropertyNode_ptr token )
@@ -326,7 +324,7 @@ string ATISEncoder::processIfToken( SGPropertyNode_ptr token )
 bool ATISEncoder::checkEmptyCondition( SGPropertyNode_ptr node, bool isEmpty ) 
 {
   SGPropertyNode_ptr n1 = node->getNode( "token", false );
-  if( false == n1.valid() ) {
+  if( !n1.valid() ) {
       SG_LOG(SG_ATC, SG_WARN, "missing <token> node for (not)-empty"  );
       return false;
   }
@@ -384,7 +382,7 @@ string ATISEncoder::getTime( SGPropertyNode_ptr )
 static inline FGRunwayRef findBestRunwayForWind( FGAirportRef airport, int windDeg, int windKt )
 {
   struct FGAirport::FindBestRunwayForHeadingParams p;
-  //TODO: ramp down the heading weight with wind speed
+  // TODO: ramp down the heading weight with wind speed
   p.ilsWeight = 4;
   return airport->findBestRunwayForHeading( windDeg, &p );
 }
@@ -394,7 +392,7 @@ string ATISEncoder::getApproachType( SGPropertyNode_ptr )
   FGRunwayRef runway = findBestRunwayForWind( airport, _atis->getWindDeg(), _atis->getWindSpeedKt() );
   if( runway.valid() ) {
     if( NULL != runway->ILS() ) return globals->get_locale()->getLocalizedString("ils", "atc", "ils" );
-    //TODO: any chance to find other approach types? localizer-dme, vor-dme, vor, ndb?
+    // TODO: any chance to find other approach types? localizer-dme, vor-dme, vor, ndb?
   }
 
   return globals->get_locale()->getLocalizedString("visual", "atc", "visual" );
@@ -414,15 +412,15 @@ string ATISEncoder::getLandingRunway( SGPropertyNode_ptr )
 
 string ATISEncoder::getTakeoffRunway( SGPropertyNode_ptr p )
 {
-  //TODO: if the airport has more than one runway, probably pick another one?
+  // TODO: if the airport has more than one runway, probably pick another one?
   return getLandingRunway( p );
 }
 
-string ATISEncoder::getTransitionLevel( SGPropertyNode_ptr )
+string ATISEncoder::getTransitionLevel(SGPropertyNode_ptr)
 {
-  double hPa = _atis->getQnh();
+    double hPa = _atis->getQnh();
 
-  /* Transition level is the flight level above which aircraft must use standard pressure and below
+    /* Transition level is the flight level above which aircraft must use standard pressure and below
    * which airport pressure settings must be used.
    * Following definitions are taken from German ATIS:
    *      QNH <=  977 hPa: TRL 80
@@ -430,26 +428,26 @@ string ATISEncoder::getTransitionLevel( SGPropertyNode_ptr )
    *      QNH >  1013 hPa: TRL 60
    * (maybe differs slightly for other countries...)
    */
-  int tl;
-  if (hPa <= 978) {
-    tl = 80;
-  } else if( hPa > 978 && hPa <= 1013 ) {
-    tl = 70;
-  } else if( hPa > 1013 && hPa <= 1046 ) {
-    tl = 60;
-  } else {
-    tl = 50;
-  }
+    int tl;
+    if (hPa <= 978) {
+        tl = 80;
+    } else if (hPa <= 1013) {
+        tl = 70;
+    } else if (hPa <= 1046) {
+        tl = 60;
+    } else {
+        tl = 50;
+    }
 
-  // add an offset to the transition level for high altitude airports (just guessing here,
-  // seems reasonable)
-  int e = int(airport->getElevation() / 1000.0);
-  if (e >= 3) {
-    // TL steps in 10(00)ft
-    tl += (e-2)*10;
-  }
+    // add an offset to the transition level for high altitude airports (just guessing here,
+    // seems reasonable)
+    int e = int(airport->getElevation() / 1000.0);
+    if (e >= 3) {
+        // TL steps in 10(00)ft
+        tl += (e - 2) * 10;
+    }
 
-  return getSpokenNumber(tl);
+    return getSpokenNumber(tl);
 }
 
 string ATISEncoder::getWindDirection( SGPropertyNode_ptr )
@@ -527,10 +525,12 @@ string ATISEncoder::getClouds( SGPropertyNode_ptr )
 
   ATISInformationProvider::CloudEntries cloudEntries = _atis->getClouds();
 
-  for( ATISInformationProvider::CloudEntries::iterator it = cloudEntries.begin(); it != cloudEntries.end(); it++ ) {
-    if( false == reply.empty() ) reply.SPACE;
+  for (ATISInformationProvider::CloudEntries::iterator it = cloudEntries.begin(); it != cloudEntries.end(); ++it) {
+    if (!reply.empty())
+      reply.SPACE;
     reply.append( it->second ).SPACE.append( getSpokenAltitude(it->first).SPACE.append( FEET ) );
   }
+
   return reply;
 }
 
@@ -540,10 +540,12 @@ string ATISEncoder::getCloudsBrief( SGPropertyNode_ptr )
 
   ATISInformationProvider::CloudEntries cloudEntries = _atis->getClouds();
 
-  for( ATISInformationProvider::CloudEntries::iterator it = cloudEntries.begin(); it != cloudEntries.end(); it++ ) {
-    if( false == reply.empty() ) reply.append(",").SPACE;
+  for (ATISInformationProvider::CloudEntries::iterator it = cloudEntries.begin(); it != cloudEntries.end(); ++it) {
+    if (!reply.empty())
+      reply.append(",").SPACE;
     reply.append( it->second ).SPACE.append( getSpokenAltitude(it->first) );
   }
+
   return reply;
 }
 
