@@ -1,25 +1,9 @@
-// JSBsim.cxx -- interface to the JSBsim flight model
-//
-// Written by Curtis Olson, started February 1999.
-//
-// Copyright (C) 1999  Curtis L. Olson  - curt@flightgear.org
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id: FlightGear.cxx,v 1.15 2014/01/28 09:42:20 ehofman Exp $
-
+/*
+ * SPDX-FileName: JSBsim.cxx
+ * SPDX-FileComment: interface to the JSBsim flight model
+ * SPDX-FileCopyrightText: Copyright (C) 1999  Curtis L. Olson  - curt@flightgear.org
+ * SPDX-License-Identifier: LGPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -39,6 +23,7 @@
 #include <simgear/bvh/BVHMaterial.hxx>
 
 #include <FDM/flight.hxx>
+#include <FDM/groundreactions.hxx>
 
 #include <Aircraft/controls.hxx>
 #include <Main/globals.hxx>
@@ -113,29 +98,12 @@ public:
     return agl;
   }
 
-  double GetTerrainGeoCentRadius(double t, const FGLocation& l) const override {
-    double contact[3], normal[3], vel[3], angularVel[3];
-    mInterface->get_agl_ft(t, l, SG_METER_TO_FEET*2, contact,
-                           normal, vel, angularVel);
-    return sqrt(contact[0]*contact[0]+contact[1]*contact[1]+contact[2]*contact[2]);
-  }
-
-  double GetSeaLevelRadius(const FGLocation& l) const override {
-    double seaLevelRadius, latGeoc;
-
-    sgGeodToGeoc(l.GetGeodLatitudeRad(), l.GetGeodAltitude(),
-                 &seaLevelRadius, &latGeoc);
-
-    return seaLevelRadius * SG_METER_TO_FEET;
-  }
-
-  void SetTerrainGeoCentRadius(double radius) override {}
 private:
   FGJSBsim* mInterface;
 };
 
 // FG uses a squared normalized magnitude for turbulence
-// this lookup table maps fg's severity levels 
+// this lookup table maps fg's severity levels
 // none(0), light(1/3), moderate(2/3) and severe(3/3)
 // to the POE table indexes 0, 3, 4 and 7
 class FGTurbulenceSeverityTable : public FGTable {
@@ -200,9 +168,6 @@ FGJSBsim::FGJSBsim( double dt )
     fdmex = new FGFDMExec( PropertyManager );
     fdmex->Hold();
 
-    // Register ground callback.
-    fdmex->SetGroundCallback( new FGFSGroundCallback(this) );
-
     Atmosphere      = fdmex->GetAtmosphere();
     Winds           = fdmex->GetWinds();
     FCS             = fdmex->GetFCS();
@@ -216,6 +181,9 @@ FGJSBsim::FGJSBsim( double dt )
     GroundReactions = fdmex->GetGroundReactions();
     Accelerations   = fdmex->GetAccelerations();
 
+    // Register ground callback.
+    Inertial->SetGroundCallback( new FGFSGroundCallback(this) );
+
     fgic=fdmex->GetIC();
     needTrim=true;
 
@@ -230,8 +198,8 @@ FGJSBsim::FGJSBsim( double dt )
 // deprecate sim-time-sec for simulation/sim-time-sec
 // remove alias with increased configuration file version number (2.1 or later)
     SGPropertyNode * node = fgGetNode("/fdm/jsbsim/simulation/sim-time-sec");
-    fgGetNode("/fdm/jsbsim/sim-time-sec", true)->alias( node );
-// end of sim-time-sec deprecation patch
+    fgGetNode("/fdm/jsbsim/sim-time-sec", true)->alias(node, false);
+    // end of sim-time-sec deprecation patch
 
     _ai_wake_enabled = fgGetNode("fdm/ai-wake/enabled", true);
 
@@ -352,6 +320,7 @@ FGJSBsim::FGJSBsim( double dt )
     release_hook = fgGetNode("/fdm/jsbsim/systems/hook/tailhook-release-cmd", true);
 
     altitude = fgGetNode("/position/altitude-ft");
+    precipitation = fgGetNode("/environment/rain-norm", true);
     temperature = fgGetNode("/environment/temperature-degc",true);
     pressure = fgGetNode("/environment/pressure-inhg",true);
     pressureSL = fgGetNode("/environment/pressure-sea-level-inhg",true);
@@ -383,14 +352,15 @@ FGJSBsim::FGJSBsim( double dt )
     crashed = false;
 
     mesh = new AircraftMesh(fgGetDouble("/fdm/jsbsim/metrics/bw-ft"),
-                            fgGetDouble("/fdm/jsbsim/metrics/cbarw-ft"));
+                            fgGetDouble("/fdm/jsbsim/metrics/cbarw-ft"),
+                            fdmex->GetModelName());
 
     // Trim once to initialize all output parameters
     FGTrim *fgtrim = new FGTrim(fdmex,tFull);
     fgtrim->DoTrim();
     delete fgtrim;
 
-    string directive_file = fgGetString("/sim/jsbsim/output-directive-file");
+    std::string directive_file = fgGetString("/sim/jsbsim/output-directive-file");
     if (!directive_file.empty())
       fdmex->SetOutputDirectives(directive_file);
 }
@@ -574,30 +544,6 @@ void FGJSBsim::update( double dt )
       update_external_forces(fdmex->GetSimTime() + i * fdmex->GetDeltaT());
     }
 
-    FGJSBBase::Message* msg;
-    while ((msg = fdmex->ProcessNextMessage()) != NULL) {
-//      msg = fdmex->ProcessNextMessage();
-      switch (msg->type) {
-      case FGJSBBase::Message::eText:
-        if (msg->text == "Crash Detected: Simulation FREEZE.")
-          crashed = true;
-        SG_LOG( SG_FLIGHT, SG_INFO, msg->messageId << ": " << msg->text );
-        break;
-      case FGJSBBase::Message::eBool:
-        SG_LOG( SG_FLIGHT, SG_INFO, msg->messageId << ": " << msg->text << " " << msg->bVal );
-        break;
-      case FGJSBBase::Message::eInteger:
-        SG_LOG( SG_FLIGHT, SG_INFO, msg->messageId << ": " << msg->text << " " << msg->iVal );
-        break;
-      case FGJSBBase::Message::eDouble:
-        SG_LOG( SG_FLIGHT, SG_INFO, msg->messageId << ": " << msg->text << " " << msg->dVal );
-        break;
-      default:
-        SG_LOG( SG_FLIGHT, SG_INFO, "Unrecognized message type." );
-        break;
-      }
-    }
-
     reset_wake_group();
 
     // translate JSBsim back to FG structure so that the
@@ -651,7 +597,7 @@ bool FGJSBsim::copy_to_JSBsim()
 
         if (ab_brake_engaged->getBoolValue()) {
             left_brake = ab_brake_left_pct->getDoubleValue();
-            right_brake = ab_brake_right_pct->getDoubleValue(); 
+            right_brake = ab_brake_right_pct->getDoubleValue();
         }
 
         FCS->SetLBrake(FMAX(left_brake, parking_brake));
@@ -1116,7 +1062,7 @@ void FGJSBsim::set_V_calibrated_kts(double vc)
     fgic->SetVcalibratedKtsIC(vc);
   else {
     double p=pressure->getDoubleValue();
-    double mach = FGJSBBase::MachFromVcalibrated(vc, p);
+    double mach = Auxiliary->MachFromVcalibrated(vc, p);
     double temp = 1.8*(temperature->getDoubleValue()+273.15);
     double soundSpeed = sqrt(1.4*1716.0*temp);
     FGColumnVector3 vUVW = Propagate->GetUVW();
@@ -1261,18 +1207,32 @@ void FGJSBsim::init_gear(void )
 {
     FGGroundReactions* gr=fdmex->GetGroundReactions();
     int Ngear=GroundReactions->GetNumGearUnits();
+    double max = 1.0;
     for (int i=0;i<Ngear;i++) {
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
+      // coordinates are in feet so convert
       node->setDoubleValue("xoffset-in", gear->GetBodyLocation()(1) * 12);
       node->setDoubleValue("yoffset-in", gear->GetBodyLocation()(2) * 12);
       node->setDoubleValue("zoffset-in", gear->GetBodyLocation()(3) * 12);
+
+      node->setDoubleValue("xoffset-m", gear->GetBodyLocation()(1) * SG_FEET_TO_METER);
+      node->setDoubleValue("yoffset-m", gear->GetBodyLocation()(2) * SG_FEET_TO_METER);
+      node->setDoubleValue("zoffset-m", gear->GetBodyLocation()(3) * SG_FEET_TO_METER);
+
       node->setBoolValue("wow", gear->GetWOW());
-      node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel()*0.3043);
+      node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel() * SG_FEET_TO_METER);
       node->setBoolValue("has-brake", gear->GetBrakeGroup() > 0);
       node->setDoubleValue("position-norm", gear->GetGearUnitPos());
 //    node->setDoubleValue("tire-pressure-norm", gear->GetTirePressure());
-      node->setDoubleValue("compression-norm", gear->GetCompLen());
+      max = node->getDoubleValue("max-compression-ft");
+      if (max < 0.00001) {
+        max = node->getDoubleValue("max-compression-m");
+        if (max < 0.00001) max = 1.0;
+        else max /= SG_FEET_TO_METER;
+      }
+      node->setDoubleValue("compression-norm", gear->GetCompLen() / max);
+      node->setDoubleValue("compression-m", gear->GetCompLen() * SG_FEET_TO_METER);
       node->setDoubleValue("compression-ft", gear->GetCompLen());
       if ( gear->GetSteerable() )
         node->setDoubleValue("steering-norm", gear->GetSteerNorm());
@@ -1283,14 +1243,22 @@ void FGJSBsim::update_gear(void)
 {
     FGGroundReactions* gr=fdmex->GetGroundReactions();
     int Ngear=GroundReactions->GetNumGearUnits();
+    double max = 1.0;
     for (int i=0;i<Ngear;i++) {
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
       node->getChild("wow", 0, true)->setBoolValue( gear->GetWOW());
-      node->getChild("rollspeed-ms", 0, true)->setDoubleValue(gear->GetWheelRollVel()*0.3043);
+      node->getChild("rollspeed-ms", 0, true)->setDoubleValue(gear->GetWheelRollVel() * SG_FEET_TO_METER);
       node->getChild("position-norm", 0, true)->setDoubleValue(gear->GetGearUnitPos());
 //    gear->SetTirePressure(node->getDoubleValue("tire-pressure-norm"));
-      node->setDoubleValue("compression-norm", gear->GetCompLen());
+      max = node->getDoubleValue("max-compression-ft");
+      if (max < 0.00001) {
+        max = node->getDoubleValue("max-compression-m");
+        if (max < 0.00001) max = 1.0;
+        else max /= SG_FEET_TO_METER;
+      }
+      node->setDoubleValue("compression-norm", gear->GetCompLen() / max);
+      node->setDoubleValue("compression-m", gear->GetCompLen() * SG_FEET_TO_METER);
       node->setDoubleValue("compression-ft", gear->GetCompLen());
       if ( gear->GetSteerable() )
         node->setDoubleValue("steering-norm", gear->GetSteerNorm());
@@ -1396,15 +1364,15 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
   if (terrain_active)
   {
     static bool material_valid = false;
-    if (material) {
-      GroundReactions->SetStaticFFactor((*material).get_friction_factor());
-      GroundReactions->SetRollingFFactor((*material).get_rolling_friction()/0.02);
-      // 1 Pascal = 0.00014503773800721815 lbs/in^2
-      double pressure = (*material).get_load_resistance(); // N/m^2 (or Pascal)
-      GroundReactions->SetMaximumForce(pressure*0.00014503773800721815);
 
-      GroundReactions->SetBumpiness((*material).get_bumpiness());
-      GroundReactions->SetSolid((*material).get_solid());
+    if (updateGroundReactions(material)) {
+      setPosition(pt);
+      setHeading(Propagate->GetEuler(FGJSBBase::ePsi));
+      GroundReactions->SetRollingFFactor(getRolingFrictionFactor());
+      GroundReactions->SetStaticFFactor(getStaticFrictionFactor());
+      GroundReactions->SetMaximumForce(getPressure()*0.00014503773800721815);
+      GroundReactions->SetBumpiness(getBumpiness());
+      GroundReactions->SetSolid(getSolid());
       GroundReactions->SetPosition(pt);
       material_valid = true;
     } else {
@@ -1417,7 +1385,8 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
 #else
   terrain->setBoolValue("valid", false);
 #endif
-  return dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt));
+  return dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt)) +
+         getGroundDisplacement();
 }
 
 inline static double sqr(double x)
@@ -1429,7 +1398,7 @@ static double angle_diff(double a, double b)
 {
     double diff = fabs(a - b);
     if (diff > 180) diff = 360 - diff;
-    
+
     return diff;
 }
 
@@ -1442,7 +1411,7 @@ static void check_hook_solution(const FGColumnVector3& ground_normal_body, doubl
 	cos_fis[*points] = cos_fi_guess;
 	fis[*points] = atan2(sin_fi_guess, cos_fi_guess) * SG_RADIANS_TO_DEGREES;
 	(*points)++;
-    } 
+    }
 }
 
 
@@ -1463,15 +1432,15 @@ void FGJSBsim::update_external_forces(double t_off)
     const FGMatrix33& Tl2b = Propagate->GetTl2b();
     const FGLocation& Location = Propagate->GetLocation();
     const FGMatrix33& Tec2l = Location.GetTec2l();
-        
+
     double hook_area[4][3];
-    
+
     FGColumnVector3 hook_root_body = MassBalance->StructuralToBody(hook_root_struct);
     FGColumnVector3 hook_root = Location.LocalToLocation(Tb2l *   hook_root_body);
     hook_area[1][0] = hook_root(1);
     hook_area[1][1] = hook_root(2);
     hook_area[1][2] = hook_root(3);
-    
+
     hook_length = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-length-ft", 6.75);
     double fi_min = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-min-deg", -18);
     double fi_max = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-max-deg", 30);
@@ -1481,7 +1450,7 @@ void FGJSBsim::update_external_forces(double t_off)
 
     FGColumnVector3 hook_tip_body = hook_root_body;
     hook_tip_body(1) -= hook_length * cos_fi;
-    hook_tip_body(3) += hook_length * sin_fi;    
+    hook_tip_body(3) += hook_length * sin_fi;
     if (!arrestor_wire_engaged_hook->getBoolValue()) {
       double contact[3];
       double ground_normal[3];
@@ -1508,22 +1477,22 @@ void FGJSBsim::update_external_forces(double t_off)
 		// and rearrange to get a quadratic with coeffs:
         	double a = sqr(hook_length) * (sqr(ground_normal_body(1)) + sqr(ground_normal_body(3)));
         	double b = 2 * E * ground_normal_body(3) * hook_length;
-        	double c = sqr(E) - sqr(ground_normal_body(1) * hook_length);	
+        	double c = sqr(E) - sqr(ground_normal_body(1) * hook_length);
 
         	double disc = sqr(b) - 4 * a * c;
         	if (disc >= 0) {
 		    double delta = sqrt(disc) / (2 * a);
-		
+
 		    // allow 4 solutions for safety, should never happen
 		    double sin_fis[4];
 		    double cos_fis[4];
 		    double fis[4];
 		    int points = 0;
-		
+
         	    double sin_fi_guess = -b / (2 * a) - delta;
 		    check_hook_solution(ground_normal_body, E, hook_length, sin_fi_guess, sin_fis, cos_fis, fis, &points);
 		    check_hook_solution(ground_normal_body, E, hook_length, sin_fi_guess + 2 * delta, sin_fis, cos_fis, fis, &points);
-		
+
 		    if (points == 2) {
 			double diff1 = angle_diff(fi, fis[0]);
 			double diff2 = angle_diff(fi, fis[1]);
@@ -1596,7 +1565,7 @@ void FGJSBsim::update_external_forces(double t_off)
                 arrestor_wire_engaged_hook->setBoolValue(true);
         }
     }
-    
+
     // save actual position as old position ...
     last_hook_tip[0] = hook_area[0][0];
     last_hook_tip[1] = hook_area[0][1];
@@ -1604,7 +1573,7 @@ void FGJSBsim::update_external_forces(double t_off)
     last_hook_root[0] = hook_area[1][0];
     last_hook_root[1] = hook_area[1][1];
     last_hook_root[2] = hook_area[1][2];
-    
+
     fgSetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-deg", fi);
 
     if (_ai_wake_enabled->getBoolValue()) {
