@@ -4,24 +4,24 @@
  Author:       Tony Peden
  Date started: 7/1/99
 
- ------------- Copyright (C) 1999  Anthony K. Peden (apeden@earthlink.net) -------------
+ --------- Copyright (C) 1999  Anthony K. Peden (apeden@earthlink.net) ---------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
  HISTORY
 --------------------------------------------------------------------------------
@@ -30,11 +30,11 @@
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
 
-The purpose of this class is to take a set of initial conditions and provide
-a kinematically consistent set of body axis velocity components, euler
-angles, and altitude.  This class does not attempt to trim the model i.e.
-the sim will most likely start in a very dynamic state (unless, of course,
-you have chosen your IC's wisely) even after setting it up with this class.
+The purpose of this class is to take a set of initial conditions and provide a
+kinematically consistent set of body axis velocity components, euler angles, and
+altitude. This class does not attempt to trim the model i.e. the sim will most
+likely start in a very dynamic state (unless, of course, you have chosen your
+IC's wisely) even after setting it up with this class.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 SENTRY
@@ -60,8 +60,8 @@ namespace JSBSim {
 class FGFDMExec;
 class FGMatrix33;
 class FGColumnVector3;
-class FGAtmosphere;
 class FGAircraft;
+class FGAuxiliary;
 class FGPropertyManager;
 class Element;
 
@@ -74,11 +74,11 @@ CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Initializes the simulation run.
-    Takes a set of initial conditions (IC) and provide a kinematically consistent set
-    of body axis velocity components, euler angles, and altitude.  This class
-    does not attempt to trim the model i.e. the sim will most likely start in a
-    very dynamic state (unless, of course, you have chosen your IC's wisely, or
-    started on the ground) even after setting it up with this class.
+    Takes a set of initial conditions (IC) and provide a kinematically
+    consistent set of body axis velocity components, euler angles, and altitude.
+    This class does not attempt to trim the model i.e. the sim will most likely
+    start in a very dynamic state (unless, of course, you have chosen your IC's
+    wisely, or started on the ground) even after setting it up with this class.
 
    <h3>Usage Notes</h3>
 
@@ -221,11 +221,11 @@ CLASS DOCUMENTATION
 CLASS DECLARATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGInitialCondition : public FGJSBBase
+class JSBSIM_API FGInitialCondition : public FGJSBBase
 {
 public:
   /// Constructor
-  FGInitialCondition(FGFDMExec *fdmex);
+  explicit FGInitialCondition(FGFDMExec *fdmex);
   /// Destructor
   ~FGInitialCondition();
 
@@ -293,10 +293,6 @@ public:
       @param agl Altitude above ground level in feet */
   void SetAltitudeAGLFtIC(double agl);
 
-  /** Sets the initial sea level radius from planet center
-      @param sl_rad sea level radius in feet */
-  void SetSeaLevelRadiusFtIC(double slr);
-
   /** Sets the initial terrain elevation.
       @param elev Initial terrain elevation in feet */
   void SetTerrainElevationFtIC(double elev);
@@ -381,7 +377,7 @@ public:
 
   /** Gets the initial altitude above sea level.
       @return Initial altitude in feet. */
-  double GetAltitudeASLFtIC(void) const { return position.GetAltitudeASL(); }
+  double GetAltitudeASLFtIC(void) const;
 
   /** Gets the initial altitude above ground level.
       @return Initial altitude AGL in feet */
@@ -391,6 +387,10 @@ public:
       @return Initial terrain elevation in feet */
   double GetTerrainElevationFtIC(void) const;
 
+  /** Gets the initial Earth position angle.
+      @return Initial Earth position angle in radians. */
+  double GetEarthPositionAngleIC(void) const { return epa; }
+
   /** Sets the initial ground speed.
       @param vg Initial ground speed in feet/second */
   void SetVgroundFpsIC(double vg);
@@ -487,27 +487,23 @@ public:
 
   /** Gets the initial wind velocity in the NED local frame
       @return Initial wind velocity in NED frame in feet/second */
-  const FGColumnVector3 GetWindNEDFpsIC(void) const {
-    const FGMatrix33& Tb2l = orientation.GetTInv();
-    FGColumnVector3 _vt_NED = Tb2l * Tw2b * FGColumnVector3(vt, 0., 0.);
-    return _vt_NED - vUVW_NED;
-  }
+  FGColumnVector3 GetWindNEDFpsIC(void) const;
 
   /** Gets the initial wind velocity in local frame.
       @return Initial wind velocity toward north in feet/second */
-  double GetWindNFpsIC(void) const { return GetNEDWindFpsIC(eX); }
+  double GetWindNFpsIC(void) const { return GetWindNEDFpsIC()(eX); }
 
   /** Gets the initial wind velocity in local frame.
       @return Initial wind velocity eastwards in feet/second */
-  double GetWindEFpsIC(void) const { return GetNEDWindFpsIC(eY); }
+  double GetWindEFpsIC(void) const { return GetWindNEDFpsIC()(eY); }
 
   /** Gets the initial wind velocity in local frame.
       @return Initial wind velocity downwards in feet/second */
-  double GetWindDFpsIC(void) const { return GetNEDWindFpsIC(eZ); }
+  double GetWindDFpsIC(void) const { return GetWindNEDFpsIC()(eZ); }
 
   /** Gets the initial total wind velocity in feet/sec.
       @return Initial wind velocity in feet/second */
-  double GetWindFpsIC(void)  const;
+  double GetWindFpsIC(void) const;
 
   /** Gets the initial wind direction.
       @return Initial wind direction in feet/second */
@@ -519,7 +515,7 @@ public:
   {
     const FGMatrix33& Tb2l = orientation.GetTInv();
     FGColumnVector3 _vt_NED = Tb2l * Tw2b * FGColumnVector3(vt, 0., 0.);
-    return _vt_NED(eW);
+    return -_vt_NED(eW);
   }
 
   /** Gets the initial body velocity
@@ -669,19 +665,22 @@ public:
 
   /** Loads the initial conditions.
       @param rstname The name of an initial conditions file
-      @param useStoredPath true if the stored path to the IC file should be used
+      @param useAircraftPath true if path is given relative to the aircraft path.
       @return true if successful */
-  bool Load(const SGPath& rstname, bool useStoredPath = true );
+  bool Load(const SGPath& rstname, bool useAircraftPath = true );
 
   /** Is an engine running ?
       @param index of the engine to be checked
       @return true if the engine is running. */
   bool IsEngineRunning(unsigned int n) const { return (enginesRunning & (1 << n)) != 0; }
-  
+
   /** Does initialization file call for trim ?
       @return Trim type, if any requested (version 1). */
   int TrimRequested(void) const { return trimRequested; }
 
+  /** Initialize the initial conditions to default values */
+  void InitializeIC(void);
+
   void bind(FGPropertyManager* pm);
 
 private:
@@ -694,8 +693,8 @@ private:
   double targetNlfIC;
 
   FGMatrix33 Tw2b, Tb2w;
-  double  alpha, beta;
-  double a, e2;
+  double alpha, beta;
+  double epa;
 
   speedset lastSpeedSet;
   altitudeset lastAltitudeSet;
@@ -704,18 +703,16 @@ private:
   int trimRequested;
 
   FGFDMExec *fdmex;
-  FGAtmosphere* Atmosphere;
   FGAircraft* Aircraft;
+  FGAuxiliary* Auxiliary;
 
   bool Load_v1(Element* document);
   bool Load_v2(Element* document);
 
-  void InitializeIC(void);
   void SetEulerAngleRadIC(int idx, double angle);
   void SetBodyVelFpsIC(int idx, double vel);
   void SetNEDVelFpsIC(int idx, double vel);
   double GetBodyWindFpsIC(int idx) const;
-  double GetNEDWindFpsIC(int idx) const;
   double GetBodyVelFpsIC(int idx) const;
   void calcAeroAngles(const FGColumnVector3& _vt_BODY);
   void calcThetaBeta(double alfa, const FGColumnVector3& _vt_NED);
