@@ -62,7 +62,6 @@ FGOutputType::FGOutputType(FGFDMExec* fdmex) :
   Aerodynamics = FDMExec->GetAerodynamics();
   Auxiliary = FDMExec->GetAuxiliary();
   Aircraft = FDMExec->GetAircraft();
-  Atmosphere = FDMExec->GetAtmosphere();
   Winds = FDMExec->GetWinds();
   Propulsion = FDMExec->GetPropulsion();
   MassBalance = FDMExec->GetMassBalance();
@@ -92,7 +91,7 @@ void FGOutputType::SetIdx(unsigned int idx)
 {
   string outputProp = CreateIndexedPropertyName("simulation/output", idx);
 
-  PropertyManager->Tie(outputProp + "/log_rate_hz", this, &FGOutputType::GetRateHz, &FGOutputType::SetRateHz, false);
+  PropertyManager->Tie(outputProp + "/log_rate_hz", this, &FGOutputType::GetRateHz, &FGOutputType::SetRateHz);
   PropertyManager->Tie(outputProp + "/enabled", &enabled);
   OutputIdx = idx;
 }
