@@ -150,7 +150,7 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
 
   // Make sure that the desired time is reached and executed.
   EndTime += 0.99*FDMExec->GetDeltaT();
-  
+
   // read aircraft and initialization files
 
   element = document->FindElement("use");
@@ -193,7 +193,7 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
   while (element) {
     if (!FDMExec->GetInput()->Load(element))
       return false;
- 
+
     element = document->FindNextElement("input");
   }
 
@@ -206,7 +206,7 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
   while (element) {
     if (!FDMExec->GetOutput()->Load(element, scriptDir))
       return false;
- 
+
     element = document->FindNextElement("output");
   }
 
@@ -295,7 +295,7 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
         }
         else
           newEvent->NotifyProperties.push_back(new FGPropertyValue(notifyPropertyName, PropertyManager));
-        
+
         string caption_attribute = notify_property_element->GetAttributeValue("caption");
         if (caption_attribute.empty()) {
           newEvent->DisplayString.push_back(notifyPropertyName);
@@ -496,7 +496,7 @@ bool FGScript::RunScript(void)
         } else  {
           cout << endl << underon
                << highint << thisEvent.Name << normint << underoff
-               << " (Event " << event_ctr << ")" 
+               << " (Event " << event_ctr << ")"
                << " executed at time: " << highint << currentTime << normint
                << endl;
         }
@@ -573,7 +573,7 @@ void FGScript::Debug(int from)
              << " = " << node->getDoubleValue()
              << endl;
       }
-      
+
       if (LocalProperties.empty()) cout << endl;
 
       for (unsigned i=0; i<Events.size(); i++) {
@@ -597,38 +597,38 @@ void FGScript::Debug(int from)
         for (unsigned j=0; j<Events[i].SetValue.size(); j++) {
           if (Events[i].SetValue[j] == 0.0 && Events[i].Functions[j] != 0L) {
             if (Events[i].SetParam[j] == 0) {
-              if (Events[i].SetParamName[j].size() == 0) {
-              cerr << fgred << highint << endl
-                   << "  An attempt has been made to access a non-existent property" << endl
-                   << "  in this event. Please check the property names used, spelling, etc."
-                   << reset << endl;
-              exit(-1);
+              if (Events[i].SetParamName[j].empty()) {
+                stringstream s;
+                s << "  An attempt has been made to access a non-existent property" << endl
+                  << "  in this event. Please check the property names used, spelling, etc.";
+                cerr << fgred << highint << endl << s.str() << reset << endl;
+                throw BaseException(s.str());
               } else {
                 cout << endl << "      set " << Events[i].SetParamName[j]
                      << " to function value (Late Bound)";
-            }
+              }
             } else {
-            cout << endl << "      set "
-                 << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
-                 << " to function value";
+              cout << endl << "      set "
+                   << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
+                   << " to function value";
             }
           } else {
             if (Events[i].SetParam[j] == 0) {
-              if (Events[i].SetParamName[j].size() == 0) {
-              cerr << fgred << highint << endl
-                   << "  An attempt has been made to access a non-existent property" << endl
-                   << "  in this event. Please check the property names used, spelling, etc."
-                   << reset << endl;
-              exit(-1);
+              if (Events[i].SetParamName[j].empty()) {
+                stringstream s;
+                s << "  An attempt has been made to access a non-existent property" << endl
+                  << "  in this event. Please check the property names used, spelling, etc.";
+                cerr << fgred << highint << endl << s.str() << reset << endl;
+                throw BaseException(s.str());
               } else {
                 cout << endl << "      set " << Events[i].SetParamName[j]
                      << " to function value (Late Bound)";
-            }
+              }
             } else {
-            cout << endl << "      set "
-                 << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
-                 << " to " << Events[i].SetValue[j];
-          }
+              cout << endl << "      set "
+                   << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
+                   << " to " << Events[i].SetValue[j];
+            }
           }
 
           switch (Events[i].Type[j]) {
@@ -664,7 +664,7 @@ void FGScript::Debug(int from)
 
         // Print notifications
         if (Events[i].Notify) {
-          if (Events[i].NotifyProperties.size() > 0) {
+          if (!Events[i].NotifyProperties.empty()) {
             if (Events[i].NotifyKML) {
               cout << "  Notifications (KML Format):" << endl << "    {"
                    << endl;
