@@ -28,6 +28,9 @@
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <iostream>
+#include <sstream>  // for assembling the error messages / what of exceptions.
+#include <stdexcept>  // using domain_error, invalid_argument, and length_error.
 #include "FGXMLElement.h"
 #include "FGJSBBase.h"
 
@@ -121,7 +124,7 @@ Element::Element(const string& nm)
     convert["N"]["LBS"] = 0.22482;
     convert["LBS"]["N"] = 1.0/convert["N"]["LBS"];
     // Velocity
-    convert["KTS"]["FT/SEC"] = 1.68781;
+    convert["KTS"]["FT/SEC"] = 1.6878098571;
     convert["FT/SEC"]["KTS"] = 1.0/convert["KTS"]["FT/SEC"];
     convert["M/S"]["FT/S"] = 3.2808399;
     convert["M/S"]["KTS"] = convert["M/S"]["FT/S"]/convert["KTS"]["FT/SEC"];
@@ -154,6 +157,7 @@ Element::Element(const string& nm)
     convert["PA"]["LBS/FT2"] = 1.0/convert["LBS/FT2"]["PA"];
     // Mass flow
     convert["KG/MIN"]["LBS/MIN"] = convert["KG"]["LBS"];
+    convert["KG/SEC"]["LBS/SEC"] = convert["KG"]["LBS"];
     convert ["N/SEC"]["LBS/SEC"] = 0.224808943;
     convert ["LBS/SEC"]["N/SEC"] = 1.0/convert ["N/SEC"]["LBS/SEC"];
     // Fuel Consumption
@@ -162,6 +166,9 @@ Element::Element(const string& nm)
     // Density
     convert["KG/L"]["LBS/GAL"] = 8.3454045;
     convert["LBS/GAL"]["KG/L"] = 1.0/convert["KG/L"]["LBS/GAL"];
+    // Gravitational
+    convert["FT3/SEC2"]["M3/SEC2"] = convert["FT3"]["M3"];
+    convert["M3/SEC2"]["FT3/SEC2"] = convert["M3"]["FT3"];
 
     // Length
     convert["M"]["M"] = 1.00;
@@ -235,6 +242,9 @@ Element::Element(const string& nm)
     // Density
     convert["KG/L"]["KG/L"] = 1.0;
     convert["LBS/GAL"]["LBS/GAL"] = 1.0;
+    // Gravitational
+    convert["FT3/SEC2"]["FT3/SEC2"] = 1.0;
+    convert["M3/SEC2"]["M3/SEC2"] = 1.0;
   }
 }
 
@@ -272,18 +282,20 @@ double Element::GetAttributeValueAsNumber(const string& attr)
   string attribute = GetAttributeValue(attr);
 
   if (attribute.empty()) {
-    cerr << ReadFrom() << "Expecting numeric attribute value, but got no data"
-         << endl;
-    exit(-1);
+    std::stringstream s;
+    s << ReadFrom() << "Expecting numeric attribute value, but got no data";
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
   else {
     double number=0;
     if (is_number(trim(attribute)))
-      number = atof(attribute.c_str());
+      number = atof_locale_c(attribute);
     else {
-      cerr << ReadFrom() << "Expecting numeric attribute value, but got: "
-           << attribute << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Expecting numeric attribute value, but got: " << attribute;
+      cerr << s.str() << endl;
+      throw BaseException(s.str());
     }
 
     return (number);
@@ -321,7 +333,7 @@ Element* Element::GetNextElement(void)
 
 string Element::GetDataLine(unsigned int i)
 {
-  if (data_lines.size() > 0) return data_lines[i];
+  if (!data_lines.empty()) return data_lines[i];
   else return string("");
 }
 
@@ -332,24 +344,31 @@ double Element::GetDataAsNumber(void)
   if (data_lines.size() == 1) {
     double number=0;
     if (is_number(trim(data_lines[0])))
-      number = atof(data_lines[0].c_str());
+      number = atof_locale_c(data_lines[0]);
     else {
-      cerr << ReadFrom() << "Expected numeric value, but got: " << data_lines[0]
-           << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Expected numeric value, but got: " << data_lines[0];
+      cerr << s.str() << endl;
+      throw BaseException(s.str());
     }
 
     return number;
-  } else if (data_lines.size() == 0) {
-    cerr << ReadFrom() << "Expected numeric value, but got no data" << endl;
-    exit(-1);
+  } else if (data_lines.empty()) {
+    std::stringstream s;
+    s << ReadFrom() << "Expected numeric value, but got no data";
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   } else {
     cerr << ReadFrom() << "Attempting to get single data value in element "
          << "<" << name << ">" << endl
          << " from multiple lines:" << endl;
     for(unsigned int i=0; i<data_lines.size(); ++i)
       cerr << data_lines[i] << endl;
-    exit(-1);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get single data value in element "
+      << "<" << name << ">"
+      << " from multiple lines (" << data_lines.size() << ").";
+    throw length_error(s.str());
   }
 }
 
@@ -416,9 +435,10 @@ double Element::FindElementValueAsNumber(const string& el)
     value = DisperseValue(element, value);
     return value;
   } else {
-    cerr << ReadFrom() << "Attempting to get non-existent element " << el
-         << endl;
-    exit(-1);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
 }
 
@@ -463,24 +483,28 @@ double Element::FindElementValueAsNumberConvertTo(const string& el, const string
   Element* element = FindElement(el);
 
   if (!element) {
-    cerr << ReadFrom() << "Attempting to get non-existent element " << el
-         << endl;
-    exit(-1);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
 
   string supplied_units = element->GetAttributeValue("unit");
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << element->ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" does not exist (typo?)." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << element->ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" cannot be converted to " << target_units
-           << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
@@ -528,21 +552,26 @@ double Element::FindElementValueAsNumberConvertFromTo( const string& el,
   Element* element = FindElement(el);
 
   if (!element) {
-    cerr << "Attempting to get non-existent element " << el << endl;
-    exit(-1);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << element->ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" does not exist (typo?)." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << element->ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" cannot be converted to " << target_units
-           << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
@@ -567,15 +596,18 @@ FGColumnVector3 Element::FindElementTripletConvertTo( const string& target_units
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" does not exist (typo?)." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << ReadFrom() << "Supplied unit: \""
-           << supplied_units << "\" cannot be converted to " << target_units
-           << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
@@ -635,23 +667,25 @@ double Element::DisperseValue(Element *e, double val, const std::string& supplie
     double disp = e->GetAttributeValueAsNumber("dispersion");
     if (!supplied_units.empty()) disp *= convert[supplied_units][target_units];
     string attType = e->GetAttributeValue("type");
+    RandomNumberGenerator generator;
+
     if (attType == "gaussian" || attType == "gaussiansigned") {
-      double grn = FGJSBBase::GaussianRandomNumber();
-    if (attType == "gaussian") {
-      value = val + disp*grn;
-      } else { // Assume gaussiansigned
-        value = (val + disp*grn)*(fabs(grn)/grn);
-      }
+      double grn = generator.GetNormalRandomNumber();
+      if (attType == "gaussian")
+        value = val + disp*grn;
+      else // Assume gaussiansigned
+        value = (val + disp*grn)*FGJSBBase::sign(grn);
     } else if (attType == "uniform" || attType == "uniformsigned") {
-      double urn = ((((double)rand()/RAND_MAX)-0.5)*2.0);
-      if (attType == "uniform") {
-      value = val + disp * urn;
-      } else { // Assume uniformsigned
-        value = (val + disp * urn)*(fabs(urn)/urn);
-      }
+      double urn = generator.GetUniformRandomNumber();
+      if (attType == "uniform")
+        value = val + disp * urn;
+      else // Assume uniformsigned
+        value = (val + disp * urn)*FGJSBBase::sign(urn);
     } else {
-      cerr << ReadFrom() << "Unknown dispersion type" << attType << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Unknown dispersion type" << attType;
+      cerr << s.str() << endl;
+      throw domain_error(s.str());
     }
 
   }
