@@ -67,9 +67,6 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
   FGPropertyManager* PropertyManager = exec->GetPropertyManager();
   MassBalance = exec->GetMassBalance();
 
-  gasCellJ = FGMatrix33();
-  gasCellM = FGColumnVector3();
-
   Buoyancy = MaxVolume = MaxOverpressure = Temperature = Pressure =
     Contents = Volume = dVolumeIdeal = 0.0;
   Xradius = Yradius = Zradius = Xwidth = Ywidth = Zwidth = 0.0;
@@ -89,8 +86,9 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
   if (element) {
     vXYZ = element->FindElementTripletConvertTo("IN");
   } else {
-    cerr << "Fatal Error: No location found for this gas cell." << endl;
-    exit(-1);
+    const string s("Fatal Error: No location found for this gas cell.");
+    cerr << el->ReadFrom() << endl << s << endl;
+    throw BaseException(s);
   }
   if ((el->FindElement("x_radius") || el->FindElement("x_width")) &&
       (el->FindElement("y_radius") || el->FindElement("y_width")) &&
@@ -129,7 +127,7 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
       MaxVolume = M_PI * Yradius * Zradius * Xwidth;
     } else {
       cerr << "Warning: Unsupported gas cell shape." << endl;
-      MaxVolume = 
+      MaxVolume =
         (4.0  * M_PI * Xradius * Yradius * Zradius / 3.0 +
          M_PI * Yradius * Zradius * Xwidth +
          M_PI * Xradius * Zradius * Ywidth +
@@ -140,8 +138,9 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
          Xwidth * Ywidth * Zwidth);
     }
   } else {
-    cerr << "Fatal Error: Gas cell shape must be given." << endl;
-    exit(-1);
+    const string s("Fatal Error: Gas cell shape must be given.");
+    cerr << el->ReadFrom() << endl << s << endl;
+    throw BaseException(s);
   }
   if (el->FindElement("max_overpressure")) {
     MaxOverpressure = el->FindElementValueAsNumberConvertTo("max_overpressure",
@@ -149,12 +148,12 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
   }
   if (el->FindElement("fullness")) {
     const double Fullness = el->FindElementValueAsNumber("fullness");
-    if (0 <= Fullness) { 
-      Volume = Fullness * MaxVolume; 
+    if (0 <= Fullness) {
+      Volume = Fullness * MaxVolume;
     } else {
       cerr << "Warning: Invalid initial gas cell fullness value." << endl;
     }
-  }  
+  }
   if (el->FindElement("valve_coefficient")) {
     ValveCoefficient =
       el->FindElementValueAsNumberConvertTo("valve_coefficient",
@@ -174,7 +173,7 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
   if (Volume != 0.0) {
     // Calculate initial gas content.
     Contents = Pressure * Volume / (R * Temperature);
-    
+
     // Clip to max allowed value.
     const double IdealPressure = Contents * R * Temperature / MaxVolume;
     if (IdealPressure > Pressure + MaxOverpressure) {
@@ -197,20 +196,20 @@ FGGasCell::FGGasCell(FGFDMExec* exec, Element* el, unsigned int num,
   base_property_name = CreateIndexedPropertyName("buoyant_forces/gas-cell", CellNum);
 
   property_name = base_property_name + "/max_volume-ft3";
-  PropertyManager->Tie( property_name.c_str(), &MaxVolume, false );
+  PropertyManager->Tie( property_name.c_str(), &MaxVolume);
   PropertyManager->GetNode()->SetWritable( property_name, false );
   property_name = base_property_name + "/temp-R";
-  PropertyManager->Tie( property_name.c_str(), &Temperature, false );
+  PropertyManager->Tie( property_name.c_str(), &Temperature);
   property_name = base_property_name + "/pressure-psf";
-  PropertyManager->Tie( property_name.c_str(), &Pressure, false );
+  PropertyManager->Tie( property_name.c_str(), &Pressure);
   property_name = base_property_name + "/volume-ft3";
-  PropertyManager->Tie( property_name.c_str(), &Volume, false );
+  PropertyManager->Tie( property_name.c_str(), &Volume);
   property_name = base_property_name + "/buoyancy-lbs";
-  PropertyManager->Tie( property_name.c_str(), &Buoyancy, false );
+  PropertyManager->Tie( property_name.c_str(), &Buoyancy);
   property_name = base_property_name + "/contents-mol";
-  PropertyManager->Tie( property_name.c_str(), &Contents, false );
+  PropertyManager->Tie( property_name.c_str(), &Contents);
   property_name = base_property_name + "/valve_open";
-  PropertyManager->Tie( property_name.c_str(), &ValveOpen, false );
+  PropertyManager->Tie( property_name.c_str(), &ValveOpen);
 
   Debug(0);
 
@@ -266,7 +265,7 @@ void FGGasCell::Calculate(double dt)
   const size_t no_ballonets = Ballonet.size();
 
   //-- Read ballonet state --
-  // NOTE: This model might need a more proper integration technique. 
+  // NOTE: This model might need a more proper integration technique.
   double BallonetsVolume = 0.0;
   double BallonetsHeatFlow = 0.0;
   for (i = 0; i < no_ballonets; i++) {
@@ -276,7 +275,7 @@ void FGGasCell::Calculate(double dt)
 
   //-- Gas temperature --
 
-  if (HeatTransferCoeff.size() > 0) {
+  if (!HeatTransferCoeff.empty()) {
     // The model is based on the ideal gas law.
     // However, it does look a bit fishy. Please verify.
     //   dT/dt = dU / (Cv n R)
@@ -285,7 +284,7 @@ void FGGasCell::Calculate(double dt)
       dU += HeatTransferCoeff[i]->GetValue();
     }
     // Don't include dt when accounting for adiabatic expansion/contraction.
-    // The rate of adiabatic cooling looks about right: ~5.4 Rankine/1000ft. 
+    // The rate of adiabatic cooling looks about right: ~5.4 Rankine/1000ft.
     if (Contents > 0) {
       Temperature +=
         (dU * dt - Pressure * dVolumeIdeal - BallonetsHeatFlow) /
@@ -327,7 +326,7 @@ void FGGasCell::Calculate(double dt)
     const double VolumeValved =
       ValveOpen * ValveCoefficient * DeltaPressure * dt;
     Contents =
-      max(0.0, Contents - Pressure * VolumeValved / (R * Temperature));
+      max(1E-8, Contents - Pressure * VolumeValved / (R * Temperature));
   }
 
   //-- Update ballonets. --
@@ -357,17 +356,17 @@ void FGGasCell::Calculate(double dt)
   //-- Current buoyancy --
   // The buoyancy is computed using the atmospheres local density.
   Buoyancy = Volume * AirDensity * g;
-  
+
   // Note: This is gross buoyancy. The weight of the gas itself and
   // any ballonets is not deducted here as the effects of the gas mass
   // is handled by FGMassBalance.
-  vFn.InitMatrix(0.0, 0.0, - Buoyancy);
+  vFn = {0.0, 0.0, - Buoyancy};
 
   // Compute the inertia of the gas cell.
   // Consider the gas cell as a shape of uniform density.
   // FIXME: If the cell isn't ellipsoid or cylindrical the inertia will
   //        be wrong.
-  gasCellJ = FGMatrix33();
+  gasCellJ.InitMatrix();
   const double mass = Contents * M_gas();
   double Ixx, Iyy, Izz;
   if ((Xradius != 0.0) && (Yradius != 0.0) && (Zradius != 0.0) &&
@@ -375,7 +374,7 @@ void FGGasCell::Calculate(double dt)
     // Ellipsoid volume.
     Ixx = (1.0 / 5.0) * mass * (Yradius*Yradius + Zradius*Zradius);
     Iyy = (1.0 / 5.0) * mass * (Xradius*Xradius + Zradius*Zradius);
-    Izz = (1.0 / 5.0) * mass * (Xradius*Xradius + Yradius*Yradius);     
+    Izz = (1.0 / 5.0) * mass * (Xradius*Xradius + Yradius*Yradius);
   } else if  ((Xradius == 0.0) && (Yradius != 0.0) && (Zradius != 0.0) &&
               (Xwidth  != 0.0) && (Ywidth  == 0.0) && (Zwidth  == 0.0)) {
     // Cylindrical volume (might not be valid with an elliptical cross-section).
@@ -410,7 +409,7 @@ void FGGasCell::Calculate(double dt)
     // Add the mass, moment and inertia of any ballonets.
     for (i = 0; i < no_ballonets; i++) {
       Mass += Ballonet[i]->GetMass();
-       
+
       // Add ballonet moments due to mass (in the structural frame).
       gasCellM(eX) +=
         Ballonet[i]->GetXYZ(eX) * Ballonet[i]->GetMass()*slugtolb;
@@ -418,7 +417,7 @@ void FGGasCell::Calculate(double dt)
         Ballonet[i]->GetXYZ(eY) * Ballonet[i]->GetMass()*slugtolb;
       gasCellM(eZ) +=
         Ballonet[i]->GetXYZ(eZ) * Ballonet[i]->GetMass()*slugtolb;
-      
+
       gasCellJ += Ballonet[i]->GetInertia();
     }
   }
@@ -454,9 +453,9 @@ void FGGasCell::Debug(int from)
       cout << "      Cell location (X, Y, Z) (in.): " << vXYZ(eX) << ", " <<
         vXYZ(eY) << ", " << vXYZ(eZ) << endl;
       cout << "      Maximum volume: " << MaxVolume << " ft3" << endl;
-      cout << "      Relief valve release pressure: " << MaxOverpressure << 
+      cout << "      Relief valve release pressure: " << MaxOverpressure <<
         " lbs/ft2" << endl;
-      cout << "      Manual valve coefficient: " << ValveCoefficient << 
+      cout << "      Manual valve coefficient: " << ValveCoefficient <<
         " ft4*sec/slug" << endl;
       cout << "      Initial temperature: " << Temperature << " Rankine" <<
         endl;
@@ -474,7 +473,7 @@ void FGGasCell::Debug(int from)
   }
   if (debug_lvl & 4 ) { // Run() method entry print for FGModel-derived objects
   }
-  if (debug_lvl & 8 ) { // Runtime state variables    
+  if (debug_lvl & 8 ) { // Runtime state variables
       cout << "      " << type << " cell holds " << Contents << " mol " << endl;
       cout << "      Temperature: " << Temperature << " Rankine" << endl;
       cout << "      Pressure: " << Pressure << " lbs/ft2" << endl;
@@ -507,8 +506,6 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
   FGPropertyManager* PropertyManager = exec->GetPropertyManager();
   MassBalance = exec->GetMassBalance();
 
-  ballonetJ = FGMatrix33();
-
   MaxVolume = MaxOverpressure = Temperature = Pressure =
     Contents = Volume = dVolumeIdeal = dU = 0.0;
   Xradius = Yradius = Zradius = Xwidth = Ywidth = Zwidth = 0.0;
@@ -522,8 +519,9 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
   if (element) {
     vXYZ = element->FindElementTripletConvertTo("IN");
   } else {
-    cerr << "Fatal Error: No location found for this ballonet." << endl;
-    exit(-1);
+    const string s("Fatal Error: No location found for this ballonet.");
+    cerr << el->ReadFrom() << endl << s << endl;
+    throw BaseException(s);
   }
   if ((el->FindElement("x_radius") || el->FindElement("x_width")) &&
       (el->FindElement("y_radius") || el->FindElement("y_width")) &&
@@ -562,7 +560,7 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
       MaxVolume = M_PI * Yradius * Zradius * Xwidth;
     } else {
       cerr << "Warning: Unsupported ballonet shape." << endl;
-      MaxVolume = 
+      MaxVolume =
         (4.0  * M_PI * Xradius * Yradius * Zradius / 3.0 +
          M_PI * Yradius * Zradius * Xwidth +
          M_PI * Xradius * Zradius * Ywidth +
@@ -573,8 +571,9 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
          Xwidth * Ywidth * Zwidth);
     }
   } else {
-    cerr << "Fatal Error: Ballonet shape must be given." << endl;
-    exit(-1);
+    const string s("Fatal Error: Ballonet shape must be given.");
+    cerr << el->ReadFrom() << endl << s << endl;
+    throw BaseException(s);
   }
   if (el->FindElement("max_overpressure")) {
     MaxOverpressure = el->FindElementValueAsNumberConvertTo("max_overpressure",
@@ -582,12 +581,12 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
   }
   if (el->FindElement("fullness")) {
     const double Fullness = el->FindElementValueAsNumber("fullness");
-    if (0 <= Fullness) { 
-      Volume = Fullness * MaxVolume; 
+    if (0 <= Fullness) {
+      Volume = Fullness * MaxVolume;
     } else {
       cerr << "Warning: Invalid initial ballonet fullness value." << endl;
     }
-  }  
+  }
   if (el->FindElement("valve_coefficient")) {
     ValveCoefficient =
       el->FindElementValueAsNumberConvertTo("valve_coefficient",
@@ -605,7 +604,7 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
   if (Volume != 0.0) {
     // Calculate initial air content.
     Contents = Pressure * Volume / (R * Temperature);
-    
+
     // Clip to max allowed value.
     const double IdealPressure = Contents * R * Temperature / MaxVolume;
     if (IdealPressure > Pressure + MaxOverpressure) {
@@ -627,23 +626,23 @@ FGBallonet::FGBallonet(FGFDMExec* exec, Element* el, unsigned int num,
   base_property_name = CreateIndexedPropertyName(base_property_name + "/ballonet", CellNum);
 
   property_name = base_property_name + "/max_volume-ft3";
-  PropertyManager->Tie( property_name, &MaxVolume, false );
+  PropertyManager->Tie( property_name, &MaxVolume);
   PropertyManager->GetNode()->SetWritable( property_name, false );
 
   property_name = base_property_name + "/temp-R";
-  PropertyManager->Tie( property_name, &Temperature, false );
+  PropertyManager->Tie( property_name, &Temperature);
 
   property_name = base_property_name + "/pressure-psf";
-  PropertyManager->Tie( property_name, &Pressure, false );
+  PropertyManager->Tie( property_name, &Pressure);
 
   property_name = base_property_name + "/volume-ft3";
-  PropertyManager->Tie( property_name, &Volume, false );
+  PropertyManager->Tie( property_name, &Volume);
 
   property_name = base_property_name + "/contents-mol";
-  PropertyManager->Tie( property_name, &Contents, false );
+  PropertyManager->Tie( property_name, &Contents);
 
   property_name = base_property_name + "/valve_open";
-  PropertyManager->Tie( property_name, &ValveOpen, false );
+  PropertyManager->Tie( property_name, &ValveOpen);
 
   Debug(0);
 
@@ -743,7 +742,7 @@ void FGBallonet::Calculate(double dt)
   // Consider the ballonet as a shape of uniform density.
   // FIXME: If the ballonet isn't ellipsoid or cylindrical the inertia will
   //        be wrong.
-  ballonetJ = FGMatrix33();
+  ballonetJ.InitMatrix();
   const double mass = Contents * M_air;
   double Ixx, Iyy, Izz;
   if ((Xradius != 0.0) && (Yradius != 0.0) && (Zradius != 0.0) &&
@@ -751,7 +750,7 @@ void FGBallonet::Calculate(double dt)
     // Ellipsoid volume.
     Ixx = (1.0 / 5.0) * mass * (Yradius*Yradius + Zradius*Zradius);
     Iyy = (1.0 / 5.0) * mass * (Xradius*Xradius + Zradius*Zradius);
-    Izz = (1.0 / 5.0) * mass * (Xradius*Xradius + Yradius*Yradius);     
+    Izz = (1.0 / 5.0) * mass * (Xradius*Xradius + Yradius*Yradius);
   } else if  ((Xradius == 0.0) && (Yradius != 0.0) && (Zradius != 0.0) &&
               (Xwidth  != 0.0) && (Ywidth  == 0.0) && (Zwidth  == 0.0)) {
     // Cylindrical volume (might not be valid with an elliptical cross-section).
@@ -803,9 +802,9 @@ void FGBallonet::Debug(int from)
       cout << "        Location (X, Y, Z) (in.): " << vXYZ(eX) << ", " <<
         vXYZ(eY) << ", " << vXYZ(eZ) << endl;
       cout << "        Maximum volume: " << MaxVolume << " ft3" << endl;
-      cout << "        Relief valve release pressure: " << MaxOverpressure << 
+      cout << "        Relief valve release pressure: " << MaxOverpressure <<
         " lbs/ft2" << endl;
-      cout << "        Relief valve coefficient: " << ValveCoefficient << 
+      cout << "        Relief valve coefficient: " << ValveCoefficient <<
         " ft4*sec/slug" << endl;
       cout << "        Initial temperature: " << Temperature << " Rankine" <<
         endl;
@@ -823,7 +822,7 @@ void FGBallonet::Debug(int from)
   }
   if (debug_lvl & 4 ) { // Run() method entry print for FGModel-derived objects
   }
-  if (debug_lvl & 8 ) { // Runtime state variables    
+  if (debug_lvl & 8 ) { // Runtime state variables
       cout << "        Ballonet holds " << Contents <<
         " mol air" << endl;
       cout << "        Temperature: " << Temperature << " Rankine" << endl;
