@@ -9,21 +9,21 @@
  ------------- Copyright (C) 2011  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -45,38 +45,30 @@ COMMENTS, REFERENCES,  and NOTES
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
 #include <iomanip>
-#include <cstdlib>
+
 #include "FGFDMExec.h"
 #include "FGStandardAtmosphere.h"
 
+using namespace std;
+
 namespace JSBSim {
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-// Effective radius of the earth at a specific latitude per ISA 1976 (converted to ft)
-// r0 = 6356766 m
-const double FGStandardAtmosphere::EarthRadius = 6356766.0/FGJSBBase::fttom;
-/** Sonntag constants based on ref [2]. They are valid for temperatures between
-    -45 degC (-49 degF) and 60 degC (140 degF) with a precision of +/-0.35 degC
-    (+/-0.63 degF) */
-const double FGStandardAtmosphere::a = 611.2/FGJSBBase::psftopa; // psf
-const double FGStandardAtmosphere::b = 17.62; // 1/degC
-const double FGStandardAtmosphere::c = 243.12; // degC
-const double FGStandardAtmosphere::Mwater = 18.016 * FGJSBBase::kgtoslug / 1000.0; // slug/mol
-
 FGStandardAtmosphere::FGStandardAtmosphere(FGFDMExec* fdmex)
-  : FGAtmosphere(fdmex), TemperatureBias(0.0), TemperatureDeltaGradient(0.0),
-    VaporPressure(0.0), SaturatedVaporPressure(0.0), StdAtmosTemperatureTable(9)
+  : FGAtmosphere(fdmex), StdSLpressure(StdDaySLpressure), TemperatureBias(0.0),
+    TemperatureDeltaGradient(0.0), VaporMassFraction(0.0),
+    SaturatedVaporPressure(StdDaySLpressure), StdAtmosTemperatureTable(9),
+    MaxVaporMassFraction(10)
 {
   Name = "FGStandardAtmosphere";
 
   // This is the U.S. Standard Atmosphere table for temperature in degrees
   // Rankine, based on geometric altitude. The table values are often given
-  // in literature relative to geopotential altitude. 
+  // in literature relative to geopotential altitude.
   //
   //                        GeoMet Alt    Temp      GeoPot Alt  GeoMet Alt
   //                           (ft)      (deg R)      (km)        (km)
@@ -89,7 +81,7 @@ FGStandardAtmosphere::FGStandardAtmosphere(FGFDMExec* fdmex)
   //                         << 168676.12 << 487.20  //   51.000      51.413
   //                         << 235570.77 << 386.40  //   71.000      71.802
   //                         << 282152.08 << 336.50  //   84.852      86.000
-  //                         << 298556.40 << 336.50; //               91.000 - First layer in high altitude regime 
+  //                         << 298556.40 << 336.50; //               91.000 - First layer in high altitude regime
 
   //                            GeoPot Alt    Temp       GeoPot Alt  GeoMet Alt
   //                               (ft)      (deg R)        (km)        (km)
@@ -102,16 +94,49 @@ FGStandardAtmosphere::FGStandardAtmosphere(FGFDMExec* fdmex)
                            << 167322.8346 << 487.17  //   51.000      51.413
                            << 232939.6325 << 386.37  //   71.000      71.802
                            << 278385.8268 << 336.5028  // 84.852      86.000
-                           << 298556.4304 << 336.5028; //             91.000 - First layer in high altitude regime 
+                           << 298556.4304 << 336.5028; //             91.000 - First layer in high altitude regime
+
+
+  // This is the maximum water vapor mass fraction in ppm (parts per million) of
+  // dry air measured in the atmosphere according to the ISA 1976 document.
+  // Values at altitude below 8 km are record high. All other values are 1%
+  // high.
+
+  //                      Geopot Alt    Water     Geopot Alt
+  //                         (ft)       (ppm)        (km)
+  //                      ----------    -----     ----------
+  MaxVaporMassFraction <<     0.0000 << 35000.  //  0.0000 - Record high
+                       <<  3280.8399 << 31000.  //  1.0000
+                       <<  6561.6798 << 28000.  //  2.0000
+                       << 13123.3596 << 22000.  //  4.0000
+                       << 19685.0394 <<  8900.  //  6.0000
+                       << 26246.7192 <<  4700.  //  8.0000 - Record high
+                       << 32808.3990 <<  1300.  // 10.0000 - 1% high
+                       << 39370.0787 <<   230.  // 12.0000
+                       << 45931.7585 <<    48.  // 14.0000
+                       << 52493.4383 <<    38.; // 16.0000 - 1% high
 
   unsigned int numRows = StdAtmosTemperatureTable.GetNumRows();
-  PressureBreakpoints.resize(numRows);
-  LapseRates.resize(numRows);
+
+  // Initialize the standard atmosphere lapse rates.
+  CalculateLapseRates();
+  StdLapseRates = LapseRates;
 
   // Assume the altitude to fade out the gradient at is at the highest
   // altitude in the table. Above that, other functions are used to
   // calculate temperature.
-  GradientFadeoutAltitude = StdAtmosTemperatureTable(StdAtmosTemperatureTable.GetNumRows(),0);
+  GradientFadeoutAltitude = StdAtmosTemperatureTable(numRows, 0);
+
+  // Initialize the standard atmosphere pressure break points.
+  PressureBreakpoints.resize(numRows);
+  CalculatePressureBreakpoints(StdSLpressure);
+  StdPressureBreakpoints = PressureBreakpoints;
+
+  StdSLtemperature = StdAtmosTemperatureTable(1, 1);
+  StdSLdensity     = StdSLpressure / (Rdry * StdSLtemperature);
+
+  CalculateStdDensityBreakpoints();
+  StdSLsoundspeed = sqrt(SHRatio*Rdry*StdSLtemperature);
 
   bind();
   Debug(0);
@@ -128,24 +153,25 @@ FGStandardAtmosphere::~FGStandardAtmosphere()
 
 bool FGStandardAtmosphere::InitModel(void)
 {
-  PressureBreakpoints[0] = StdSLpressure = SLpressure = Pressure = 2116.228; // psf
+
+  // Assume the altitude to fade out the gradient at is at the highest
+  // altitude in the table. Above that, other functions are used to
+  // calculate temperature.
+  GradientFadeoutAltitude = StdAtmosTemperatureTable(StdAtmosTemperatureTable.GetNumRows(), 0);
+
   TemperatureDeltaGradient = 0.0;
   TemperatureBias = 0.0;
-  CalculateLapseRates();
-  CalculatePressureBreakpoints();
+  LapseRates = StdLapseRates;
 
-  StdSLtemperature = SLtemperature = StdAtmosTemperatureTable(1, 1);
-  StdSLdensity     = SLdensity = StdSLpressure / (Reng * StdSLtemperature);
+  PressureBreakpoints = StdPressureBreakpoints;
 
-  StdPressureBreakpoints = PressureBreakpoints;
-  StdLapseRates = LapseRates;
-
-  CalculateStdDensityBreakpoints();
+  SLpressure    = StdSLpressure;
+  SLtemperature = StdSLtemperature;
+  SLdensity     = StdSLdensity;
+  SLsoundspeed  = StdSLsoundspeed;
 
   Calculate(0.0);
 
-  StdSLsoundspeed = SLsoundspeed = Soundspeed;
-
 //  PrintStandardAtmosphereTable();
 
   return true;
@@ -156,11 +182,7 @@ void FGStandardAtmosphere::Calculate(double altitude)
 {
   FGAtmosphere::Calculate(altitude);
   SaturatedVaporPressure = CalculateVaporPressure(Temperature);
-
-  // Make sure the partial pressure of water vapor never becomes higher than the
-  // saturated vapor pressure.
-  if (VaporPressure > SaturatedVaporPressure)
-    VaporPressure = SaturatedVaporPressure;
+  ValidateVaporMassFraction(altitude);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -193,22 +215,21 @@ double FGStandardAtmosphere::GetPressure(double altitude) const
   double Lmb = LapseRates[b];
 
   if (Lmb != 0.0) {
-    double Exp = g0*Mair / (Rstar*Lmb);
+    double Exp = g0 / (Rdry*Lmb);
     double factor = Tmb/(Tmb + Lmb*deltaH);
     return PressureBreakpoints[b]*pow(factor, Exp);
   } else
-    return PressureBreakpoints[b]*exp(-g0*Mair*deltaH/(Rstar*Tmb));
+    return PressureBreakpoints[b]*exp(-g0*deltaH/(Rdry*Tmb));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 void FGStandardAtmosphere::SetPressureSL(ePressure unit, double pressure)
 {
-  double press = ConvertToPSF(pressure, unit);
-
-  SLpressure = press;
+  double p = ConvertToPSF(pressure, unit);
+  SLpressure = ValidatePressure(p, "Sea Level pressure");
   CalculateSLDensity();
-  CalculatePressureBreakpoints();
+  CalculatePressureBreakpoints(SLpressure);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -289,11 +310,11 @@ double FGStandardAtmosphere::GetStdPressure(double altitude) const
   double Lmb = LapseRates[b];
 
   if (Lmb != 0.0) {
-    double Exp = g0*Mair / (Rstar*Lmb);
+    double Exp = g0 / (Rdry*Lmb);
     double factor = Tmb/(Tmb + Lmb*deltaH);
     return StdPressureBreakpoints[b]*pow(factor, Exp);
   } else
-    return StdPressureBreakpoints[b]*exp(-g0*Mair*deltaH/(Rstar*Tmb));
+    return StdPressureBreakpoints[b]*exp(-g0*deltaH/(Rdry*Tmb));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -301,18 +322,7 @@ double FGStandardAtmosphere::GetStdPressure(double altitude) const
 
 double FGStandardAtmosphere::GetStdDensity(double altitude) const
 {
-  return GetStdPressure(altitude)/(Reng * GetStdTemperature(altitude));
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// Get the density, taking into account the water vapor in the air.
-
-double FGStandardAtmosphere::GetDensity(double altitude) const
-{
-  double P = GetPressure(altitude);
-  double Pa = P - VaporPressure; // Partial pressure of air
-
-  return (Pa/Reng + Mwater*VaporPressure/Rstar)/GetTemperature(altitude);
+  return GetStdPressure(altitude)/(Rdry * GetStdTemperature(altitude));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -321,13 +331,13 @@ void FGStandardAtmosphere::SetTemperature(double t, double h, eTemperature unit)
 {
   double targetTemp = ConvertToRankine(t, unit);
   double GeoPotAlt = GeopotentialAltitude(h);
-
-  TemperatureBias = targetTemp - GetStdTemperature(h);
+  double bias = targetTemp - GetStdTemperature(h);
 
   if (GeoPotAlt <= GradientFadeoutAltitude)
-    TemperatureBias -= TemperatureDeltaGradient * (GradientFadeoutAltitude - GeoPotAlt);
+    bias -= TemperatureDeltaGradient * (GradientFadeoutAltitude - GeoPotAlt);
 
-  CalculatePressureBreakpoints();
+  SetTemperatureBias(eRankine, bias);
+  CalculatePressureBreakpoints(SLpressure);
 
   SLtemperature = GetTemperature(0.0);
   CalculateSLSoundSpeedAndDensity();
@@ -337,11 +347,29 @@ void FGStandardAtmosphere::SetTemperature(double t, double h, eTemperature unit)
 
 void FGStandardAtmosphere::SetTemperatureBias(eTemperature unit, double t)
 {
+  // Retrieve the minimum temperature in the standard atmosphere, may not be the
+  // last row in future if for example it's extended and maybe there is some
+  // temperature inversion layer etc. So run through and find the minimum.
+  const double minStdAtmosphereTemp = StdAtmosTemperatureTable.GetMinValue();
+
+  // Minimum known temperature in the universe currently
+  constexpr double minUniverseTemperature = KelvinToRankine(1.0);
+
   if (unit == eCelsius || unit == eKelvin)
     t *= 1.80; // If temp delta "t" is given in metric, scale up to English
 
   TemperatureBias = t;
-  CalculatePressureBreakpoints();
+  // Confirm the temperature bias isn't going to result in an atmosphere
+  // temperature lower than the  lowest known temperature in the universe
+  if (minStdAtmosphereTemp + TemperatureBias < minUniverseTemperature) {
+    double minBias = minUniverseTemperature - minStdAtmosphereTemp;
+    cerr << "The temperature bias " << TemperatureBias << " R is too low. "
+         << "It could result in temperatures below the absolute zero." << endl
+         << "Temperature bias is therefore capped to " << minBias << endl;
+    TemperatureBias = minBias;
+  }
+
+  CalculatePressureBreakpoints(SLpressure);
 
   SLtemperature = GetTemperature(0.0);
   CalculateSLSoundSpeedAndDensity();
@@ -375,12 +403,25 @@ void FGStandardAtmosphere::SetSLTemperatureGradedDelta(eTemperature unit, double
 
 void FGStandardAtmosphere::SetTemperatureGradedDelta(double deltemp, double h, eTemperature unit)
 {
+  // Retrieve the minimum temperature in the standard atmosphere, may not be the
+  // last row in future if for example it's extended and maybe there is some
+  // temperature inversion layer etc. So run through and find the minimum.
+  const double minStdAtmosphereTemp = StdAtmosTemperatureTable.GetMinValue();
+  const double minDeltaTemperature = minStdAtmosphereTemp - StdSLtemperature;
+
   if (unit == eCelsius || unit == eKelvin)
     deltemp *= 1.80; // If temp delta "t" is given in metric, scale up to English
 
+  if (deltemp <= minDeltaTemperature) {
+    cerr << "The temperature delta " << deltemp << " R is too low. "
+         << "It could result in temperatures below the absolute zero." << endl
+         << "Temperature delta is therefore capped to " << minDeltaTemperature << endl;
+    deltemp = minDeltaTemperature;
+  }
+
   TemperatureDeltaGradient = deltemp/(GradientFadeoutAltitude - GeopotentialAltitude(h));
   CalculateLapseRates();
-  CalculatePressureBreakpoints();
+  CalculatePressureBreakpoints(SLpressure);
 
   SLtemperature = GetTemperature(0.0);
   CalculateSLSoundSpeedAndDensity();
@@ -407,9 +448,9 @@ void FGStandardAtmosphere::SetTemperatureGradedDelta(double deltemp, double h, e
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 // This function calculates (or recalculates) the lapse rate over an altitude range
-// where the "bh" in this case refers to the index of the base height in the 
-// StdAtmosTemperatureTable table. This function should be called anytime the 
-// temperature table is altered, such as when a gradient is applied across the 
+// where the "bh" in this case refers to the index of the base height in the
+// StdAtmosTemperatureTable table. This function should be called anytime the
+// temperature table is altered, such as when a gradient is applied across the
 // temperature table for a range of altitudes.
 
 void FGStandardAtmosphere::CalculateLapseRates()
@@ -429,9 +470,9 @@ void FGStandardAtmosphere::CalculateLapseRates()
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGStandardAtmosphere::CalculatePressureBreakpoints()
+void FGStandardAtmosphere::CalculatePressureBreakpoints(double SLpress)
 {
-  PressureBreakpoints[0] = SLpressure;
+  PressureBreakpoints[0] = SLpress;
 
   for (unsigned int b=0; b<PressureBreakpoints.size()-1; b++) {
     double BaseTemp = StdAtmosTemperatureTable(b+1,1);
@@ -439,15 +480,15 @@ void FGStandardAtmosphere::CalculatePressureBreakpoints()
     double UpperAlt = StdAtmosTemperatureTable(b+2,0);
     double deltaH = UpperAlt - BaseAlt;
     double Tmb = BaseTemp
-                 + TemperatureBias 
+                 + TemperatureBias
                  + (GradientFadeoutAltitude - BaseAlt)*TemperatureDeltaGradient;
     if (LapseRates[b] != 0.00) {
       double Lmb = LapseRates[b];
-      double Exp = g0*Mair / (Rstar*Lmb);
+      double Exp = g0 / (Rdry*Lmb);
       double factor = Tmb/(Tmb + Lmb*deltaH);
       PressureBreakpoints[b+1] = PressureBreakpoints[b]*pow(factor, Exp);
     } else {
-      PressureBreakpoints[b+1] = PressureBreakpoints[b]*exp(-g0*Mair*deltaH/(Rstar*Tmb));
+      PressureBreakpoints[b+1] = PressureBreakpoints[b]*exp(-g0*deltaH/(Rdry*Tmb));
     }
   }
 }
@@ -458,7 +499,7 @@ void FGStandardAtmosphere::ResetSLTemperature()
 {
   TemperatureBias = TemperatureDeltaGradient = 0.0;
   CalculateLapseRates();
-  CalculatePressureBreakpoints();
+  CalculatePressureBreakpoints(SLpressure);
 
   SLtemperature = StdSLtemperature;
   CalculateSLSoundSpeedAndDensity();
@@ -470,7 +511,7 @@ void FGStandardAtmosphere::ResetSLPressure()
 {
   SLpressure  = StdSLpressure;
   CalculateSLDensity();
-  CalculatePressureBreakpoints();
+  CalculatePressureBreakpoints(StdSLpressure);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -479,7 +520,7 @@ void FGStandardAtmosphere::CalculateStdDensityBreakpoints()
 {
   StdDensityBreakpoints.clear();
   for (unsigned int i = 0; i < StdPressureBreakpoints.size(); i++)
-    StdDensityBreakpoints.push_back(StdPressureBreakpoints[i] / (Reng * StdAtmosTemperatureTable(i + 1, 1)));
+    StdDensityBreakpoints.push_back(StdPressureBreakpoints[i] / (Rdry * StdAtmosTemperatureTable(i + 1, 1)));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -503,10 +544,10 @@ double FGStandardAtmosphere::CalculateDensityAltitude(double density, double geo
 
   // https://en.wikipedia.org/wiki/Barometric_formula for density solved for H
   if (Lmb != 0.0) {
-    double Exp = -1.0 / (1.0 + (g0*Mair)/(Rstar*Lmb));
+    double Exp = -1.0 / (1.0 + g0/(Rdry*Lmb));
     density_altitude = Hb + (Tmb / Lmb) * (pow(density / pb, Exp) - 1);
   } else {
-    double Factor = -(Rstar*Tmb) / (g0*Mair);
+    double Factor = -Rdry*Tmb / g0;
     density_altitude = Hb + Factor * log(density / pb);
   }
 
@@ -534,11 +575,11 @@ double FGStandardAtmosphere::CalculatePressureAltitude(double pressure, double g
 
   if (Lmb != 0.00) {
     // Equation 33(a) from ISA document solved for H
-    double Exp = -(Rstar*Lmb) / (g0*Mair);
+    double Exp = -Rdry*Lmb / g0;
     pressure_altitude = Hb + (Tmb / Lmb) * (pow(pressure / Pb, Exp) - 1);
   } else {
     // Equation 33(b) from ISA document solved for H
-    double Factor = -(Rstar*Tmb) / (g0*Mair);
+    double Factor = -Rdry*Tmb / g0;
     pressure_altitude = Hb + Factor * log(pressure / Pb);
   }
 
@@ -549,15 +590,52 @@ double FGStandardAtmosphere::CalculatePressureAltitude(double pressure, double g
 
 double FGStandardAtmosphere::CalculateVaporPressure(double temperature)
 {
-  double temperature_degC = temperature/1.8-273.15;
+  double temperature_degC = RankineToCelsius(temperature);
   return a*exp(b*temperature_degC/(c+temperature_degC));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+void FGStandardAtmosphere::ValidateVaporMassFraction(double h)
+{
+  if (SaturatedVaporPressure < Pressure) {
+    double VaporPressure = Pressure*VaporMassFraction / (VaporMassFraction+Rdry/Rwater);
+    if (VaporPressure > SaturatedVaporPressure)
+      VaporMassFraction = Rdry * SaturatedVaporPressure / (Rwater * (Pressure - SaturatedVaporPressure));
+  }
+
+  double GeoPotAlt = GeopotentialAltitude(h);
+  double maxFraction = 1E-6*MaxVaporMassFraction.GetValue(GeoPotAlt);
+
+  if ((VaporMassFraction > maxFraction) || (VaporMassFraction < 0.0))
+    VaporMassFraction = maxFraction;
+
+  // Update the gas constant factor
+  Reng = (VaporMassFraction*Rwater + Rdry)/(1.0 + VaporMassFraction);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGStandardAtmosphere::SetDewPoint(eTemperature unit, double dewpoint)
 {
-  VaporPressure = CalculateVaporPressure(ConvertToRankine(dewpoint, unit));
+  double dewPoint_R = ConvertToRankine(dewpoint, unit);
+  constexpr double minDewPoint = CelsiusToRankine(-c) + 1.0;
+
+  if (dewPoint_R <= minDewPoint) {
+    cerr << "The dew point temperature " << dewPoint_R << " is lower than "
+         << minDewPoint << " R." << endl
+         << "Dew point is therefore capped to " << minDewPoint << endl;
+    dewPoint_R = minDewPoint;
+  }
+
+  double VaporPressure = CalculateVaporPressure(dewPoint_R);
+  SetVaporPressure(ePSF, VaporPressure);
+
+  // double finalizedDewPoint = GetDewPoint(eRankine);
+  // if (finalizedDewPoint < dewPoint_R) {
+  //   cerr << "Dew point temperature has been capped to " << finalizedDewPoint
+  //        << endl;
+  // }
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -565,6 +643,7 @@ void FGStandardAtmosphere::SetDewPoint(eTemperature unit, double dewpoint)
 double FGStandardAtmosphere::GetDewPoint(eTemperature to) const
 {
   double dewpoint_degC;
+  double VaporPressure = Pressure*VaporMassFraction / (VaporMassFraction+Rdry/Rwater);
 
   if (VaporPressure <= 0.0)
     dewpoint_degC = -c;
@@ -573,20 +652,34 @@ double FGStandardAtmosphere::GetDewPoint(eTemperature to) const
     dewpoint_degC = c*x / (b - x);
   }
 
-  return ConvertFromRankine(1.8*(dewpoint_degC + 273.15), to);
+  return ConvertFromRankine(CelsiusToRankine(dewpoint_degC), to);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 void FGStandardAtmosphere::SetVaporPressure(ePressure unit, double Pa)
 {
-  VaporPressure = ConvertToPSF(Pa, unit);
+  double altitude = CalculatePressureAltitude(Pressure, 0.0);
+  double VaporPressure = ConvertToPSF(Pa, unit);
+  if (VaporPressure < 0.0) {
+    cerr << "The vapor pressure cannot be negative." << endl
+         << "Vapor pressure is set to 0.0" << endl;
+    VaporPressure = 0.0;
+  } else if (VaporPressure >= Pressure) {
+    cerr << "The vapor pressure " << VaporPressure
+         << " PSF is higher than the ambient pressure." << endl
+         << "Vapor pressure is therefore capped to " << Pressure-1.0 << endl;
+    VaporPressure = Pressure - 1.0;
+  }
+  VaporMassFraction = Rdry * VaporPressure / (Rwater * (Pressure - VaporPressure));
+  ValidateVaporMassFraction(altitude);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 double FGStandardAtmosphere::GetVaporPressure(ePressure to) const
 {
+  double VaporPressure = Pressure*VaporMassFraction / (VaporMassFraction+Rdry/Rwater);
   return ConvertFromPSF(VaporPressure, to);
 }
 
@@ -601,6 +694,7 @@ double FGStandardAtmosphere::GetSaturatedVaporPressure(ePressure to) const
 
 double FGStandardAtmosphere::GetRelativeHumidity(void) const
 {
+  double VaporPressure = Pressure*VaporMassFraction / (VaporMassFraction+Rdry/Rwater);
   return 100.0*VaporPressure/SaturatedVaporPressure;
 }
 
@@ -608,7 +702,39 @@ double FGStandardAtmosphere::GetRelativeHumidity(void) const
 
 void FGStandardAtmosphere::SetRelativeHumidity(double RH)
 {
-  VaporPressure = 0.01*RH*SaturatedVaporPressure;
+  if (RH < 0.0) {
+    cerr << "The relative humidity cannot be negative." << endl
+         << "Relative humidity is set to 0%" << endl;
+    RH = 0.0;
+  } else if (RH > 100.0) {
+    cerr << "The relative humidity cannot be higher than 100%." << endl
+         << "Relative humidity is set to 100%" << endl;
+    RH = 100.0;
+  }
+
+  double VaporPressure = 0.01*RH*SaturatedVaporPressure;
+  SetVaporPressure(ePSF, VaporPressure);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+double FGStandardAtmosphere::GetVaporMassFractionPPM(void) const
+{
+  return VaporMassFraction*1E6;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGStandardAtmosphere::SetVaporMassFractionPPM(double frac)
+{
+  double altitude = CalculatePressureAltitude(Pressure, 0.0);
+  VaporMassFraction = frac*1E-6;
+  ValidateVaporMassFraction(altitude);
+
+  if (fabs(VaporMassFraction*1E6-frac)>1E-2) {
+    cerr << "The vapor mass fraction " << frac << " has been capped to "
+         << VaporMassFraction*1E6 << "PPM." << endl;
+  }
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -637,6 +763,9 @@ void FGStandardAtmosphere::bind(void)
   PropertyManager->Tie("atmosphere/RH", this,
                        &FGStandardAtmosphere::GetRelativeHumidity,
                        &FGStandardAtmosphere::SetRelativeHumidity);
+  PropertyManager->Tie("atmosphere/vapor-fraction-ppm", this,
+                       &FGStandardAtmosphere::GetVaporMassFractionPPM,
+                       &FGStandardAtmosphere::SetVaporMassFractionPPM);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -676,7 +805,7 @@ void FGStandardAtmosphere::Debug(int from)
   }
   if (debug_lvl & 16) { // Sanity checking
   }
-  if (debug_lvl & 128) { // 
+  if (debug_lvl & 128) { //
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
