@@ -113,12 +113,12 @@ protected:
   unsigned int delay;
   int index;
   double dt;
-  bool IsOutput;
   bool clip, cyclic_clip;
 
   void Delay(void);
   void Clip(void);
-  virtual void bind();
+  void CheckInputNodes(size_t MinNodes, size_t MaxNodes, Element* el);
+  virtual void bind(Element* el);
   virtual void Debug(int from);
 };
 
