@@ -20,7 +20,6 @@
 
 #include "AirportDiagram.hxx"
 
-#include <limits>
 
 #include <simgear/sg_inlines.h>
 
@@ -45,10 +44,10 @@ static double distanceToLineSegment(const QVector2D& p, const QVector2D& a,
 {
     QVector2D ab(b - a);
     QVector2D ac(p - a);
-    
+
     // Squared length, to avoid a sqrt
     const qreal len2 = ab.lengthSquared();
-    
+
     // Line null, the projection can't exist, we return the first point
     if (qIsNull(len2)) {
         if (outT) {
@@ -56,10 +55,10 @@ static double distanceToLineSegment(const QVector2D& p, const QVector2D& a,
         }
         return (p - a).length();
     }
-    
+
     // Parametric value of the projection on the line
     const qreal t = (ac.x() * ab.x() + ac.y() * ab.y()) / len2;
-    
+
     if (t < 0.0) {
         // Point is before the first point
         if (outT) {
@@ -76,12 +75,10 @@ static double distanceToLineSegment(const QVector2D& p, const QVector2D& a,
         if (outT) {
             *outT = t;
         }
-        
+
         const QVector2D proj = a + t * ab;
         return (proj - p).length();
     }
-    
-    return 0.0;
 }
 
 static double unitLengthAfterMapping(const QTransform& t)
@@ -117,15 +114,9 @@ AirportDiagram::AirportDiagram(QQuickItem* pr) :
     QFont f;
     f.setPixelSize(24.0);
     f.setBold(true);
-    QFontMetricsF metrics(f);
-    qreal xOffset = metrics.width("H") * 0.5;
-#if QT_VERSION >= 0x050800
+    QFontMetrics metrics(f);
+    qreal xOffset = metrics.horizontalAdvance("H") * 0.5;
     qreal yOffset = metrics.capHeight() * 0.5;
-#else
-    // capHeight is not avaialble in 5.7 and lower, compute
-    // it using tightBoundingRect
-    qreal yOffset = -0.5 * metrics.tightBoundingRect("H").y();
-#endif
     m_helipadIconPath.addText(-xOffset, yOffset, f, "H");
 }
 
@@ -359,13 +350,13 @@ void AirportDiagram::paintContents(QPainter* p)
         if ((r.runway == runwaySelection) || (r.runway->reciprocalRunway() == runwaySelection)) {
             color = Qt::yellow;
         }
-        
+
         p->setTransform(t);
 
         QPen pen(color);
         pen.setWidth(r.widthM);
         p->setPen(pen);
-        
+
         p->drawLine(r.p1, r.p2);
 
     // draw idents
@@ -375,7 +366,7 @@ void AirportDiagram::paintContents(QPainter* p)
         p->rotate(r.runway->headingDeg());
         // invert scaling factor so we can use screen pixel sizes here
         p->scale(1.0 / m_scale, 1.0/ m_scale);
-        
+
         p->setPen((r.runway == runwaySelection) ? Qt::yellow : Qt::magenta);
         p->drawText(QRect(-100, 5, 200, 200), ident, Qt::AlignHCenter | Qt::AlignTop);
 
@@ -599,11 +590,6 @@ void AirportDiagram::mouseReleaseEvent(QMouseEvent* me)
             return;
         }
     }
-
-
-#if 0
-    img.save("/Users/jmt/Desktop/img.png");
-#endif
 }
 
 QPainterPath AirportDiagram::pathForRunway(const RunwayData& r, const QTransform& t,
@@ -668,8 +654,10 @@ void AirportDiagram::buildTaxiways()
 void AirportDiagram::buildPavements()
 {
     m_pavements.clear();
-    for (unsigned int pIndex=0; pIndex < m_airport->numPavements(); ++pIndex) {
-        FGPavementRef pave = m_airport->getPavementByIndex(pIndex);
+    auto pavementlist = m_airport->getPavements();
+    for (auto pvtiter = pavementlist.begin(); pvtiter != pavementlist.end(); ++pvtiter)
+    {
+        FGPavementRef pave = *pvtiter;
         if (pave->getNodeList().empty()) {
             continue;
         }
