@@ -1,3 +1,10 @@
+
+// SPDX-FileCopyrightText: 2018 James Turner
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+#include "config.h"
+#include "simgear/package/Root.hxx"
+
 #include "LauncherController.hxx"
 
 // Qt headers
@@ -13,6 +20,10 @@
 #include <QQuickWindow>
 #include <QSettings>
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 1, 0))
+#include <QNetworkInformation>
+#endif
+
 // simgear headers
 #include <simgear/package/Install.hxx>
 #include <simgear/environment/metar.hxx>
@@ -25,16 +36,18 @@
 #include <Main/options.hxx>
 #include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
-#include "version.h"
 #include <Main/sentryIntegration.hxx>
 
-#include "AircraftModel.hxx"
-#include "AircraftSearchFilterModel.hxx"
+#include "AircraftItemModel.hxx"
+#include "AircraftProxyModel.hxx"
 #include "AirportDiagram.hxx"
 #include "CarrierDiagram.hxx"
 #include "CarriersLocationModel.hxx"
 #include "DefaultAircraftLocator.hxx"
 #include "FlightPlanController.hxx"
+#include "GettingStartedScope.hxx"
+#include "GettingStartedTip.hxx"
+#include "GettingStartedTipsController.hxx"
 #include "HoverArea.hxx"
 #include "LaunchConfig.hxx"
 #include "LauncherArgumentTokenizer.hxx"
@@ -50,21 +63,22 @@
 #include "QmlAircraftInfo.hxx"
 #include "QmlPositioned.hxx"
 #include "QmlRadioButtonHelper.hxx"
+#include "QmlStringListModel.hxx"
 #include "QtLauncher.hxx"
 #include "RecentAircraftModel.hxx"
 #include "RecentLocationsModel.hxx"
 #include "RouteDiagram.hxx"
+#include "SettingsWrapper.hxx"
 #include "SetupRootDialog.hxx"
 #include "StackController.hxx"
 #include "ThumbnailImageItem.hxx"
+#include "TipBackgroundBox.hxx"
 #include "UnitsModel.hxx"
 #include "UpdateChecker.hxx"
-#include "GettingStartedTipsController.hxx"
-#include "GettingStartedTip.hxx"
-#include "TipBackgroundBox.hxx"
-#include "GettingStartedScope.hxx"
+
 
 using namespace simgear::pkg;
+using namespace std::chrono_literals;
 
 LauncherController::LauncherController(QObject *parent, QWindow* window) :
     QObject(parent),
@@ -75,6 +89,9 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
     m_locationHistory = new RecentLocationsModel(this);
     m_selectedAircraftInfo = new QmlAircraftInfo(this);
 
+    // ensure uninstall disables the fly button
+    connect(m_selectedAircraftInfo, &QmlAircraftInfo::downloadChanged, this, &LauncherController::updateSelectedAircraft);
+
     m_config = new LaunchConfig(this);
     connect(m_config, &LaunchConfig::collect, this, &LauncherController::collectAircraftArgs);
     connect(m_config, &LaunchConfig::save, this, &LauncherController::saveAircraft);
@@ -86,8 +103,6 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
     connect(m_location, &LocationController::descriptionChanged,
             this, &LauncherController::summaryChanged);
 
-    initQML();
-
     m_aircraftModel = new AircraftItemModel(this);
     m_installedAircraftModel = new AircraftProxyModel(this, m_aircraftModel);
     m_installedAircraftModel->setInstalledFilterEnabled(true);
@@ -115,7 +130,7 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
             &LocalAircraftCache::scanCompleted,
             this, &LauncherController::updateSelectedAircraft);
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     m_aircraftModel->setPackageRoot(globals->packageRoot());
 
     m_aircraftGridMode = settings.value("aircraftGridMode").toBool();
@@ -126,7 +141,7 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
        {globals->get_subsystem_mgr()->update(0.0);});
     m_subsystemIdleTimer->start();
 
-    QRect winRect= settings.value("window-geometry").toRect();
+    QRect winRect = settings.value("window-geometry").toRect();
 
     if (winRect.isValid()) {
         m_window->setGeometry(winRect);
@@ -137,7 +152,14 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
 
     if (settings.contains("window-state")) {
         const auto ws = static_cast<Qt::WindowState>(settings.value("window-state").toInt());
-        m_window->setWindowState(ws);
+        const auto options = flightgear::Options::sharedInstance();
+        if (ws == Qt::WindowState::WindowFullScreen && options->isBoolOptionDisable("fullscreen")) {
+            // Last time we used fullscreen, but now we explicitly turn off fullscreen
+            m_window->setWindowState(Qt::WindowState::WindowNoState);
+        }
+        else {
+            m_window->setWindowState(ws);
+        }
     }
 
     // count launches; we use this to trigger first-run and periodic notices
@@ -154,10 +176,31 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
         settings.setValue(versionedCountKey, m_versionLaunchCount + 1);
     }
 
-    QTimer::singleShot(2000, this, &LauncherController::checkForOldDownloadDir);
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 1, 0))
+    if (QNetworkInformation::instance()) {
+        connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged,
+                this, &LauncherController::onReachabilityChanged);
+        onReachabilityChanged();
+    } else {
+        qWarning() << "No network information available";
+        delayedPackageRefresh();
+    }
+#else
+    delayedPackageRefresh();
+#endif
+
+    QTimer::singleShot(2s, this, &LauncherController::checkForOldDownloadDir);
 }
 
-void LauncherController::initQML()
+void LauncherController::delayedPackageRefresh()
+{
+QTimer::singleShot(2s, this, []() {
+        auto pkgRoot = globals->packageRoot();
+        pkgRoot->refresh(true);
+    });
+}
+
+void LauncherController::initQML(int& styleTypeId)
 {
     qmlRegisterUncreatableType<LauncherController>("FlightGear.Launcher", 1, 0, "LauncherController", "no");
     qmlRegisterUncreatableType<LocationController>("FlightGear.Launcher", 1, 0, "LocationController", "no");
@@ -200,8 +243,11 @@ void LauncherController::initQML()
     qmlRegisterType<StackController>("FlightGear", 1, 0, "StackController");
 
     qmlRegisterType<ModelDataExtractor>("FlightGear", 1, 0, "ModelDataExtractor");
+    qmlRegisterType<QmlStringListModel>("FlightGear", 1, 0, "StringListModel");
+
 
     qmlRegisterSingletonType(QUrl("qrc:/qml/OverlayShared.qml"), "FlightGear", 1, 0, "OverlayShared");
+    styleTypeId = qmlRegisterSingletonType(QUrl("qrc:/qml/Style.qml"), "FlightGear", 1, 0, "Style");
 
     qmlRegisterType<GettingStartedScope>("FlightGear", 1, 0, "GettingStartedScope");
     qmlRegisterType<GettingStartedTipsController>("FlightGear", 1, 0, "GettingStartedController");
@@ -274,11 +320,11 @@ void LauncherController::initialRestoreSettings()
 
 void LauncherController::saveSettings()
 {
-    QSettings settings;
-    settings.setValue("window-geometry", m_window->geometry());
-    if (m_window->windowState() != Qt::WindowNoState) {
-        settings.setValue("window-state", m_window->windowState());
+    auto settings = flightgear::getQSettings();
+    if (m_window->windowState() != Qt::WindowMaximized) {
+        settings.setValue("window-geometry", m_window->geometry());
     }
+    settings.setValue("window-state", m_window->windowState());
 
     m_config->saveConfigToINI();
     m_aircraftHistory->saveToSettings();
@@ -297,17 +343,12 @@ void LauncherController::collectAircraftArgs()
             m_config->setArg("aircraft-dir", setFileInfo.dir().absolutePath());
             QString setFile = setFileInfo.fileName();
             Q_ASSERT(setFile.endsWith("-set.xml"));
-            setFile.truncate(setFile.count() - 8); // drop the '-set.xml' portion
+            setFile.truncate(setFile.size() - 8); // drop the '-set.xml' portion
             m_config->setArg("aircraft", setFile);
-
-            flightgear::addSentryBreadcrumb("Launcher: collectAircraftArgs:" + m_config->getArg("aircraft").toStdString()
-                + " at " + m_config->getArg("aircraft-dir").toStdString(), "info");
         } else if (m_selectedAircraft.scheme() == "package") {
             // no need to set aircraft-dir, handled by the corresponding code
             // in fgInitAircraft
             m_config->setArg("aircraft", m_selectedAircraft.path());
-
-            flightgear::addSentryBreadcrumb("Launcher: collectAircraftArgs:" + m_config->getArg("aircraft").toStdString(), "info");
         } else {
             qWarning() << "unsupported aircraft launch URL" << m_selectedAircraft;
         }
@@ -317,9 +358,6 @@ void LauncherController::collectAircraftArgs()
         QString state = m_aircraftState;
         if ((m_aircraftState == "auto") && !m_selectedAircraftInfo->haveExplicitAutoState()) {
             state = selectAircraftStateAutomatically();
-#if QT_VERSION >= 0x050600
-            qInfo() << "doing launcher auto state selection, picked:" + state;
-#endif
         }
 
         if (!state.isEmpty()) {
@@ -330,7 +368,6 @@ void LauncherController::collectAircraftArgs()
 
 void LauncherController::saveAircraft()
 {
-    flightgear::addSentryBreadcrumb("Launcher: saveAircraft:" + m_selectedAircraft.toString().toStdString(), "info");
     m_config->setValueForKey("", "selected-aircraft", m_selectedAircraft);
     if (!m_aircraftState.isEmpty()) {
         m_config->setValueForKey("", "selected-aircraft-state", m_aircraftState);
@@ -361,7 +398,7 @@ void LauncherController::doRun()
     flightgear::addSentryBreadcrumb("acft path:" + m_selectedAircraftInfo->pathOnDisk().toStdString(), "info");
 
     // aircraft paths
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     QString downloadDir = settings.value("downloadSettings/downloadDir").toString();
     if (!downloadDir.isEmpty()) {
         QDir d(downloadDir);
@@ -399,7 +436,7 @@ void LauncherController::doApply()
             QFileInfo setFileInfo(m_selectedAircraft.toLocalFile());
             QString setFile = setFileInfo.fileName();
             Q_ASSERT(setFile.endsWith("-set.xml"));
-            setFile.truncate(setFile.count() - 8); // drop the '-set.xml' portion
+            setFile.truncate(setFile.size() - 8); // drop the '-set.xml' portion
             aircraftDir = setFileInfo.dir().absolutePath().toStdString();
             aircraftPropValue = setFile.toStdString();
         } else if (m_selectedAircraft.scheme() == "package") {
@@ -521,7 +558,7 @@ void LauncherController::downloadDirChanged(QString path)
 
     auto options = flightgear::Options::sharedInstance();
     if (options->valueForOption("download-dir") == path.toStdString()) {
-        // this works because we propogate the value from QSettings to
+        // this works because we propagate the value from QSettings to
         // the flightgear::Options object in runLauncherDialog()
         // so the options object always contains our current idea of this
         // value
@@ -771,7 +808,7 @@ QVariantList LauncherController::defaultSplashUrls() const
 
 QVariant LauncherController::loadUISetting(QString name, QVariant defaultValue) const
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     if (!settings.contains(name))
         return defaultValue;
     return settings.value(name);
@@ -779,7 +816,7 @@ QVariant LauncherController::loadUISetting(QString name, QVariant defaultValue)
 
 void LauncherController::saveUISetting(QString name, QVariant value) const
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     settings.setValue(name, value);
 }
 
@@ -826,8 +863,8 @@ void LauncherController::requestRestoreDefaults()
 	}
 
 	{
-		QSettings settings;
-		settings.clear();
+        auto settings = flightgear::getQSettings();
+        settings.clear();
 		settings.setValue("restore-defaults-on-run", true);
 	}
 
@@ -837,8 +874,8 @@ void LauncherController::requestRestoreDefaults()
 void LauncherController::requestChangeDataPath()
 {
 	QString currentLocText;
-	QSettings settings;
-	QString root = settings.value(SetupRootDialog::rootPathKey()).toString();
+    auto settings = flightgear::getQSettings();
+    QString root = settings.value(SetupRootDialog::rootPathKey()).toString();
 	if (root.isNull()) {
 		currentLocText = tr("Currently the built-in data files are being used");
 	}
@@ -894,7 +931,7 @@ void LauncherController::setAircraftGridMode(bool aircraftGridMode)
     if (m_aircraftGridMode == aircraftGridMode)
         return;
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     settings.setValue("aircraftGridMode", aircraftGridMode);
     m_aircraftGridMode = aircraftGridMode;
     emit aircraftGridModeChanged(m_aircraftGridMode);
@@ -903,7 +940,7 @@ void LauncherController::setAircraftGridMode(bool aircraftGridMode)
 void LauncherController::resetGettingStartedTips()
 {
     {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         settings.beginGroup("GettingStarted-DontShow");
         settings.remove(""); // remove all keys in the current group
         settings.endGroup();
@@ -924,14 +961,14 @@ void LauncherController::setMinWindowSize(QSize sz)
 QUrl LauncherController::flyIconUrl() const
 {
     if (m_aircraftType == Helicopter) {
-        return QUrl{"qrc:///svg/toolbox-fly-heli"};
+        return QUrl{"image://colored-icon/toolbox-fly-heli"};
     } else if (m_selectedAircraftInfo) {
         if (m_selectedAircraftInfo->hasTag("spaceship")) {
-            return QUrl{"qrc:///svg/toolbox-fly-alt"};
+            return QUrl{"image://colored-icon/toolbox-fly-alt"};
         }
     }
 
-    return QUrl{"qrc:///svg/toolbox-fly"};
+    return QUrl{"image://colored-icon/toolbox-fly"};
 }
 
 QString LauncherController::flyButtonLabel() const
@@ -996,3 +1033,27 @@ bool LauncherController::haveOldWindowsDownloadDir() const
     simgear::Dir texCacheDir(p / "TextureCache");
     return (texCacheDir.exists() && !texCacheDir.isEmpty());
 }
+
+bool LauncherController::isNetworkAvailable() const
+{
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 1, 0))
+    return QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online;
+#else
+    return true;
+#endif
+}
+
+void LauncherController::onReachabilityChanged()
+{
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 1, 0))
+    const auto online = (QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online);
+    auto pkgRoot = globals->packageRoot();
+    pkgRoot->setOnlineMode(online);
+
+    if (online) {
+        pkgRoot->refresh(true);
+    }
+
+    emit networkAvailableChanged();
+#endif
+}
