@@ -26,6 +26,7 @@
  **************************************************************************/
 
 
+#include "MouseCursor.hxx"
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -101,7 +102,7 @@ const __fg_gui_fn_t __fg_gui_fn[] = {
 // and we don't want to miss any, either.)
 void mkDialog (const char *txt)
 {
-    NewGUI *gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     if (!gui)
         return;
     SGPropertyNode *master = gui->getDialogProperties("message");
@@ -120,7 +121,7 @@ void mkDialog (const char *txt)
         if (!msg->getNode(name.c_str(), false))
             break;
 
-        if (!strcmp(txt, msg->getNode(name.c_str())->getStringValue("message"))) {
+        if (!strcmp(txt, msg->getNode(name.c_str())->getStringValue("message").c_str())) {
             SG_LOG(SG_GENERAL, SG_WARN, "mkDialog(): duplicate of message " << txt);
             return;
         }
@@ -180,10 +181,9 @@ bool openBrowser(const std::string& aAddress)
         // resolve local file path
         SGPath path(address);
         path = globals->resolve_maybe_aircraft_path(address);
-        if (!path.isNull())
-            address = path.local8BitStr();
-        else
-        {
+        if (!path.isNull()) {
+            address = "file://" + path.local8BitStr();
+        } else {
             mkDialog ("Sorry, file not found!");
             SG_LOG(SG_GENERAL, SG_ALERT, "openBrowser: Cannot find requested file '"  
                     << address << "'.");
@@ -192,10 +192,6 @@ bool openBrowser(const std::string& aAddress)
     }
 
 #ifdef SG_MAC
-  if (address.find("://")==string::npos) {
-    address = "file://" + address;
-  }
-  
   cocoaOpenUrl(address);
 #elif defined _WIN32
 
@@ -206,8 +202,8 @@ bool openBrowser(const std::string& aAddress)
 # else
     strncpy(win32_name,address.c_str(), 1024);
 # endif
-    ShellExecute ( NULL, "open", win32_name, NULL, NULL,
-                   SW_SHOWNORMAL ) ;
+    ShellExecuteA(NULL, "open", win32_name, NULL, NULL,
+                  SW_SHOWNORMAL);
 #else
     // Linux, BSD, SGI etc
     string command = globals->get_browser();
@@ -234,7 +230,7 @@ void fgHiResDump()
     string message;
     bool menu_status = fgGetBool("/sim/menubar/visibility");
     char *filename = new char [24];
-    static int count = 1;
+    static unsigned short count = 1;
 
     SGPropertyNode *master_freeze = fgGetNode("/sim/freeze/master");
 
@@ -244,8 +240,8 @@ void fgHiResDump()
     }
 
     fgSetBool("/sim/menubar/visibility", false);
-    int mouse = fgGetMouseCursor();
-    fgSetMouseCursor(MOUSE_CURSOR_NONE);
+    const auto mouse = fgGetMouseCursor();
+    fgSetMouseCursor(FGMouseCursor::CURSOR_NONE);
 
     FGRenderer *renderer = globals->get_renderer();
 //     renderer->init();
@@ -397,11 +393,11 @@ void fgHiResDump()
     glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
     glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
     glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_DONT_CARE);
-    if ( (!strcmp(fgGetString("/sim/rendering/fog"), "disabled")) ||
+    if ( fgGetString("/sim/rendering/fog") == "disabled" ||
          (!fgGetBool("/sim/rendering/shading"))) {
         // if fastest fog requested, or if flat shading force fastest
         glHint ( GL_FOG_HINT, GL_FASTEST );
-    } else if ( !strcmp(fgGetString("/sim/rendering/fog"), "nicest") ) {
+    } else if ( fgGetString("/sim/rendering/fog") == "nicest" ) {
         glHint ( GL_FOG_HINT, GL_DONT_CARE );
     }
 
@@ -508,7 +504,7 @@ namespace
             if (!_freeze)
                 _master_freeze->setBoolValue(true);
 
-            fgSetMouseCursor(MOUSE_CURSOR_NONE);
+            fgSetMouseCursor(FGMouseCursor::CURSOR_NONE);
 
             SGPath dir = SGPath::fromUtf8(fgGetString("/sim/paths/screenshot-dir"));
             if (dir.isNull())
@@ -527,7 +523,7 @@ namespace
             FGRenderer *renderer = globals->get_renderer();
             renderer->resize(_xsize, _ysize);
             globals->get_event_mgr()->addTask("SnapShotTimer",
-                    this, &GUISnapShotOperation::timerExpired,
+                    [this](){ this->timerExpired(); },
                     0.1, false);
         }
 
@@ -563,7 +559,7 @@ namespace
         SGPropertyNode_ptr _master_freeze;
         bool _freeze;
         bool _result;
-        int _mouse;
+        FGMouseCursor::Cursor _mouse;
         int _xsize, _ysize;
         SGPath _path;
     };
@@ -590,7 +586,7 @@ void fgDumpSceneGraph()
 {
     char *filename = new char [24];
     string message;
-    static int count = 1;
+    static unsigned short count = 1;
 
     SGPropertyNode *master_freeze = fgGetNode("/sim/freeze/master");
 
@@ -632,7 +628,7 @@ void fgDumpTerrainBranch()
 {
     char *filename = new char [24];
     string message;
-    static int count = 1;
+    static unsigned short count = 1;
 
     SGPropertyNode *master_freeze = fgGetNode("/sim/freeze/master");
 
