@@ -1,26 +1,10 @@
-// FGJoystickInput.cxx -- handle user input from joystick devices
-//
-// Written by Torsten Dreyer, started August 2009
-// Based on work from David Megginson, started May 2001.
-//
-// Copyright (C) 2009 Torsten Dreyer, Torsten (at) t3r _dot_ de
-// Copyright (C) 2001 David Megginson, david@megginson.com
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: FGJoystickInput.cxx
+ * SPDX-FileComment: handle user input from joystick devices, based on work from David Megginson, started May 2001.
+ * SPDX-FileCopyrightText: Copyright (C) 2009 Torsten Dreyer, Torsten (at) t3r _dot_ de
+ * SPDX-FileContributor: Copyright (C) 2001 David Megginson, david@megginson.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include "config.h"
 
@@ -32,7 +16,9 @@
 
 #include <cmath>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/props/props_io.hxx>
+
 #include "FGDeviceConfigurationMap.hxx"
 #include <Main/fg_props.hxx>
 #include <Scripting/NasalSys.hxx>
@@ -138,7 +124,7 @@ void FGJoystickInput::init()
       SGPropertyNode_ptr named;
 
       // allow distinguishing duplicated devices by the name
-      string indexedName = computeDeviceIndexName(name, i);
+      std::string indexedName = computeDeviceIndexName(name, i);
       if (configMap.hasConfiguration(indexedName)) {
           named = configMap.configurationForDeviceName(indexedName);
           std::string source = named->getStringValue("source", "user defined");
@@ -193,7 +179,7 @@ void FGJoystickInput::reinit()
 
 void FGJoystickInput::postinit()
 {
-  FGNasalSys *nasalsys = (FGNasalSys *)globals->get_subsystem("nasal");
+  auto nasalsys = globals->get_subsystem<FGNasalSys>();
   SGPropertyNode_ptr js_nodes = fgGetNode("/input/joysticks");
 
   for (int i = 0; i < MAX_JOYSTICKS; i++) {
@@ -202,6 +188,9 @@ void FGJoystickInput::postinit()
     if (!js_node || js->notWorking())
       continue;
 
+    // FIXME : need to get input device path to disambiguate
+    simgear::ErrorReportContext errCtx("input-device", "");
+
 #ifdef WIN32
     JOYCAPS jsCaps ;
     joyGetDevCaps( i, &jsCaps, sizeof(jsCaps) );
@@ -244,7 +233,12 @@ void FGJoystickInput::postinit()
     unsigned int j;
     for (j = 0; j < nasal.size(); j++) {
       nasal[j]->setStringValue("module", module.c_str());
-      nasalsys->handleCommand(nasal[j],nullptr);
+      bool ok = nasalsys->handleCommand(nasal[j], nullptr);
+      if (!ok) {
+          // TODO: get the Nasal errors logged properly
+          simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::InputDeviceConfig,
+                                 "Failed to parse input device Nasal");
+      }
     }
 
     //
