@@ -70,7 +70,7 @@ void KLN89AptPage::Update(double dt) {
 	bool multi;  // Not set by FindFirst...
 	bool exact = false;
 	if(_apt_id.size() == 4) exact = true;
-	// TODO - move this search out to where the button is pressed, and cache the result!
+	// TODO: move this search out to where the button is pressed, and cache the result!
 	if(_apt_id != _last_apt_id || ap == NULL) ap = _kln89->FindFirstAptById(_apt_id, multi, exact);
 	//if(np == NULL) cout << "NULL... ";
 	//if(b == false) cout << "false...\n";
@@ -149,8 +149,8 @@ void KLN89AptPage::Update(double dt) {
 			_kln89->DrawText("UTC", 2, 0, 2);
 			if(hrDiff != 0) {
 				_kln89->DrawText(ap->getLongitude() >= 0.0 ? "+" : "-", 2, 3, 2);
-				char buf[3];
-				snprintf(buf, 3, "%02i", hrDiff);
+				char buf[12];
+				snprintf(buf, sizeof(buf), "%02i", hrDiff);
 				_kln89->DrawText((string)buf, 2, 4, 2);
 				_kln89->DrawText("(   DT)", 2, 6, 2);
 				if(ap->getLongitude() >= 0.0) {
@@ -159,13 +159,13 @@ void KLN89AptPage::Update(double dt) {
 					hrDiff--;
 				}
 				_kln89->DrawText(ap->getLongitude() >= 0.0 ? "+" : "-", 2, 7, 2);
-				snprintf(buf, 3, "%02i", hrDiff);
+				snprintf(buf, sizeof(buf), "%02i", hrDiff);
 				_kln89->DrawText((string)buf, 2, 8, 2);
 			}
 			// I guess we can make a heuristic guess as to fuel availability from the runway sizes
 			// For now assume that airports with asphalt or concrete runways will have at least 100L,
 			// and that runways over 4000ft will have JET.
-			if(_aptRwys[0]->surface() <= 2) {
+			if(_aptRwys[0]->isHardSurface()) {
 				if(_aptRwys[0]->lengthFt() >= 4000) {
 					_kln89->DrawText("JET 100L", 2, 0, 1);
 				} else {
@@ -175,7 +175,7 @@ void KLN89AptPage::Update(double dt) {
 			if(_iaps.empty()) {
 				_kln89->DrawText("NO APR", 2, 0, 0);
 			} else {
-				// TODO - output proper differentiation of ILS and NP APR and NP APR type eg GPS(R)
+				// TODO: output proper differentiation of ILS and NP APR and NP APR type eg GPS(R)
 				_kln89->DrawText("NP APR", 2, 0, 0);
 			}
 		} else if(_subPage == 3) {
@@ -196,12 +196,37 @@ void KLN89AptPage::Update(double dt) {
 				_kln89->DrawText(s, 2, 5 - s.size(), 2);
 				_kln89->DrawText((_kln89->_altUnits == GPS_ALT_UNITS_FT ? "ft" : "m"), 2, 5, 2);
 				// Surface
-				// TODO - why not store these strings as an array?
+				// TODO: why not store these strings as an array?
 				switch(_aptRwys[i]->surface()) {
 				case 1:
-					// Asphalt - fall through
-				case 2:
-					// Concrete
+				case 20: // Light colored asphalt
+				case 21:
+				case 22:
+				case 23:
+				case 24: // Asphalt
+				case 25:
+				case 26:
+				case 27: // Darker colored asphalt
+				case 28:
+				case 29:
+				case 30:
+				case 31: // Very dark colored asphalt
+				case 32:
+				case 33:
+				case 34:
+				case 35: // Near black, ‘new’ looking asphalt
+				case 36:
+				case 37:
+				case 38:
+				case 2:  //Concrete
+				case 50: // Light “new” looking concrete
+				case 51:
+				case 52:
+				case 53: // Concrete
+				case 54:
+				case 55: // Dark concrete
+				case 56:
+				case 57:
 					_kln89->DrawText("HRD", 2, 9, 2);
 					break;
 				case 3:
@@ -246,12 +271,37 @@ void KLN89AptPage::Update(double dt) {
 				_kln89->DrawText(s, 2, 5 - s.size(), 0);
 				_kln89->DrawText((_kln89->_altUnits == GPS_ALT_UNITS_FT ? "ft" : "m"), 2, 5, 0);
 				// Surface
-				// TODO - why not store these strings as an array?
+				// TODO: why not store these strings as an array?
 				switch(_aptRwys[i]->surface()) {
 				case 1:
-					// Asphalt - fall through
-				case 2:
-					// Concrete
+				case 20: // Light colored asphalt
+				case 21:
+				case 22:
+				case 23:
+				case 24: // Asphalt
+				case 25:
+				case 26:
+				case 27: // Darker colored asphalt
+				case 28:
+				case 29:
+				case 30:
+				case 31: // Very dark colored asphalt
+				case 32:
+				case 33:
+				case 34:
+				case 35: // Near black, ‘new’ looking asphalt
+				case 36:
+				case 37:
+				case 38:
+				case 2:  //Concrete
+				case 50: // Light “new” looking concrete
+				case 51:
+				case 52:
+				case 53: // Concrete
+				case 54:
+				case 55: // Dark concrete
+				case 56:
+				case 57:
 					_kln89->DrawText("HRD", 2, 9, 0);
 					break;
 				case 3:
@@ -303,7 +353,7 @@ void KLN89AptPage::Update(double dt) {
 				_kln89->DrawFreq(_aptFreqs[i].freq, 2, 7, 0);
 			}
 		} else if(_subPage == 5) {
-			// TODO - user ought to be allowed to leave persistent remarks
+			// TODO: user ought to be allowed to leave persistent remarks
 			_kln89->DrawText("[Remarks]", 2, 2, 2);
 		} else if(_subPage == 6) {
 			// We don't have SID/STAR database yet
@@ -427,7 +477,7 @@ void KLN89AptPage::Update(double dt) {
 	if(_kln89->_mode == KLN89_MODE_CRSR) {
 		if(!(_subPage == 7 && (_iafDialog || _addDialog || _replaceDialog))) {
 			if(_uLinePos > 0 && _uLinePos < 5) {
-				// TODO - blink as well
+				// TODO: blink as well
 				_kln89->Underline(2, _uLinePos, 3, 1);
 			}
 			for(unsigned int i = 0; i < _apt_id.size(); ++i) {
@@ -507,7 +557,7 @@ void KLN89AptPage::UpdateAirport(const string& id) {
 	if(_nRwyPages < 1) _nRwyPages = 1;
 	
 	// Instrument approaches
-	// Only non-precision for now - TODO - handle precision approaches if necessary
+	// Only non-precision for now - TODO: handle precision approaches if necessary
 	_iaps.clear();
 	iap_map_iterator itr = _kln89->_np_iap.find(id);
 	if(itr != _kln89->_np_iap.end()) {
@@ -517,7 +567,7 @@ void KLN89AptPage::UpdateAirport(const string& id) {
 		if(_iafDialog || _addDialog || _replaceDialog) {
 			// Eek - major logic error if an airport details cache update occurs
 			// with one of these dialogs active.
-			// TODO - output a warning.
+			// TODO: output a warning.
 			//cout << "HELP!!!!!!!!!!\n";
 		} else {
 			_maxULinePos = 4 + _iaps.size();	// We shouldn't need to check the crsr for out-of-bounds here since we only update the airport details when the airport code is changed - ie. _uLinePos <= 4!
@@ -585,7 +635,7 @@ void KLN89AptPage::ClrPressed() {
 				_iafDialog = true;
 				_maxULinePos = 1;
 				// Don't reset _curIaf since it is remembed.
-				_uLinePos = 1 + _curIaf;	// TODO - make this robust to more than 3 IAF
+				_uLinePos = 1 + _curIaf;	// TODO: make this robust to more than 3 IAF
 			} else {
 				_maxULinePos = 4 + _iaps.size();
 				if(_iaps.empty()) {
@@ -623,7 +673,7 @@ void KLN89AptPage::EntPressed() {
 					_curIaf = _uLinePos - 1 + _iafStart;
 				}
 				//cout << "_curIaf = " << _curIaf << '\n';
-				// TODO - delete the waypoints inside _approachFP before clearing them!!!!!!!
+				// TODO: delete the waypoints inside _approachFP before clearing them!!!!!!!
 				_kln89->_approachFP->waypoints.clear();
 				GPSWaypoint* wp = new GPSWaypoint;
 				*wp = *(_approachRoutes[_curIaf]->waypoints[0]);	// Need to make copies here since we're going to alter ID and type sometimes
@@ -679,7 +729,7 @@ void KLN89AptPage::EntPressed() {
 					_kln89->_approachLoaded = true;
 					//_kln89->_messageStack.push_back("*Press ALT To Set Baro");
 					// Actually - this message is only sent when we go into appraoch-arm mode.
-					// TODO - check the flightplan for consistency
+					// TODO: check the flightplan for consistency
 					_kln89->OrientateToActiveFlightPlan();
 					_kln89->_mode = KLN89_MODE_DISP;
 					_kln89->_curPage = 7;
@@ -687,14 +737,14 @@ void KLN89AptPage::EntPressed() {
 				}
 			}
 		} else if(_replaceDialog) {
-			// TODO - load the approach!
+			// TODO: load the approach!
 		} else if(_uLinePos > 4) {
 			_approachRoutes.clear();
 			_IAP.clear();
 			_curIaf = 0;
 			_approachRoutes = ((FGNPIAP*)(_iaps[_uLinePos-5]))->_approachRoutes;
 			_IAP = ((FGNPIAP*)(_iaps[_uLinePos-5]))->_IAP;
-			_curIap = _uLinePos - 5;	// TODO - handle the start of list ! no. 1, and the end of list not sequential!
+			_curIap = _uLinePos - 5;	// TODO: handle the start of list ! no. 1, and the end of list not sequential!
 			_uLinePos = 1;
 			if(_approachRoutes.size() > 1) {
 				// More than 1 IAF - display the selection dialog
@@ -702,7 +752,7 @@ void KLN89AptPage::EntPressed() {
 				_maxULinePos = _approachRoutes.size();
 			} else {
 				// There is only 1 IAF, so load the waypoints into the approach flightplan here.
-				// TODO - there is nasty code duplication loading the approach FP between the case here where we have only one
+				// TODO: there is nasty code duplication loading the approach FP between the case here where we have only one
 				// IAF and the case where we must choose the IAF from a list.  Try to tidy this after it is all working properly.
 				_kln89->_approachFP->waypoints.clear();
 				GPSWaypoint* wp = new GPSWaypoint;
@@ -814,7 +864,7 @@ void KLN89AptPage::Knob2Left1() {
 			}
 		} else {
 			if(_subPage == 0) {
-				// TODO - set by name
+				// TODO: set by name
 			} else {
 				// NO-OP - to/fr is cycled by clr button
 			}
@@ -866,7 +916,7 @@ void KLN89AptPage::Knob2Right1() {
 			}
 		} else {
 			if(_subPage == 0) {
-				// TODO - set by name
+				// TODO: set by name
 			} else {
 				// NO-OP - to/fr is cycled by clr button
 			}
