@@ -1,11 +1,13 @@
-// heading_indicator.cxx - a vacuum-powered heading indicator.
-// Written by David Megginson, started 2002.
-//
-// This file is in the Public Domain and comes with no warranty.
+/*
+ * SPDX-FileName: heading_indicator.cxx
+ * SPDX-FileComment: a vacuum-powered heading indicator.
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-FileContributor:  Written by David Megginson, started 2002.
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
+
+#include "heading_indicator.hxx"
 
 #include <simgear/compiler.h>
 #include <simgear/sg_inlines.h>
@@ -14,7 +16,6 @@
 #include <string>
 #include <sstream>
 
-#include "heading_indicator.hxx"
 #include <Main/fg_props.hxx>
 #include <Main/util.hxx>
 
@@ -36,13 +37,13 @@ HeadingIndicator::init ()
     std::string branch;
     branch = "/instrumentation/" + _name;
 
-    SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
+    SGPropertyNode *node = fgGetNode(branch, _num, true );
     if( NULL == (_offset_node = node->getChild("offset-deg", 0, false)) ) {
       _offset_node = node->getChild("offset-deg", 0, true);
       _offset_node->setDoubleValue( -fgGetDouble("/environment/magnetic-variation-deg") );
     }
     _heading_in_node = fgGetNode("/orientation/heading-deg", true);
-    _suction_node = fgGetNode(_suction.c_str(), true);
+    _suction_node = fgGetNode(_suction, true);
     _heading_out_node = node->getChild("indicated-heading-deg", 0, true);
     _heading_bug_error_node = node->getChild("heading-bug-error-deg", 0, true);
     _heading_bug_node = node->getChild("heading-bug-deg", 0, true);
