@@ -27,7 +27,7 @@
 #include <stdio.h>	// snprintf
 
 #include <simgear/compiler.h>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/timing/sg_time.hxx>
 #include <simgear/sound/sample_group.hxx>
@@ -114,7 +114,7 @@ FGKR_87::~FGKR_87() {
 
 
 void FGKR_87::init () {
-    SGSoundMgr *smgr = globals->get_subsystem<SGSoundMgr>();
+    auto smgr = globals->get_subsystem<SGSoundMgr>();
     _sgr = smgr->find("avionics", true);
     _sgr->tie_to_listener();
 }
@@ -402,9 +402,10 @@ void FGKR_87::update( double dt_sec ) {
     if ( big > 99 ) {
         big = 99;
     }
-    char formatted_timer[128];
+
+    char formatted_timer[24];
     // cout << big << ":" << little << endl;
-    snprintf(formatted_timer, 6, "%02d:%02d", big, little);
+    snprintf(formatted_timer, 24, "%02d:%02d", big, little);
     fgSetString( "/instrumentation/kr-87/outputs/timer-string",
                  formatted_timer );
 
