@@ -12,7 +12,7 @@
 
 #include <simgear/compiler.h>
 #include <simgear/math/sg_geodesy.hxx>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 
 #include <Main/fg_props.hxx>
 #include <Navaids/navlist.hxx>
@@ -71,7 +71,7 @@ void TACAN::init()
 {
     std::string branch = "/instrumentation/" + _name;
 
-    SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
+    SGPropertyNode *node = fgGetNode(branch, _num, true );
 
     _serviceable_node = node->getChild("serviceable", 0, true);
     _ident_node = node->getChild("ident", 0, true);
@@ -291,7 +291,7 @@ TACAN::valueChanged(SGPropertyNode *prop)
     int index = prop->getIndex();
     std::string channel = _channel;
 
-    if (std::string("selected-mhz") == prop->getName())
+    if (std::string("selected-mhz") == prop->getNameString())
     {
         FGTACANRecord *rec= globals->get_channellist()->findByFrequency(prop->getDoubleValue()*1000);
         if (rec != nullptr) {
