@@ -1,23 +1,12 @@
 /*
- * Copyright (C) 2021 James Turner
- *
- * This file is part of the program FlightGear.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-FileName: ErrorReporter.cxx
+ * SPDX-FileComment: This file is part of the program FlightGear
+ * SPDX-FileCopyrightText: Copyright (C) 2021 James Turner
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
+#include "fg_props.hxx"
 
 #include "ErrorReporter.hxx"
 
@@ -26,6 +15,7 @@
 #include <deque>
 #include <map>
 #include <mutex>
+#include <string>
 
 #include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/LogCallback.hxx>
@@ -35,14 +25,18 @@
 #include <simgear/structure/commands.hxx>
 
 #include <GUI/MessageBox.hxx>
+#include <GUI/dialog.hxx>
 #include <GUI/new_gui.hxx>
+
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
 #include <Main/locale.hxx>
 #include <Main/options.hxx>
 #include <Main/sentryIntegration.hxx>
+#include <Model/validateSharedModels.hxx>
 #include <Scripting/NasalClipboard.hxx> // clipboard access
 
+using simgear::LoadFailure;
 using std::string;
 
 namespace {
@@ -72,10 +66,12 @@ enum class Aggregation {
     InputDevice,
     FGData,
     MultiPlayer,
-    Unknown,     ///< error coudln't be attributed more specifcially
-    OutOfMemory, ///< seperate category to give it a custom message
+    Unknown,      ///< error coudln't be attributed more specifcially
+    OutOfMemory,  ///< separate category to give it a custom message
+    SharedModels, ///< special category, for when shared models are not found
     Traffic,
-    ShadersEffects
+    ShadersEffects,
+    NetworkFailure, ///< category for issues related to (local) connectivity
 };
 
 // these should correspond to simgear::ErrorCode enum
@@ -119,8 +115,10 @@ string_list static_categoryIds = {
     "error-category-multiplayer",
     "error-category-unknown",
     "error-category-out-of-memory",
+    "error-category-shared-models",
     "error-category-traffic",
-    "error-category-shaders"};
+    "error-category-shaders",
+    "error-category-network"};
 
 class RecentLogCallback : public simgear::LogCallback
 {
@@ -131,7 +129,7 @@ public:
 
     bool doProcessEntry(const simgear::LogEntry& e) override
     {
-        ostringstream os;
+        std::ostringstream os;
         if (e.file != nullptr) {
             os << e.file << ":" << e.line << ":\t";
         }
@@ -159,12 +157,18 @@ public:
 
 private:
     mutable std::mutex _lock;
-    int _preceedingLogMessageCount = 6;
+    size_t _preceedingLogMessageCount = 6;
 
     using LogDeque = std::deque<string>;
     LogDeque _recentLogEntries;
 };
 
+std::string lastPathComponent(const std::string& d)
+{
+    const auto lastSlash = d.rfind('/');
+    return d.substr(lastSlash + 1);
+}
+
 } // namespace
 
 namespace flightgear {
@@ -176,15 +180,18 @@ public:
     std::mutex _lock;
     SGTimeStamp _nextShowTimeout;
     bool _haveDonePostInit = false;
+    bool _isEnabled = true;
 
+    bool _enabled = true;
     SGPropertyNode_ptr _enabledNode;
     SGPropertyNode_ptr _displayNode;
     SGPropertyNode_ptr _activeErrorNode;
     SGPropertyNode_ptr _mpReportNode;
+    bool _popupEnabled = true;
 
     using ErrorContext = std::map<std::string, std::string>;
     /**
-            strucutre representing a single error which has cocurred
+    @brief structure representing a single error which has occurred
      */
     struct ErrorOcurrence {
         simgear::ErrorCode code;
@@ -220,13 +227,20 @@ public:
     string _aircraftDirectoryName;
 
     /**
-        @brief hueristic to identify relative paths as origination from the main aircraft as opposed
+        @brief heuristic to identify relative paths as origination from the main aircraft as opposed
                 to something else. This is based on containing the aircraft directory name.
      */
     bool isMainAircraftPath(const std::string& path) const;
 
+    bool isAnyAircraftPath(const std::string& path) const;
+
+    static bool isFGDataPath(const std::string& path);
+
     /**
-        structure representing one or more errors, aggregated together
+    @brief structure representing one or more errors, aggregated together
+
+    This is what we send to Sentry, or present to the user, after combining occurrences from
+    the same / related sources, or duplicates.
      */
     struct AggregateReport {
         Aggregation type;
@@ -236,8 +250,12 @@ public:
         bool haveShownToUser = false;
         OccurrenceVec errors;
         bool isCritical = false;
+        bool isMessage = false;
 
         bool addOccurence(const ErrorOcurrence& err);
+
+        using OcurrencePredicate = std::function<bool(const ErrorOcurrence&)>;
+        bool allOccurences(OcurrencePredicate pred) const;
     };
 
 
@@ -253,12 +271,14 @@ public:
 
     AggregateErrors::iterator getAggregate(Aggregation ag, const std::string& param = {});
 
+    AggregateErrors::iterator mainAircraftAggregate();
+
 
     void collectError(simgear::LoadFailure type, simgear::ErrorCode code, const std::string& details, const sg_location& location)
     {
-        ErrorOcurrence occurrence{code, type, details, location, time(nullptr)};
+        ErrorOcurrence occurrence{code, type, details, location, time(nullptr), string_list(), ErrorContext()};
 
-        // snapshot the top of the context stacks into our occurence data
+        // snapshot the top of the context stacks into our occurrence data
         for (const auto& c : thread_errorContextStack) {
             occurrence.context[c.first] = c.second.back();
         }
@@ -268,7 +288,7 @@ public:
         std::lock_guard<std::mutex> g(_lock); // begin access to shared state
         auto it = getAggregateForOccurence(occurrence);
 
-        // add to the occurence, if it's not a duplicate
+        // add to the occurrence, if it's not a duplicate
         if (!it->addOccurence(occurrence)) {
             return; // duplicate, nothing else to do
         }
@@ -276,24 +296,20 @@ public:
         // log it once we know it's not a duplicate
         SG_LOG(SG_GENERAL, SG_WARN, "Error:" << static_errorTypeIds.at(static_cast<int>(type)) << " from " << static_errorIds.at(static_cast<int>(code)) << "::" << details << "\n\t" << location.asString());
 
+        if (!_enabled) {
+            return;
+        }
+
         it->lastErrorTime.stamp();
         _reportsDirty = true;
 
         const auto ty = it->type;
         // decide if it's a critical error or not
-        if ((ty == Aggregation::OutOfMemory) || (ty == Aggregation::InputDevice)) {
-            it->isCritical = true;
-        }
-
         // aircraft errors are critical if they occur during initial
         // aircraft load, otherwise we just show the warning
         if (!_haveDonePostInit && (ty == Aggregation::MainAircraft)) {
             it->isCritical = true;
         }
-
-        if (code == simgear::ErrorCode::LoadEffectsShaders) {
-            it->isCritical = true;
-        }
     }
 
     void collectContext(const std::string& key, const std::string& value)
@@ -328,7 +344,7 @@ public:
         // remove any existing error children
         _displayNode->removeChildren("error");
 
-        ostringstream detailsTextStream;
+        std::ostringstream detailsTextStream;
 
         // add all the discrete errors as child nodes with all their information
         for (const auto& e : report.errors) {
@@ -366,15 +382,40 @@ public:
         assert(it != _aggregated.end());
         _activeReportIndex = static_cast<int>(std::distance(_aggregated.begin(), it));
         _displayNode->setBoolValue("index", _activeReportIndex);
-        _displayNode->setBoolValue("have-next", _activeReportIndex < (_aggregated.size() - 1));
+        _displayNode->setBoolValue("have-next", _activeReportIndex < (int)_aggregated.size() - 1);
         _displayNode->setBoolValue("have-previous", _activeReportIndex > 0);
     }
 
     void sendReportToSentry(AggregateReport& report)
     {
+        if (report.type == Aggregation::InputDevice) {
+            // for non-official input configs, don't report, it's just noise
+            // https://gitlab.com/flightgear/flightgear/-/issues/3177
+            if (report.allOccurences([](const ErrorOcurrence& oc) {
+                    return !isFGDataPath(oc.origin.getPath());
+                })) {
+                return;
+            }
+        }
+
+        // don't report socket or local IO issues to Sentry
+        // https://gitlab.com/flightgear/fgmeta/-/issues/33
+        if (report.type == Aggregation::NetworkFailure) {
+            return;
+        }
+
+        if (report.type == Aggregation::TerraSync) {
+            if (report.allOccurences([](const ErrorOcurrence& oc) {
+                    return (oc.type == LoadFailure::IOError) || (oc.type == LoadFailure::NetworkError);
+                })) {
+                return;
+            }
+        }
+
         const int catId = static_cast<int>(report.type);
-        flightgear::sentryReportUserError(static_categoryIds.at(catId), 
-            _displayNode->getStringValue("details-text"));
+        flightgear::sentryReportUserError(static_categoryIds.at(catId),
+                                          report.parameter,
+                                          _displayNode->getStringValue());
     }
 
     void writeReportToStream(const AggregateReport& report, std::ostream& os) const;
@@ -385,8 +426,29 @@ public:
     bool dismissReportCommand(const SGPropertyNode* args, SGPropertyNode*);
     bool saveReportCommand(const SGPropertyNode* args, SGPropertyNode*);
     bool showErrorReportCommand(const SGPropertyNode* args, SGPropertyNode*);
+
+    /**
+     * @brief check if the report is still adding new errors, we avoid showing the UI
+     * until all the errors are collected/
+     */
+    bool reportIsOngoing(const AggregateReport& report) const;
 };
 
+auto ErrorReporter::ErrorReporterPrivate::mainAircraftAggregate()
+    -> AggregateErrors::iterator
+{
+    const auto fullId = fgGetString("/sim/aircraft-id");
+
+    // we use the dir name so we combine reports from different variants, on Sentry
+    const auto aircraftDirName = lastPathComponent(fgGetString("/sim/aircraft-dir"));
+
+    if (fullId != fgGetString("/sim/aircraft")) {
+        return getAggregate(Aggregation::MainAircraft, aircraftDirName);
+    }
+
+    return getAggregate(Aggregation::HangarAircraft, aircraftDirName);
+}
+
 auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorReporter::ErrorReporterPrivate::ErrorOcurrence& oc)
     -> AggregateErrors::iterator
 {
@@ -397,13 +459,16 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorRe
         return getAggregate(Aggregation::OutOfMemory, {});
     }
 
-    if (oc.hasContextKey("primary-aircraft")) {
-        const auto fullId = fgGetString("/sim/aircraft-id");
-        if (fullId != fgGetString("/sim/aircraft")) {
-            return getAggregate(Aggregation::MainAircraft, fullId);
+    // errors about shared models typically indicate that the shared Models are
+    // missing or out of date.
+    if ((oc.type == simgear::LoadFailure::NotFound) || (oc.type == simgear::LoadFailure::BadData)) {
+        if (oc.detailedInfo.find("OBJECT_SHARED") != std::string::npos) {
+            return getAggregate(Aggregation::SharedModels);
         }
+    }
 
-        return getAggregate(Aggregation::HangarAircraft, fullId);
+    if (oc.hasContextKey("primary-aircraft")) {
+        return mainAircraftAggregate();
     }
 
     if (oc.hasContextKey("multiplayer")) {
@@ -423,6 +488,17 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorRe
     // all TerraSync coded errors go there: this is errors for the
     // actual download process (eg, failed to write to disk)
     if (oc.code == simgear::ErrorCode::TerraSync) {
+        const auto details = oc.detailedInfo;
+        if ((details.find("socket error") != std::string::npos) ||
+            (details.find("SSL connect error") != std::string::npos) ||
+            (details.find("Couldn't resolve host name") != std::string::npos)) {
+            return getAggregate(Aggregation::NetworkFailure, {});
+        }
+
+        if (oc.type == simgear::LoadFailure::NetworkError) {
+            return getAggregate(Aggregation::NetworkFailure, {});
+        }
+
         return getAggregate(Aggregation::TerraSync, {});
     }
 
@@ -435,11 +511,6 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorRe
         // incorrectly, we attribute the error to the scenery, which is
         // likely what we want/expect
         auto path = oc.hasContextKey("terrain-stg") ? oc.getContextValue("terrain-stg") : oc.getContextValue("btg");
-        if (simgear::strutils::starts_with(path, _fgdataPathPrefix)) {
-            return getAggregate(Aggregation::FGData, {});
-        } else if (simgear::strutils::starts_with(path, _terrasyncPathPrefix)) {
-            return getAggregate(Aggregation::TerraSync, {});
-        }
 
         // custom scenery, find out the prefix
         for (const auto& sceneryPath : globals->get_fg_scenery()) {
@@ -449,10 +520,22 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorRe
             }
         }
 
+        // try generic paths
+        if (simgear::strutils::starts_with(path, _fgdataPathPrefix)) {
+            return getAggregate(Aggregation::FGData, {});
+        } else if (simgear::strutils::starts_with(path, _terrasyncPathPrefix)) {
+            return getAggregate(Aggregation::TerraSync, {});
+        }
+
         // shouldn't ever happen
         return getAggregate(Aggregation::CustomScenery, {});
     }
 
+    // assume only FGData can define materials, for now.
+    if (oc.hasContextKey("materials")) {
+        return getAggregate(Aggregation::FGData);
+    }
+
     if (oc.hasContextKey("scenario-path")) {
         const auto scenarioPath = oc.getContextValue("scenario-path");
         return getAggregate(Aggregation::Scenario, scenarioPath);
@@ -462,29 +545,49 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregateForOccurence(const ErrorRe
         return getAggregate(Aggregation::InputDevice, oc.getContextValue("input-device"));
     }
 
+    if (oc.code == simgear::ErrorCode::AircraftSystems) {
+        if (isMainAircraftPath(oc.origin.asString())) {
+            return mainAircraftAggregate();
+        }
+
+        if (isFGDataPath(oc.origin.asString())) {
+            return getAggregate(Aggregation::FGData);
+        }
+    }
+
     // start guessing :)
     // from this point on we're using less reliable inferences about where the
     // error came from, trying to avoid 'unknown'
     if (isMainAircraftPath(oc.origin.asString())) {
-        const auto fullId = fgGetString("/sim/aircraft-id");
-        if (fullId != fgGetString("/sim/aircraft")) {
-            return getAggregate(Aggregation::MainAircraft, fullId);
-        }
-
-        return getAggregate(Aggregation::HangarAircraft, fullId);
+        return mainAircraftAggregate();
     }
 
     // GUI dialog errors often have no context
     if (oc.code == simgear::ErrorCode::GUIDialog) {
+        // check if it's an aircraft dialog
+        if (isMainAircraftPath(oc.origin.asString())) {
+            const auto aircraftDirName = lastPathComponent(fgGetString("/sim/aircraft-dir"));
+            return getAggregate(Aggregation::MainAircraft, aircraftDirName);
+        }
+
         // check if it's an add-on and use that
         return getAggregate(Aggregation::FGData);
     }
 
-    // becuase we report shader errors from the main thread, they don't
+    // because we report shader errors from the main thread, they don't
     // get attributed. Collect them into their own category, which also
     // means we can display a more specific message
     if (oc.code == simgear::ErrorCode::LoadEffectsShaders) {
-        return getAggregate(Aggregation::ShadersEffects);
+        // we use the effect name to split shader errors
+        return getAggregate(Aggregation::ShadersEffects, oc.getContextValue("effect"));
+    }
+
+    // if we've got this far, and the path looks like an aircraft path, assume it was a cross-aircraft
+    // error, and report as such
+    // see https://gitlab.com/flightgear/flightgear/-/issues/3008
+    if (isAnyAircraftPath(oc.origin.asString())) {
+        const auto aircraftDirName = lastPathComponent(fgGetString("/sim/aircraft-dir"));
+        return getAggregate(Aggregation::MainAircraft, aircraftDirName);
     }
 
     return getAggregate(Aggregation::Unknown);
@@ -506,6 +609,14 @@ auto ErrorReporter::ErrorReporterPrivate::getAggregate(Aggregation ag, const std
         it = _aggregated.end() - 1;
     }
 
+    switch (ag) {
+    case Aggregation::OutOfMemory:
+    case Aggregation::NetworkFailure:
+        it->isMessage = true;
+    default:
+        break;
+    }
+
     return it;
 }
 
@@ -516,46 +627,46 @@ void ErrorReporter::ErrorReporterPrivate::writeReportToStream(const AggregateRep
         char buf[64];
         time_t now = time(nullptr);
         strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S GMT", gmtime(&now));
-        os << buf << endl;
+        os << buf << std::endl;
     }
 
-    os << "Category:" << static_categoryIds.at(static_cast<int>(report.type)) << endl;
+    os << "Category:" << static_categoryIds.at(static_cast<int>(report.type)) << std::endl;
     if (!report.parameter.empty()) {
-        os << "\tParameter:" << report.parameter << endl;
+        os << "\tParameter:" << report.parameter << std::endl;
     }
 
-    os << endl; // insert a blank line after header data
+    os << std::endl; // insert a blank line after header data
 
     int index = 1;
     char whenBuf[64];
 
     for (const auto& err : report.errors) {
         os << "Error " << index++ << std::endl;
-        os << "\tcode:" << static_errorIds.at(static_cast<int>(err.code)) << endl;
-        os << "\ttype:" << static_errorTypeIds.at(static_cast<int>(err.type)) << endl;
+        os << "\tcode:" << static_errorIds.at(static_cast<int>(err.code)) << std::endl;
+        os << "\ttype:" << static_errorTypeIds.at(static_cast<int>(err.type)) << std::endl;
 
         strftime(whenBuf, sizeof(whenBuf), "%H:%M:%S GMT", gmtime(&err.when));
-        os << "\twhen:" << whenBuf << endl;
+        os << "\twhen:" << whenBuf << std::endl;
 
         os << "\t" << err.detailedInfo << std::endl;
-        os << "\tlocation:" << err.origin.asString() << endl;
+        os << "\tlocation:" << err.origin.asString() << std::endl;
         writeContextToStream(err, os);
         writeLogToStream(err, os);
         os << std::endl; // trailing blank line
     }
 
-    os << "Command line / launcher / fgfsrc options" << endl;
+    os << "Command line / launcher / fgfsrc options" << std::endl;
     for (auto o : Options::sharedInstance()->extractOptions()) {
         os << "\t" << o << "\n";
     }
-    os << endl;
+    os << std::endl;
 
     writeSignificantPropertiesToStream(os);
 }
 
 void ErrorReporter::ErrorReporterPrivate::writeSignificantPropertiesToStream(std::ostream& os) const
 {
-    os << "Properties:" << endl;
+    os << "Properties:" << std::endl;
     for (const auto& ps : _significantProperties) {
         auto node = fgGetNode(ps);
         if (!node) {
@@ -564,7 +675,7 @@ void ErrorReporter::ErrorReporterPrivate::writeSignificantPropertiesToStream(std
             os << "\t" << ps << ": " << node->getStringValue() << "\n";
         }
     }
-    os << endl;
+    os << std::endl;
 }
 
 
@@ -680,18 +791,8 @@ void ErrorReporter::ErrorReporterPrivate::writeLogToStream(const ErrorOcurrence&
 
 bool ErrorReporter::ErrorReporterPrivate::AggregateReport::addOccurence(const ErrorOcurrence& err)
 {
-    // hacky reduction in noise for 2020.3: special case this error.
-    // file was removed but update-in-place means it will persist
-    // this is the removed RNLAF traffic which has many errors
-    if (type == Aggregation::Traffic) {
-        const auto originPath = err.origin.asString();
-        if (simgear::strutils::ends_with(originPath, "Traffic/R/RNAF.xml")) {
-            return false; // don't add
-        }
-    }
-
     auto it = std::find_if(errors.begin(), errors.end(), [err](const ErrorOcurrence& ext) {
-        // check if the two occurences match for the purposes of
+        // check if the two occurrences match for the purposes of
         // de-duplication.
         return (ext.code == err.code) &&
                (ext.type == err.type) &&
@@ -708,12 +809,63 @@ bool ErrorReporter::ErrorReporterPrivate::AggregateReport::addOccurence(const Er
     return true;
 }
 
+bool ErrorReporter::ErrorReporterPrivate::isFGDataPath(const std::string& path)
+{
+    for (const auto& dp : globals->get_data_paths()) {
+        if (path.find(dp.utf8Str()) == 0) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool ErrorReporter::ErrorReporterPrivate::AggregateReport::allOccurences(OcurrencePredicate p) const
+{
+    auto it = std::find_if(errors.begin(), errors.end(), [p](const ErrorOcurrence& ext) {
+        return !p(ext);
+    });
+
+    return it == errors.end();
+}
+
 bool ErrorReporter::ErrorReporterPrivate::isMainAircraftPath(const std::string& path) const
 {
     const auto pos = path.find(_aircraftDirectoryName);
     return pos != std::string::npos;
 }
 
+/**
+ * @brief helper to determine if a file looks like it belongs to an aircraft.
+ *
+ * This is used to detect potential cross-aircraft packaging bugs which often lead to
+ * 'not found' error reports. So we're looking for paths which look plausibly like any
+ * aircraft path, even though @ref isMainAircraftPath didn't match on them.
+ */
+bool ErrorReporter::ErrorReporterPrivate::isAnyAircraftPath(const std::string& path) const
+{
+    const auto pos = path.find("Aircraft/");
+    if (pos == 0) {
+        // relative path which starts with "Aircraft/"
+        return true;
+    }
+
+    // TODO: decide if we also include paths which contain 'Aircraft' as a component
+    return false;
+}
+
+bool ErrorReporter::ErrorReporterPrivate::reportIsOngoing(const AggregateReport& report) const
+{
+    // don't wait for these to end, since they might continue forever
+    if ((report.type == Aggregation::NetworkFailure) || (report.type == Aggregation::OutOfMemory)) {
+        return false;
+    }
+
+    SGTimeStamp n = SGTimeStamp::now();
+    const auto ageSec = (n - report.lastErrorTime).toSecs();
+    return (ageSec < NoNewErrorsTimeout);
+}
+
 
 ////////////////////////////////////////////
 
@@ -759,6 +911,10 @@ void ErrorReporter::bind()
     SGPropertyNode_ptr n = fgGetNode("/sim/error-report", true);
 
     d->_enabledNode = n->getNode("enabled", true);
+    if (!d->_enabledNode->hasValue()) {
+        d->_enabledNode->setBoolValue(false); // default to off for now
+    }
+
     d->_displayNode = n->getNode("display", true);
     d->_activeErrorNode = n->getNode("active", true);
     d->_mpReportNode = n->getNode("mp-report-enabled", true);
@@ -784,6 +940,13 @@ void ErrorReporter::preinit()
 
     sglog().addCallback(d->_logCallback.get());
     d->_logCallbackRegistered = true;
+
+    // cache these values here
+    d->_fgdataPathPrefix = globals->get_fg_root().utf8Str();
+    d->_terrasyncPathPrefix = globals->get_terrasync_dir().utf8Str();
+
+    const auto aircraftPath = SGPath::fromUtf8(fgGetString("/sim/aircraft-dir"));
+    d->_aircraftDirectoryName = aircraftPath.file();
 }
 
 void ErrorReporter::init()
@@ -795,27 +958,17 @@ void ErrorReporter::init()
     const auto disableInDeveloperMode = !d->_enabledNode->getParent()->getBoolValue("enable-in-developer-mode");
     const auto dd = developerMode && disableInDeveloperMode;
 
-    if (dd || !d->_enabledNode) {
-        SG_LOG(SG_GENERAL, SG_INFO, "Error reporting disabled");
-        simgear::setFailureCallback(simgear::FailureCallback());
-        simgear::setErrorContextCallback(simgear::ContextCallback());
-        if (d->_logCallbackRegistered) {
-            sglog().removeCallback(d->_logCallback.get());
-            d->_logCallbackRegistered = false;
-        }
-        return;
+    if (dd || !d->_enabledNode->getBoolValue()) {
+        d->_isEnabled = false;
+    } else {
+        d->_isEnabled = true;
     }
 
+    // most work is done in preinit(), because errors can occur early in startup, before
+    // subsystems are being initialized
     globals->get_commands()->addCommand("dismiss-error-report", d.get(), &ErrorReporterPrivate::dismissReportCommand);
     globals->get_commands()->addCommand("save-error-report-data", d.get(), &ErrorReporterPrivate::saveReportCommand);
     globals->get_commands()->addCommand("show-error-report", d.get(), &ErrorReporterPrivate::showErrorReportCommand);
-
-    // cache these values here
-    d->_fgdataPathPrefix = globals->get_fg_root().utf8Str();
-    d->_terrasyncPathPrefix = globals->get_terrasync_dir().utf8Str();
-
-    const auto aircraftPath = SGPath::fromUtf8(fgGetString("/sim/aircraft-dir"));
-    d->_aircraftDirectoryName = aircraftPath.file();
 }
 
 void ErrorReporter::update(double dt)
@@ -823,21 +976,23 @@ void ErrorReporter::update(double dt)
     bool showDialog = false;
     bool showPopup = false;
     bool havePendingReports = false;
+    bool showMessage = false;
 
     // beginning of locked section
     {
         std::lock_guard<std::mutex> g(d->_lock);
+
         if (!d->_enabledNode->getBoolValue()) {
             return;
         }
 
-        // we are into the update phase (postinit has ocurred). We treat errors
+        // we are into the update phase (postinit has occurred). We treat errors
         // after this point with lower severity, to avoid popups into a flight
         d->_haveDonePostInit = true;
 
         SGTimeStamp n = SGTimeStamp::now();
 
-        // ensure we pause between successive error dialogs
+        // ensure we delay between successive error dialogs
         const auto timeSinceLastDialog = (n - d->_nextShowTimeout).toSecs();
         if (timeSinceLastDialog < MinimumIntervalBetweenDialogs) {
             return;
@@ -857,7 +1012,15 @@ void ErrorReporter::update(double dt)
         for (auto& report : d->_aggregated) {
             if (report.type == Aggregation::MultiPlayer) {
                 if (!d->_mpReportNode->getBoolValue()) {
-                    // mark it as shown, to supress it
+                    // mark it as shown, to suppress it
+                    report.haveShownToUser = true;
+                }
+            }
+
+            // if we have shown the special 'missing shared models' warning,
+            // don't also report it here.
+            if (report.type == Aggregation::SharedModels) {
+                if (flightgear::haveShownSharedModelsError()) {
                     report.haveShownToUser = true;
                 }
             }
@@ -867,22 +1030,26 @@ void ErrorReporter::update(double dt)
                 continue;
             }
 
-            const auto ageSec = (n - report.lastErrorTime).toSecs();
-            if (ageSec > NoNewErrorsTimeout) {
-                d->presentErrorToUser(report);
-                if (report.isCritical) {
-                    showDialog = true;
-                } else {
-                    showPopup = true;
-                }
-
-                d->sendReportToSentry(report);
-
-                // if we show one report, don't consider any others for now
-                break;
-            } else {
+            if (d->reportIsOngoing(report)) {
                 havePendingReports = true;
+                continue;
             }
+
+            d->presentErrorToUser(report);
+            if (report.isCritical) {
+                showDialog = true;
+            } else if (report.isMessage) {
+                showMessage = true;
+            } else {
+                showPopup = true;
+            }
+
+            if (d->_isEnabled) {
+                d->sendReportToSentry(report);
+            }
+
+            // if we show one report, don't consider any others for now
+            break;
         } // of active aggregates iteration
 
         if (!havePendingReports) {
@@ -890,15 +1057,20 @@ void ErrorReporter::update(double dt)
         }
     } // end of locked section
 
-    if (flightgear::isHeadlessMode()) {
+    if (!d->_isEnabled || flightgear::isHeadlessMode()) {
         showDialog = false;
         showPopup = false;
+        showMessage = false;
     }
 
-// don't show any error reporting UI on stable, for the moment.
-    showDialog = false;
-    showPopup = false;
-
+    // show messages: these are local configuration pieces
+    // so always warn them to the user, eg DNS/network failure, out-of-memory, or
+    // missing shared models.
+    if (showMessage) {
+        SGPropertyNode_ptr popupArgs(new SGPropertyNode);
+        popupArgs->setStringValue("message", d->_displayNode->getStringValue("category"));
+        globals->get_commands()->execute("show-error-message-popup", popupArgs, nullptr);
+    }
 
     // do not call into another subsystem with our lock held,
     // as this can trigger deadlocks
@@ -912,7 +1084,7 @@ void ErrorReporter::update(double dt)
         pauseArgs->setBoolValue("force-pause", true);
         globals->get_commands()->execute("do_pause", pauseArgs);
 #endif
-    } else if (showPopup) {
+    } else if (showPopup && d->_popupEnabled) {
         SGPropertyNode_ptr popupArgs(new SGPropertyNode);
         popupArgs->setIntValue("index", d->_activeReportIndex);
         globals->get_commands()->execute("show-error-notification-popup", popupArgs, nullptr);
@@ -921,18 +1093,16 @@ void ErrorReporter::update(double dt)
 
 void ErrorReporter::shutdown()
 {
-    if (d->_enabledNode) {
-        globals->get_commands()->removeCommand("dismiss-error-report");
-        globals->get_commands()->removeCommand("save-error-report-data");
-        globals->get_commands()->removeCommand("show-error-report");
+    globals->get_commands()->removeCommand("dismiss-error-report");
+    globals->get_commands()->removeCommand("save-error-report-data");
+    globals->get_commands()->removeCommand("show-error-report");
 
-// during a reset we don't want to touch the log callback; it was added in
-// preinit, which does not get repeated on a reset
-        const bool inReset = fgGetBool("/sim/signals/reinit", false);
-        if (!inReset) {
-            sglog().removeCallback(d->_logCallback.get());
-            d->_logCallbackRegistered = false;
-        }
+    // during a reset we don't want to touch the log callback; it was added in
+    // preinit, which does not get repeated on a reset
+    const bool inReset = fgGetBool("/sim/signals/reinit", false);
+    if (!inReset) {
+        sglog().removeCallback(d->_logCallback.get());
+        d->_logCallbackRegistered = false;
     }
 }
 
@@ -947,3 +1117,7 @@ std::string ErrorReporter::threadSpecificContextValue(const std::string& key)
 
 
 } // namespace flightgear
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<flightgear::ErrorReporter> registrantErrorReporter(
+    SGSubsystemMgr::GENERAL);
