@@ -1,25 +1,9 @@
-// fg_init.cxx -- Flight Gear top level initialization routines
-//
-// Written by Curtis Olson, started August 1997.
-//
-// Copyright (C) 1997  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
-
+/*
+ * SPDX-FileName: fg_init.cxx
+ * SPDX-FileComment: FlightGear top level initialization routines
+ * SPDX-FileCopyrightText: Copyright (C) 1997  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include <config.h>
 
@@ -60,6 +44,7 @@
 #include <simgear/embedded_resources/EmbeddedResourceManager.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
+#include <simgear/timing/sg_time.hxx>
 
 #include <simgear/scene/material/Effect.hxx>
 #include <simgear/scene/material/matlib.hxx>
@@ -67,6 +52,7 @@
 #include <simgear/scene/model/particles.hxx>
 #include <simgear/scene/tgdb/TreeBin.hxx>
 #include <simgear/scene/tgdb/userdata.hxx>
+#include <simgear/scene/tgdb/VPBTechnique.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
 
 #include <simgear/package/Root.hxx>
@@ -75,7 +61,6 @@
 #include <simgear/package/Catalog.hxx>
 
 #include <Add-ons/AddonManager.hxx>
-
 #include <Aircraft/controls.hxx>
 #include <Aircraft/replay.hxx>
 #include <Aircraft/FlightHistory.hxx>
@@ -93,11 +78,12 @@
 
 #include <Cockpit/panel.hxx>
 #include <Cockpit/panel_io.hxx>
-
+ 
 #include <Canvas/canvas_mgr.hxx>
 #include <Canvas/gui_mgr.hxx>
 #include <Canvas/FGCanvasSystemAdapter.hxx>
 #include <GUI/new_gui.hxx>
+#include <GUI/Highlight.hxx>
 #include <GUI/MessageBox.hxx>
 #include <Input/input.hxx>
 #include <Instrumentation/instrument_mgr.hxx>
@@ -124,22 +110,25 @@
 #if defined(ENABLE_SWIFT)
 #include <Network/Swift/swift_connection.hxx>
 #endif
-#include <FDM/fdm_shell.hxx>
-#include <Environment/ephemeris.hxx>
-#include <Environment/environment_mgr.hxx>
-#include <Viewer/renderer.hxx>
-#include <Viewer/viewmgr.hxx>
-#include <Viewer/FGEventHandler.hxx>
-#include <Navaids/NavDataCache.hxx>
-#include <Instrumentation/HUD/HUD.hxx>
 #include <Cockpit/cockpitDisplayManager.hxx>
-#include <Network/HTTPClient.hxx>
+#include <Environment/environment_mgr.hxx>
+#include <Environment/ephemeris.hxx>
+#include <FDM/fdm_shell.hxx>
+#ifdef ENABLE_HUD
+#include <Instrumentation/HUD/HUD.hxx>
+#endif
+#include <Navaids/NavDataCache.hxx>
 #include <Network/DNSClient.hxx>
+#include <Network/HTTPClient.hxx>
 #include <Network/fgcom.hxx>
 #include <Network/http/httpd.hxx>
-#include <Include/version.h>
-#include <Viewer/splash.hxx>
 #include <Viewer/CameraGroup.hxx>
+#include <Viewer/FGEventHandler.hxx>
+#include <Viewer/GraphicsPresets.hxx>
+#include <Viewer/VRManager.hxx>
+#include <Viewer/renderer.hxx>
+#include <Viewer/splash.hxx>
+#include <Viewer/viewmgr.hxx>
 
 #include "fg_init.hxx"
 #include "fg_io.hxx"
@@ -153,8 +142,12 @@
 #include "positioninit.hxx"
 #include "util.hxx"
 #include "AircraftDirVisitorBase.hxx"
-#include <Main/sentryIntegration.hxx>
+
 #include <Main/ErrorReporter.hxx>
+#include <Main/MultipleInstanceLock.hxx>
+#include <Main/sentryIntegration.hxx>
+
+#include "cJSON.h"
 
 #if defined(SG_MAC)
 #include <GUI/CocoaHelpers.h> // for Mac impl of platformDefaultDataPath()
@@ -188,6 +181,40 @@ string fgBasePackageVersion(const SGPath& base_path) {
     return version;
 }
 
+std::optional<FGBasePackageInfo> fgBasePackageInfo(const SGPath& path)
+{
+    SGPath p = path / "base_package.json";
+    if (!p.exists()) {
+        return {};
+    }
+
+    sg_ifstream in(p);
+    if (!in.is_open()) {
+        return {};
+    }
+
+    const auto content = in.read_all();
+    cJSON* json = cJSON_Parse(content.c_str());
+
+    FGBasePackageInfo r;
+    
+    auto ver = cJSON_GetObjectItem(json, "version");
+    if (ver) {
+        r.version = ver->valuestring;
+    }
+
+    auto suf = cJSON_GetObjectItem(json, "fgdata-suffix");
+    if (suf) {
+        r.suffix = suf->valuestring;
+    }
+
+    r.buildDate = cJSON_GetObjectItem(json, "build-date")->valuestring;
+    r.gitRevision = cJSON_GetObjectItem(json, "fgdata-sha")->valuestring;
+
+    cJSON_Delete(json);
+    return r;
+}
+
 class FindAndCacheAircraft : public AircraftDirVistorBase
 {
 public:
@@ -195,6 +222,11 @@ public:
   {
     _cache = autoSave->getNode("sim/startup/path-cache", true);
   }
+
+  void setDidUseLauncher(bool didUseLauncher)
+  {
+      _didUseLauncher = didUseLauncher;
+  }
   
   /**
    * @brief haveExplicitAircraft - check if the combination of /sim/aircraft
@@ -291,7 +323,10 @@ public:
       SG_LOG(SG_GENERAL, SG_ALERT, "\tin paths:" << SGPath::join(globals->get_aircraft_paths(), ";"));
 
       std::string notFoundMessage;
-      bool reportToSentry = true;
+      // don't report failures where the launcher was not used, to Sentry,
+      // since they are nearly all configuration problems.
+      bool reportToSentry = _didUseLauncher;
+
       if (globals->get_aircraft_paths().empty()) {
           notFoundMessage = "The requested aircraft (" + aircraft + ") could not be found. No aircraft paths are configured.";
           reportToSentry = false; // no need to log these
@@ -352,9 +387,9 @@ private:
       return false; // cache mismatch
     }
     
-    vector<SGPropertyNode_ptr> cache = _cache->getChildren("aircraft");
+    std::vector<SGPropertyNode_ptr> cache = _cache->getChildren("aircraft");
     for (unsigned int i = 0; i < cache.size(); i++) {
-      const char *name = cache[i]->getStringValue("file", "");
+      const std::string name = cache[i]->getStringValue("file", "");
       if (!simgear::strutils::iequals(_searchAircraft, name)) {
         continue;
       }
@@ -472,7 +507,8 @@ private:
 
   std::string _searchAircraft;
   SGPath _foundPath;
-  SGPropertyNode* _cache;
+  SGPropertyNode* _cache = nullptr;
+  bool _didUseLauncher = false;
 };
 
 #ifdef _WIN32
@@ -534,7 +570,8 @@ InitHomeResult fgInitHome()
         SG_LOG(SG_GENERAL, SG_INFO, "Running with FG_HOME readonly");
         return InitHomeExplicitReadOnly;
     }
-    
+
+    // todo: move this code into MultipleInstanceLock.cxx
     InitHomeResult result = InitHomeOkay;
 #if defined(SG_WINDOWS)
 	// don't use a PID file on Windows, because deleting on close is
@@ -544,11 +581,11 @@ InitHomeResult fgInitHome()
 	static_fgHomeWriteMutex = CreateMutexA(nullptr, FALSE, "org.flightgear.fgfs.primary");
 	if (static_fgHomeWriteMutex == nullptr) {
 		printf("CreateMutex error: %d\n", GetLastError());
-		SG_LOG(SG_GENERAL, SG_POPUP, "Failed to create mutex for multi-app protection");
+		SG_LOG(SG_GENERAL, SG_ALERT, "Failed to create mutex for multi-app protection");
         return InitHomeAbort;
 	} else if (GetLastError() == ERROR_ALREADY_EXISTS) {
-		SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
-		fgSetBool("/sim/fghome-readonly", true);
+        SG_LOG(SG_GENERAL, SG_MANDATORY_INFO, "flightgear instance already running, switching to FG_HOME read-only.");
+        fgSetBool("/sim/fghome-readonly", true);
         return InitHomeReadOnly;
 	} else {
 		SG_LOG(SG_GENERAL, SG_INFO, "Created multi-app mutex, we are in writeable mode");
@@ -615,6 +652,11 @@ InitHomeResult fgInitHome()
         result = InitHomeOkay;
     }
 #endif
+    // now we know we're the main copy, take an exclusive lock until we know FGData / nav-cache are correct
+    // this will cause either the launcher init or the splash screen to block in read-only copies, until
+    // we release the exclusive lock after nav-cache build is done.
+    flightgear::ExclusiveInstanceLock::createInstance("init-home");
+
     fgSetBool("/sim/fghome-readonly", false);
     return result;
 }
@@ -708,9 +750,9 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
     }
 
     // allow command line to override
-    if (options->isOptionSet("developer")) {
-        string s = options->valueForOption("developer", "yes");
-        developerMode = simgear::strutils::to_bool(s);
+    const simgear::optional<bool> devOption = options->checkBoolOptionSet("developer");
+    if (devOption.has_value()) { // User specified the developer option
+        developerMode = devOption.value();
     }
 
     auto node = fgGetNode("/sim/developer-mode", true);
@@ -785,7 +827,7 @@ void fgInitAircraftPaths(bool reinit)
   }
 }
 
-int fgInitAircraft(bool reinit)
+int fgInitAircraft(bool reinit, bool didUseLauncher)
 {
     if (!reinit) {
         auto r = flightgear::Options::sharedInstance()->initAircraft();
@@ -794,13 +836,18 @@ int fgInitAircraft(bool reinit)
     }
     
     FindAndCacheAircraft f(globals->get_props());
+    f.setDidUseLauncher(didUseLauncher);
     const bool haveExplicit = f.haveExplicitAircraft();
-    if (haveExplicit) {
-        flightgear::addSentryBreadcrumb("Have explicit aircraft-dr and -set.xml", "info");
-    }
 
     SGSharedPtr<Root> pkgRoot(globals->packageRoot());
     SGPropertyNode* aircraftProp = fgGetNode("/sim/aircraft", true);
+    SGPropertyNode* aircraftDirProp = fgGetNode("/sim/aircraft-dir", true);
+
+    // ensure aircraft-dir survives reset, so we find the same aircraft
+    // after a reset as we did prior to it. Without this, aircraft
+    // can unintentionally change on a reset
+    aircraftDirProp->setAttribute(SGPropertyNode::PRESERVE, true);
+
     const string fullyQualifiedAircraftId = fgGetString("/sim/aircraft-id");
     string aircraftId = fullyQualifiedAircraftId.empty() ? aircraftProp->getStringValue() : fullyQualifiedAircraftId;
 
@@ -813,9 +860,7 @@ int fgInitAircraft(bool reinit)
 
     if (acftPackage) {
         if (acftPackage->isInstalled()) {
-            flightgear::addSentryBreadcrumb("Loading aircraft from installed package", "info");
             SG_LOG(SG_GENERAL, SG_INFO, "Loading aircraft from package:" << acftPackage->qualifiedId());
-            
             // if we resolved a non-qualified ID, set the full one back to /sim/aircraft-id
             fgSetString("/sim/aircraft-id", acftPackage->qualifiedId());
 
@@ -830,8 +875,6 @@ int fgInitAircraft(bool reinit)
             InstallRef acftInstall = acftPackage->install();
             fgSetString("/sim/aircraft-dir", acftInstall->path().utf8Str());
 
-            flightgear::addSentryBreadcrumb("Computed aircraft-dir as:" + acftInstall->path().utf8Str(), "info");
-
             // overwrite the fully qualified ID with the aircraft one, so the
             // code in FindAndCacheAircraft works as normal
             // note since we may be using a variant, we can't use the package ID
@@ -843,7 +886,6 @@ int fgInitAircraft(bool reinit)
 
             // run the traditional-code path below
         } else {
-            flightgear::addSentryBreadcrumb("Package is not installed:" + aircraftId, "info");
 #if 0
             // naturally the better option would be to on-demand install it!
             flightgear::fatalMessageBoxWithoutExit(
@@ -974,247 +1016,191 @@ void fgOutputSettings()
 // Returns non-zero if a problem encountered.
 void fgCreateSubsystems(bool duringReset) {
 
-    SG_LOG( SG_GENERAL, SG_INFO, "Creating Subsystems");
-    SG_LOG( SG_GENERAL, SG_INFO, "======== ==========");
+    SG_LOG( SG_GENERAL, SG_INFO, "== Creating Subsystems");
 
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the sound subsystem.
-    ////////////////////////////////////////////////////////////////////
-    // Sound manager uses an own subsystem group "SOUND" which is the last
-    // to be updated in every loop.
-    // Sound manager is updated last so it can use the CPU while the GPU
-    // is processing the scenery (doubled the frame-rate for me) -EMH-
-    globals->add_new_subsystem<FGSoundManager>(SGSubsystemMgr::SOUND);
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the event manager subsystem.
-    ////////////////////////////////////////////////////////////////////
+    // Fetch the subsystem manager.
+    auto mgr = globals->get_subsystem_mgr();
 
     globals->get_event_mgr()->init();
     globals->get_event_mgr()->setRealtimeProperty(fgGetNode("/sim/time/delta-realtime-sec", true));
 
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the property interpolator subsystem. Put into the INIT
-    // group because the "nasal" subsystem may need it at GENERAL take-down.
-    ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("prop-interpolator", new FGInterpolator, SGSubsystemMgr::INIT);
-
-
-    ////////////////////////////////////////////////////////////////////
-    // Add the FlightGear property utilities.
-    ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("properties", new FGProperties);
-
-
-    ////////////////////////////////////////////////////////////////////
-    // Add the FlightGear property utilities.
-    ////////////////////////////////////////////////////////////////////
-    globals->add_new_subsystem<flightgear::AirportDynamicsManager>();
-
-    ////////////////////////////////////////////////////////////////////
-    // Add the performance monitoring system.
-    ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("performance-mon",
-            new SGPerformanceMonitor(globals->get_subsystem_mgr(),
-                                     fgGetNode("/sim/performance-monitor", true)));
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the material property subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    SGPath mpath( globals->get_fg_root() );
-    mpath.append( fgGetString("/sim/rendering/materials-file") );
-    if ( ! globals->get_matlib()->load(globals->get_fg_root(), mpath, globals->get_props()) ) {
-       throw sg_io_exception("Error loading materials file", mpath);
-    }
-
-    // may exist already due to GUI startup
-    if (!globals->get_subsystem<FGHTTPClient>()) {
-        globals->add_new_subsystem<FGHTTPClient>();
-    }
-    globals->add_new_subsystem<FGDNSClient>();
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the flight model subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("flight", new FDMShell, SGSubsystemMgr::FDM);
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the weather subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    // Initialize the weather modeling subsystem
-    globals->add_subsystem("environment", new FGEnvironmentMgr);
-    globals->add_new_subsystem<Ephemeris>();
-    
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the aircraft systems and instrumentation (before the
-    // autopilot.)
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("systems", new FGSystemMgr, SGSubsystemMgr::FDM);
-    globals->add_subsystem("instrumentation", new FGInstrumentMgr, SGSubsystemMgr::FDM);
-    globals->add_subsystem("hud", new HUD, SGSubsystemMgr::DISPLAY);
-    globals->add_subsystem("cockpit-displays", new flightgear::CockpitDisplayManager, SGSubsystemMgr::DISPLAY);
-  
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the XML Autopilot subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem( "xml-autopilot", FGXMLAutopilotGroup::createInstance("autopilot"), SGSubsystemMgr::FDM );
-    globals->add_subsystem( "xml-proprules", FGXMLAutopilotGroup::createInstance("property-rule"), SGSubsystemMgr::GENERAL );
-    globals->add_new_subsystem<FGRouteMgr>();
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the Input-Output subsystem
-    ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem( "io", new FGIO );
-  
-    ////////////////////////////////////////////////////////////////////
-    // Create and register the logger.
-    ////////////////////////////////////////////////////////////////////
-    
-    globals->add_subsystem("logger", new FGLogger);
-
-    ////////////////////////////////////////////////////////////////////
-    // Create and register the XML GUI.
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("gui", new NewGUI, SGSubsystemMgr::INIT);
-
-    //////////////////////////////////////////////////////////////////////
-    // Initialize the 2D cloud subsystem.
-    ////////////////////////////////////////////////////////////////////
-    fgGetBool("/sim/rendering/bump-mapping", false);
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the canvas 2d drawing subsystem.
-    ////////////////////////////////////////////////////////////////////
-    simgear::canvas::Canvas::setSystemAdapter(
-      simgear::canvas::SystemAdapterPtr(new canvas::FGCanvasSystemAdapter)
-    );
-    globals->add_subsystem("Canvas", new CanvasMgr, SGSubsystemMgr::DISPLAY);
-    globals->add_subsystem("CanvasGUI", new GUIMgr, SGSubsystemMgr::DISPLAY);
-
-    ////////////////////////////////////////////////////////////////////
-   // Initialize the ATC subsystem
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_new_subsystem<PerformanceDB>(SGSubsystemMgr::POST_FDM);
-    globals->add_subsystem("ATC", new FGATCManager, SGSubsystemMgr::POST_FDM);
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize multiplayer subsystem
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("mp", new FGMultiplayMgr, SGSubsystemMgr::POST_FDM);
-
-#ifdef ENABLE_SWIFT
-    ////////////////////////////////////////////////////////////////////
-    // Initialize Swift subsystem
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("swift", new SwiftConnection, SGSubsystemMgr::POST_FDM);
-#endif
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialise the AI Model Manager
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_subsystem("ai-model", new FGAIManager, SGSubsystemMgr::POST_FDM);
-    globals->add_subsystem("submodel-mgr", new FGSubmodelMgr, SGSubsystemMgr::POST_FDM);
-
-
-    // It's probably a good idea to initialize the top level traffic manager
-    // After the AI and ATC systems have been initialized properly.
-    // AI Traffic manager
-    globals->add_subsystem("traffic-manager", new FGTrafficManager, SGSubsystemMgr::POST_FDM);
-
-    ////////////////////////////////////////////////////////////////////
-    // Add a new 2D panel.
-    ////////////////////////////////////////////////////////////////////
-
-    fgSetArchivable("/sim/panel/visibility");
-    fgSetArchivable("/sim/panel/x-offset");
-    fgSetArchivable("/sim/panel/y-offset");
-    fgSetArchivable("/sim/panel/jitter");
-  
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the controls subsystem.
-    ////////////////////////////////////////////////////////////////////
-    
-    globals->add_new_subsystem<FGControls>(SGSubsystemMgr::GENERAL);
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the input subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    globals->add_new_subsystem<FGInput>(SGSubsystemMgr::GENERAL);
-
-
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the replay subsystem
-    ////////////////////////////////////////////////////////////////////
-    globals->add_new_subsystem<FGReplay>(SGSubsystemMgr::GENERAL);
-    globals->add_subsystem("history", new FGFlightHistory);
-    
-#ifdef ENABLE_AUDIO_SUPPORT
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the sound-effects subsystem.
-    ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("voice", new FGVoiceMgr, SGSubsystemMgr::DISPLAY);
-#endif
-
-#ifdef ENABLE_IAX
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the FGCom subsystem.
-    // very important this goes in the SOUND group, since IAXClient
-    // depends on OpenAL, which is shutdown when the SOUND group
-    // shutdown.
-    // Sentry: FLIGHTGEAR-66
-
-    ////////////////////////////////////////////////////////////////////
-    globals->add_new_subsystem<FGCom>(SGSubsystemMgr::SOUND);
-#endif
-
+    // SGSubsystemMgr::INIT
     {
-      SGSubsystem * httpd = flightgear::http::FGHttpd::createInstance( fgGetNode(flightgear::http::PROPERTY_ROOT) );
-      if( NULL != httpd ) 
-        globals->add_subsystem("httpd", httpd  );
+        // Initialize the property interpolator subsystem. Put into the INIT
+        // group because the "nasal" subsystem may need it at GENERAL take-down.
+        mgr->add<FGInterpolator>();
+        mgr->add<Highlight>();
+        mgr->add<NewGUI>();
     }
 
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the lighting subsystem.
-    ////////////////////////////////////////////////////////////////////
+    // SGSubsystemMgr::GENERAL
+    {
+        mgr->add<FGProperties>();
+        mgr->add<flightgear::AirportDynamicsManager>();
+        mgr->add("performance-mon",
+                new SGPerformanceMonitor(
+                        mgr,
+                        fgGetNode("/sim/performance-monitor", true)
+                        )
+                );
 
-    // ordering here is important : Nasal (via events), then models, then views
-    if (!duringReset) {
-        globals->add_subsystem("lighting", new FGLight, SGSubsystemMgr::DISPLAY);
-        globals->add_subsystem("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
-        globals->add_subsystem("tides", new FGTide );
+        // Initialize the material property subsystem.
+        SGPath mpath( globals->get_fg_root() );
+        mpath.append( fgGetString("/sim/rendering/materials-file") );
+        if ( ! globals->get_matlib()->load(globals->get_fg_root(), mpath, globals->get_props()) ) {
+           throw sg_io_exception("Error loading materials file", mpath);
+        }
+
+        // may exist already due to GUI startup or --load-tape=http...
+        if (!mgr->get_subsystem<FGHTTPClient>()) {
+            mgr->add<FGHTTPClient>();
+        }
+        mgr->add<FGDNSClient>();
+
+        // Initialize the weather modeling subsystem
+        mgr->add<FGEnvironmentMgr>();
+        mgr->add<Ephemeris>();
+        mgr->add("xml-proprules", FGXMLAutopilotGroup::createInstance("property-rule"));
+
+        mgr->add<FGRouteMgr>();
+        mgr->add<FGIO>();
+        mgr->add<FGLogger>();
+
+        mgr->add<FGControls>();
+        mgr->add<FGInput>();
+        mgr->add<FGFlightHistory>();
+
+        {
+          SGSubsystem * httpd = flightgear::http::FGHttpd::createInstance( fgGetNode(flightgear::http::PROPERTY_ROOT) );
+          if( NULL != httpd ) 
+            mgr->add("httpd", httpd);
+        }
+
+        if (!duringReset) {
+            mgr->add<FGTide>();
+        }
     }
+    
+    // SGSubsystemMgr::FDM
+    {
+        mgr->add<FDMShell>();
 
-    globals->add_new_subsystem<FGAircraftModel>(SGSubsystemMgr::DISPLAY);
-    globals->add_new_subsystem<FGModelMgr>(SGSubsystemMgr::DISPLAY);
+        // Initialize the aircraft systems and instrumentation (before the
+        // autopilot.)
+        mgr->add<FGSystemMgr>();
+        mgr->add<FGInstrumentMgr>();
+        mgr->add("xml-autopilot", FGXMLAutopilotGroup::createInstance("autopilot"), SGSubsystemMgr::FDM);
+    }
+    
+    // SGSubsystemMgr::POST_FDM
+    {
+        mgr->add<PerformanceDB>();
+        mgr->add<FGATCManager>();
+        mgr->add<FGAIManager>();
+        mgr->add<FGMultiplayMgr>();
 
-    globals->add_new_subsystem<FGViewMgr>(SGSubsystemMgr::DISPLAY);
+        #ifdef ENABLE_SWIFT
+        mgr->add<flightgear::swift::SwiftConnection>();
+        #endif
+
+        // FGReplay.
+        //
+        // FGReplay is after FGMultiplayMgr, so that it can record the most
+        // recent MP packets.
+        //
+        // FGReplay is before FGAIManager so that, when replaying, it can push
+        // MP packets into FGMultiplayMgr's queue before FGAIManager reads from
+        // this queue.
+        //
+        // We also call FGReplay::init() method here, to work around a
+        // problem where JSBSim appears to rely on FGReplay creating certain
+        // properties before it is initialised. This caused problems when
+        // FGReplay was changed to be POST_FDM.
+        mgr->add<FGReplay>();
+        mgr->get_subsystem<FGReplay>()->init(); // Special case.
+
+        //mgr->add<FGAIManager>();
+        mgr->add<FGSubmodelMgr>();
+
+        // It's probably a good idea to initialize the top level traffic manager
+        // After the AI and ATC systems have been initialized properly.
+        // AI Traffic manager
+        mgr->add<FGTrafficManager>();
+
+        fgSetArchivable("/sim/panel/visibility");
+        fgSetArchivable("/sim/panel/x-offset");
+        fgSetArchivable("/sim/panel/y-offset");
+        fgSetArchivable("/sim/panel/jitter");
+    }
+    
+    // SGSubsystemMgr::DISPLAY
+    {
+#ifdef ENABLE_HUD
+        mgr->add<HUD>();
+#endif
+        mgr->add<flightgear::CockpitDisplayManager>();
+
+        simgear::canvas::Canvas::setSystemAdapter(
+          simgear::canvas::SystemAdapterPtr(new canvas::FGCanvasSystemAdapter)
+        );
+        mgr->add<CanvasMgr>();
+        
+        auto canvasGui = new GUIMgr;
+        mgr->add("CanvasGUI", canvasGui, SGSubsystemMgr::DISPLAY);
+        auto guiCamera = flightgear::getGUICamera(flightgear::CameraGroup::getDefault());
+        canvasGui->setGUIViewAndCamera(globals->get_renderer()->getView(), guiCamera);
+
+        #ifdef ENABLE_AUDIO_SUPPORT
+        mgr->add<FGVoiceMgr>();
+        #endif
+
+        // ordering here is important : Nasal (via events), then models, then views
+        if (!duringReset) {
+            mgr->add<FGLight>();
+            mgr->add("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
+        }
+
+        mgr->add<FGAircraftModel>();
+        mgr->add<FGModelMgr>();
+        mgr->add<FGViewMgr>();
+    }
+    
+    // SGSubsystemMgr::SOUND
+    {
+        // Sound manager uses an own subsystem group "SOUND" which is the last
+        // to be updated in every loop.
+        // Sound manager is updated last so it can use the CPU while the GPU
+        // is processing the scenery (doubled the frame-rate for me) -EMH-
+        mgr->add<FGSoundManager>();
+
+        #ifdef ENABLE_IAX
+        // Initialize the FGCom subsystem.
+        // very important this goes in the SOUND group, since IAXClient
+        // depends on OpenAL, which is shutdown when the SOUND group
+        // shutdown.
+        // Sentry: FLIGHTGEAR-66
+        mgr->add<FGCom>();
+        #endif
+    }
 }
 
 void fgPostInitSubsystems()
 {
     SGTimeStamp st;
     st.stamp();
-  
+
+    // Fetch the subsystem manager.
+    auto mgr = globals->get_subsystem_mgr();
+
     ////////////////////////////////////////////////////////////////////////
     // Initialize the Nasal interpreter.
     // Do this last, so that the loaded scripts see initialized state
     ////////////////////////////////////////////////////////////////////////
-    globals->add_new_subsystem<FGNasalSys>(SGSubsystemMgr::INIT);
+    mgr->add<FGNasalSys>();
 
     // initialize methods that depend on other subsystems.
     st.stamp();
-    globals->get_subsystem_mgr()->postinit();
+    mgr->postinit();
     SG_LOG(SG_GENERAL, SG_INFO, "Subsystems postinit took:" << st.elapsedMSec());
   
     ////////////////////////////////////////////////////////////////////////
@@ -1247,52 +1233,60 @@ void fgStartReposition()
   // set this signal so Nasal scripts can take action.
   fgSetBool("/sim/signals/reinit", true);
   fgSetBool("/sim/crashed", false);
-  
-  FDMShell* fdm = globals->get_subsystem<FDMShell>();
-  fdm->unbind();
-  
+
+  // Fetch the subsystem manager.
+  auto mgr = globals->get_subsystem_mgr();
+
+  mgr->get_subsystem<FDMShell>()->unbind();
+
   // update our position based on current presets
   // this will mark position as needed finalized which we'll do in the
   // main-loop
   flightgear::initPosition();
   
-  auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
+  auto terraSync = mgr->get_subsystem<simgear::SGTerraSync>();
   if (terraSync) {
     terraSync->reposition();
   }
   
   // Initialize the FDM
-  globals->get_subsystem<FDMShell>()->reinit();
+  mgr->get_subsystem<FDMShell>()->reinit();
   
   // reset replay buffers
-  globals->get_subsystem<FGReplay>()->reinit();
-  
+  mgr->get_subsystem<FGReplay>()->reinit();
+
   // ugly: finalizePosition waits for METAR to arrive for the new airport.
   // we don't re-init the environment manager here, since historically we did
   // not, and doing so seems to have other issues. All that's needed is to
   // schedule METAR fetch immediately, so it's available for finalizePosition.
   // So we manually extract the METAR-fetching component inside the environment
   // manager, and re-init that.
-  SGSubsystemGroup* envMgr = static_cast<SGSubsystemGroup*>(globals->get_subsystem("environment"));
+  auto envMgr = static_cast<SGSubsystemGroup*>(mgr->get_subsystem<FGEnvironmentMgr>());
   if (envMgr) {
     envMgr->get_subsystem("realwx")->reinit();
   }
   
     // needed for parking assignment to work after reposition
-    auto atcManager = globals->get_subsystem<FGATCManager>();
+    auto atcManager = mgr->get_subsystem<FGATCManager>();
     if (atcManager) {
         atcManager->reposition();
     }
 
   // need to bind FDMshell again
-  fdm->bind();
+  mgr->get_subsystem<FDMShell>()->bind();
 
   // need to reset aircraft (systems/instruments/autopilot)
   // so they can adapt to current environment
-  globals->get_subsystem("systems")->reinit();
-  globals->get_subsystem("instrumentation")->reinit();
-  globals->get_subsystem("xml-autopilot")->reinit();
-    
+  mgr->get_subsystem<FGSystemMgr>()->reinit();
+  mgr->get_subsystem<FGInstrumentMgr>()->reinit();
+  mgr->get_subsystem("xml-autopilot")->reinit();
+
+  // need to update the timezone
+  auto timeManager = mgr->get_subsystem<TimeManager>();
+  if (timeManager) {
+      timeManager->reposition();
+  }
+
   // setup state to end re-init
   fgSetBool("/sim/signals/reinit", false);
   if ( !freeze ) {
@@ -1322,7 +1316,7 @@ void fgStartNewReset()
     // pause the osgDB requests right now, but more may appear; we will
     // clear and cancel further down once shutdown has occured
     FGRenderer* render = globals->get_renderer();
-    auto pager = render->getViewer()->getDatabasePager();
+    auto pager = render->getView()->getDatabasePager();
     pager->setAcceptNewDatabaseRequests(false);
     pager->cancel();
     
@@ -1339,12 +1333,15 @@ void fgStartNewReset()
     subsystemManger->remove("nasal");
     
     subsystemManger->shutdown();
+#ifdef ENABLE_OSGXR
+    flightgear::VRManager::destroyInstance();
+#endif
     subsystemManger->unbind();
 
     // hack fix for many reset crashes relating to the static instance
     // of this class. Will be fixed better for future versions by making
     // this a proper subsystem.
-    FGATCDialogNew::hackyReset();
+    //FGATCDialogNew::hackyReset();
     
     // remove most subsystems, with a few exceptions.
     for (int g=0; g<SGSubsystemMgr::MAX_GROUPS; ++g) {
@@ -1375,14 +1372,21 @@ void fgStartNewReset()
     flightgear::NavDataCache::instance()->clearDynamicPositioneds();
 
     // needed or we crash in multi-threaded OSG mode
-    render->getViewer()->stopThreading();
-    
+    render->getViewerBase()->stopThreading();
+
+    osg::ref_ptr<osgViewer::CompositeViewer> composite_viewer = render->getCompositeViewer();
+    osg::ref_ptr<osgViewer::View> composite_viewer_view;
+    if (composite_viewer) {
+        composite_viewer_view = render->getView();
+    }
+        
     // order is important here since tile-manager shutdown needs to
     // access the scenery object
     subsystemManger->remove(FGScenery::staticSubsystemClassId());
 
     FGScenery::getPagerSingleton()->clearRequests();
     flightgear::CameraGroup::setDefault(NULL);
+
     
     osgDB::Registry::instance()->clearObjectCache();
     // Pager requests depend on this, so don't clear it until now
@@ -1398,10 +1402,10 @@ void fgStartNewReset()
     globals->set_matlib(NULL);
     
     flightgear::unregisterMainLoopProperties();
-    FGReplayData::resetStatisticsProperties();
+    FGReplay::resetStatisticsProperties();
 
-    simgear::clearSharedTreeGeometry();
     simgear::clearEffectCache();
+    simgear::VPBTechnique::clearConstraints();
     simgear::SGModelLib::resetPropertyRoot();
     simgear::ParticlesGlobalManager::clear();
     simgear::UniformFactory::instance()->reset();    
@@ -1454,26 +1458,56 @@ void fgStartNewReset()
                "Some errors restoring preserved state (read-only props?)" );
     }
 
-    fgGetNode("/sim")->removeChild("aircraft-dir");
     fgInitAircraftPaths(true);
-    fgInitAircraft(true);
-    
-    render = new FGRenderer;
+    fgInitAircraft(true, false /* not from launcher */);
+
+    auto presets = globals->get_subsystem_mgr()->add<flightgear::GraphicsPresets>();
+    presets->applyInitialPreset();
+
+    render = new FGRenderer(composite_viewer);
     render->setEventHandler(eventHandler);
     eventHandler->reset();
     globals->set_renderer(render);
     render->init();
-    render->setViewer(viewer.get());
+    
+    if (composite_viewer) {
+        render->setView(composite_viewer_view);
+    }
+    else {
+        render->setView(viewer.get());
+    }
 
     sgUserDataInit( globals->get_props() );
 
-    viewer->getDatabasePager()->setUpThreads(2, 1);
-    viewer->getDatabasePager()->setAcceptNewDatabaseRequests(true);
+    unsigned int numDBPagerThreads = std::max(fgGetNode("/sim/rendering/database-pager/threads", true)->getIntValue(1), 1);
 
-    // must do this before preinit for Rembrandthe
-    flightgear::CameraGroup::buildDefaultGroup(viewer.get());
-    render->preinit();
-    viewer->startThreading();
+    if (composite_viewer) {
+        composite_viewer_view->setDatabasePager(FGScenery::getPagerSingleton());
+        composite_viewer_view->getDatabasePager()->setUnrefImageDataAfterApplyPolicy(true, false);
+        composite_viewer_view->getDatabasePager()->setUpThreads(numDBPagerThreads, 0);
+        composite_viewer_view->getDatabasePager()->setAcceptNewDatabaseRequests(true);
+        flightgear::CameraGroup::buildDefaultGroup(composite_viewer_view);
+        composite_viewer_view->setFrameStamp(composite_viewer->getFrameStamp());
+        osg::GraphicsContext::createNewContextID();
+
+#ifdef ENABLE_OSGXR
+        // After buildDefaultGroup (which puts used graphics contexts into viewer)
+        // Before threading re-enabled
+        flightgear::VRManager::instance()->reset();
+#endif
+
+        render->setView(composite_viewer_view);
+        render->preinit();
+        composite_viewer->startThreading();
+    }
+    else {
+        viewer->getDatabasePager()->setUpThreads(numDBPagerThreads, 0);
+        viewer->getDatabasePager()->setAcceptNewDatabaseRequests(true);
+        // must do this before preinit for Rembrandthe
+        flightgear::CameraGroup::buildDefaultGroup(viewer.get());
+        render->preinit();
+        viewer->startThreading();
+    }
     
     fgOSResetProperties();
 
