@@ -1,24 +1,9 @@
-// main.cxx -- top level sim routines
-//
-// Written by Curtis Olson, started May 1997.
-//
-// Copyright (C) 1997 - 2002  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: main.cxx
+ * SPDX-FileComment: top level sim routines
+ * SPDX-FileCopyrightText: Copyright (C) 1997 - 2002  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include <config.h>
 
@@ -34,47 +19,52 @@
 
 // Class references
 #include <simgear/canvas/VGInitOperation.hxx>
-#include <simgear/scene/model/modellib.hxx>
-#include <simgear/scene/material/matlib.hxx>
-#include <simgear/scene/material/Effect.hxx>
-#include <simgear/props/AtomicChangeListener.hxx>
-#include <simgear/props/props.hxx>
-#include <simgear/timing/sg_time.hxx>
-#include <simgear/io/raw_socket.hxx>
-#include <simgear/scene/tsync/terrasync.hxx>
-#include <simgear/math/SGMath.hxx>
-#include <simgear/math/sg_random.h>
-#include <simgear/misc/strutils.hxx>
-#include <simgear/structure/commands.hxx>
+#include <simgear/debug/logdelta.hxx>
 #include <simgear/emesary/Emesary.hxx>
 #include <simgear/emesary/notifications.hxx>
+#include <simgear/io/raw_socket.hxx>
+#include <simgear/math/SGMath.hxx>
+#include <simgear/math/sg_random.hxx>
+#include <simgear/misc/strutils.hxx>
+#include <simgear/nasal/NasalEmesaryInterface.hxx>
+#include <simgear/props/AtomicChangeListener.hxx>
+#include <simgear/props/props.hxx>
+#include <simgear/scene/material/Effect.hxx>
+#include <simgear/scene/material/matlib.hxx>
+#include <simgear/scene/model/modellib.hxx>
+#include <simgear/scene/tsync/terrasync.hxx>
+#include <simgear/structure/commands.hxx>
+#include <simgear/timing/sg_time.hxx>
 
 #include <Add-ons/AddonManager.hxx>
+#include <GUI/MessageBox.hxx>
+#include <GUI/gui.h>
 #include <Main/locale.hxx>
 #include <Model/panelnode.hxx>
+#include <Navaids/NavDataCache.hxx>
 #include <Scenery/scenery.hxx>
 #include <Sound/soundmanager.hxx>
 #include <Time/TimeManager.hxx>
-#include <GUI/gui.h>
-#include <GUI/MessageBox.hxx>
-#include <Viewer/splash.hxx>
-#include <Viewer/renderer.hxx>
+#include <Viewer/CameraGroup.hxx>
+#include <Viewer/GraphicsPresets.hxx>
 #include <Viewer/WindowSystemAdapter.hxx>
-#include <Navaids/NavDataCache.hxx>
-#include <Include/version.h>
-#include <Include/build.h>
+#include <Viewer/renderer.hxx>
+#include <Viewer/splash.hxx>
+#include <flightgearBuildId.h>
 
+#include "Main/MultipleInstanceLock.hxx"
 #include "fg_commands.hxx"
-#include "fg_io.hxx"
-#include "main.hxx"
-#include "util.hxx"
 #include "fg_init.hxx"
+#include "fg_io.hxx"
 #include "fg_os.hxx"
 #include "fg_props.hxx"
+#include "main.hxx"
+#include "options.hxx"
 #include "positioninit.hxx"
 #include "screensaver_control.hxx"
 #include "subsystemFactory.hxx"
-#include "options.hxx"
+#include "util.hxx"
+#include <Main/ErrorReporter.hxx>
 #include <Main/sentryIntegration.hxx>
 
 #include <simgear/embedded_resources/EmbeddedResourceManager.hxx>
@@ -98,26 +88,21 @@ using std::vector;
 extern int _bootstrap_OSInit;
 
 static SGPropertyNode_ptr frame_signal;
-static SGPropertyNode_ptr nasal_gc_threaded;
-static SGPropertyNode_ptr nasal_gc_threaded_wait;
 
 #ifdef NASAL_BACKGROUND_GC_THREAD
-extern "C" {
-    extern void startNasalBackgroundGarbageCollection();
-    extern void stopNasalBackgroundGarbageCollection();
-    extern void performNasalBackgroundGarbageCollection();
-    extern void awaitNasalGarbageCollectionComplete(bool can_wait);
-}
+static SGPropertyNode_ptr nasal_gc_threaded;
+static SGPropertyNode_ptr nasal_gc_threaded_wait;
+static  SGSharedPtr<simgear::Notifications::MainLoopNotification> mln_begin(new simgear::Notifications::MainLoopNotification(simgear::Notifications::MainLoopNotification::Type::Begin));
+static  SGSharedPtr<simgear::Notifications::MainLoopNotification> mln_end(new simgear::Notifications::MainLoopNotification(simgear::Notifications::MainLoopNotification::Type::End));
+static  SGSharedPtr<simgear::Notifications::MainLoopNotification> mln_started(new simgear::Notifications::MainLoopNotification(simgear::Notifications::MainLoopNotification::Type::Started));
+static  SGSharedPtr<simgear::Notifications::MainLoopNotification> mln_stopped(new simgear::Notifications::MainLoopNotification(simgear::Notifications::MainLoopNotification::Type::Stopped));
+static  SGSharedPtr<simgear::Notifications::NasalGarbageCollectionConfigurationNotification> ngccn;
 #endif
-static  simgear::Notifications::MainLoopNotification mln_begin(simgear::Notifications::MainLoopNotification::Type::Begin);
-static  simgear::Notifications::MainLoopNotification mln_end(simgear::Notifications::MainLoopNotification::Type::End);
-static  simgear::Notifications::MainLoopNotification mln_started(simgear::Notifications::MainLoopNotification::Type::Started);
-static  simgear::Notifications::MainLoopNotification mln_stopped(simgear::Notifications::MainLoopNotification::Type::Stopped);
-static  simgear::Notifications::NasalGarbageCollectionConfigurationNotification *ngccn = nullptr;
 // This method is usually called after OSG has finished rendering a frame in what OSG calls an idle handler and
 // is reposonsible for invoking all of the relevant per frame processing; most of which is handled by subsystems.
 static void fgMainLoop( void )
 {
+#ifdef NASAL_BACKGROUND_GC_THREAD
     //
     // the Nasal GC will automatically run when (during allocation) it discovers that more space is needed.
     // This has a cost of between 5ms and 50ms (depending on the amount of currently active Nasal).
@@ -141,9 +126,10 @@ static void fgMainLoop( void )
     notify_gc_config = ngccn->SetActive(use_threaded_gc);
     notify_gc_config |= ngccn->SetWait(threaded_wait);
     if (notify_gc_config)
-         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(ngccn);
 
      simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_begin);
+#endif
 
     if (sglog().has_popup()) {
         std::string s = sglog().get_popup();
@@ -152,20 +138,24 @@ static void fgMainLoop( void )
 
     frame_signal->fireValueChanged();
 
-    auto timeManager = globals->get_subsystem<TimeManager>();
+    // Fetch the subsystem manager.
+    auto mgr = globals->get_subsystem_mgr();
+
     // compute simulated time (allowing for pause, warp, etc) and
     // real elapsed time
     double sim_dt, real_dt;
-    timeManager->computeTimeDeltas(sim_dt, real_dt);
+    mgr->get_subsystem<TimeManager>()->computeTimeDeltas(sim_dt, real_dt);
 
     // update all subsystems
-    globals->get_subsystem_mgr()->update(sim_dt);
+    mgr->update(sim_dt);
 
     // flush commands waiting in the queue
     SGCommandMgr::instance()->executedQueuedCommands();
     simgear::AtomicChangeListener::fireChangeListeners();
 
-     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_end);
+#ifdef NASAL_BACKGROUND_GC_THREAD
+    simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_end);
+#endif
 }
 
 static void initTerrasync()
@@ -176,6 +166,7 @@ static void initTerrasync()
     globals->append_data_path(terraSyncDir, false /* = ahead of FG_ROOT */);
 
     if (fgGetBool("/sim/fghome-readonly", false)) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "initTerrasync() failing because /sim/fghome-readonly is true");
         return;
     }
 
@@ -183,9 +174,8 @@ static void initTerrasync()
     // hence not downloaded again.
     fgSetString("/sim/terrasync/installation-dir", (globals->get_fg_root() / "Scenery").utf8Str());
 
-    simgear::SGTerraSync* terra_sync = new simgear::SGTerraSync();
+    auto terra_sync = globals->get_subsystem_mgr()->add<simgear::SGTerraSync>();
     terra_sync->setRoot(globals->get_props());
-    globals->add_subsystem("terrasync", terra_sync);
 
     terra_sync->bind();
     terra_sync->init();
@@ -273,16 +263,25 @@ void registerMainLoop()
 {
     // stash current frame signal property
     frame_signal = fgGetNode("/sim/signals/frame", true);
+
+#ifdef NASAL_BACKGROUND_GC_THREAD
     nasal_gc_threaded = fgGetNode("/sim/nasal-gc-threaded", true);
     nasal_gc_threaded_wait = fgGetNode("/sim/nasal-gc-threaded-wait", true);
+#endif
+    // init the Emesary receiver for Nasal
+    nasal::initMainLoopRecipient();
+
     fgRegisterIdleHandler( fgMainLoop );
 }
 
 void unregisterMainLoopProperties()
 {
+    nasal::shutdownMainLoopRecipient();
     frame_signal.reset();
+#ifdef NASAL_BACKGROUND_GC_THREAD
     nasal_gc_threaded.reset();
     nasal_gc_threaded_wait.reset();
+#endif
 }
 
 } // namespace flightgear
@@ -301,32 +300,44 @@ static void fgIdleFunction ( void ) {
     // our initializations out of the idle callback so that we can get a
     // splash screen up and running right away.
 
+    // Fetch the subsystem manager.
+    auto mgr = globals->get_subsystem_mgr();
+
     if ( idle_state == 0 ) {
-        if (guiInit())
-        {
+        auto camera = flightgear::getGUICamera(flightgear::CameraGroup::getDefault());
+        if (guiInit(camera->getGraphicsContext())) {
             checkOpenGLVersion();
             fgSetVideoOptions();
-            idle_state+=2;
+            idle_state++;
             fgSplashProgress("loading-aircraft-list");
             fgSetBool("/sim/rendering/initialized", true);
         }
+    } else if (idle_state == 1) {
+        if (flightgear::ExclusiveInstanceLock::isLocked()) {
+            fgSplashProgress("wait-on-shared-lock");
+            return;
+        } else {
+            idle_state++;
+        }
 
-    } else if ( idle_state == 2 ) {
+    } else if (idle_state == 2) {
         initTerrasync();
         idle_state++;
         fgSplashProgress("loading-nav-dat");
 
-    } else if ( idle_state == 3 ) {
-
+    } else if (idle_state == 3) {
         bool done = fgInitNav();
         if (done) {
           ++idle_state;
           fgSplashProgress("init-scenery");
+          // now the nav-cache is built, we can let other copies of FG run in parallel
+          // note if we used the launcher, the lock was already dropped.
+          flightgear::ExclusiveInstanceLock::destroyInstance();
         }
-    } else if ( idle_state == 4 ) {
+    } else if (idle_state == 4) {
         idle_state++;
 
-        globals->add_new_subsystem<TimeManager>(SGSubsystemMgr::INIT);
+        mgr->add<TimeManager>();
 
         // Do some quick general initializations
         if( !fgInitGeneral()) {
@@ -350,44 +361,52 @@ static void fgIdleFunction ( void ) {
         globals->set_matlib( new SGMaterialLib );
         simgear::SGModelLib::setPanelFunc(FGPanelNode::load);
 
-    } else if (( idle_state == 5 ) || (idle_state == 2005)) {
+    } else if ((idle_state == 5) || (idle_state == 2005)) {
         idle_state+=2;
         flightgear::initPosition();
-        flightgear::initTowerLocationListener();
 
         simgear::SGModelLib::init(globals->get_fg_root().utf8Str(), globals->get_props());
 
-        auto timeManager = globals->get_subsystem<TimeManager>();
+        auto timeManager = mgr->get_subsystem<TimeManager>();
         timeManager->init();
 
         ////////////////////////////////////////////////////////////////////
         // Initialize the TG scenery subsystem.
         ////////////////////////////////////////////////////////////////////
 
-        globals->add_new_subsystem<FGScenery>(SGSubsystemMgr::DISPLAY);
-        globals->get_scenery()->init();
-        globals->get_scenery()->bind();
+        auto scenery = mgr->add<FGScenery>();
+        scenery->init();
+        scenery->bind();
 
         fgSplashProgress("creating-subsystems");
-    } else if (( idle_state == 7 ) || (idle_state == 2007)) {
+    } else if ((idle_state == 7) || (idle_state == 2007)) {
         bool isReset = (idle_state == 2007);
         idle_state = 8; // from the next state on, reset & startup are identical
         SGTimeStamp st;
         st.stamp();
-        fgCreateSubsystems(isReset);
+
+        try {
+            fgCreateSubsystems(isReset);
+        } catch (std::exception& e) {
+            // attempt to trace location of illegal argument / invalid string
+            // position errors on startup
+            flightgear::sentryReportException(std::string{"Creating subsystems: caught:"} + e.what());
+            throw;
+        }
+
         SG_LOG(SG_GENERAL, SG_INFO, "Creating subsystems took:" << st.elapsedMSec());
         fgSplashProgress("binding-subsystems");
 
-    } else if ( idle_state == 8 ) {
+    } else if (idle_state == 8) {
         idle_state++;
         SGTimeStamp st;
         st.stamp();
-        globals->get_subsystem_mgr()->bind();
+        mgr->bind();
         SG_LOG(SG_GENERAL, SG_INFO, "Binding subsystems took:" << st.elapsedMSec());
 
         fgSplashProgress("init-subsystems");
-    } else if ( idle_state == 9 ) {
-        SGSubsystem::InitStatus status = globals->get_subsystem_mgr()->incrementalInit();
+    } else if (idle_state == 9) {
+        SGSubsystem::InitStatus status = mgr->incrementalInit();
         if ( status == SGSubsystem::INIT_DONE) {
           ++idle_state;
           fgSplashProgress("finishing-subsystems");
@@ -395,11 +414,11 @@ static void fgIdleFunction ( void ) {
           fgSplashProgress("init-subsystems");
         }
 
-    } else if ( idle_state == 10 ) {
+    } else if (idle_state == 10) {
         idle_state = 900;
         fgPostInitSubsystems();
         fgSplashProgress("finalize-position");
-    } else if ( idle_state == 900 ) {
+    } else if (idle_state == 900) {
         idle_state = 1000;
 
         // setup OpenGL view parameters
@@ -424,10 +443,11 @@ static void fgIdleFunction ( void ) {
         fgSetBool("sim/sceneryloaded", false);
         flightgear::registerMainLoop();
 
+#ifdef NASAL_BACKGROUND_GC_THREAD
         ngccn = new simgear::Notifications::NasalGarbageCollectionConfigurationNotification(nasal_gc_threaded->getBoolValue(), nasal_gc_threaded_wait->getBoolValue());
-         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(ngccn);
          simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_started);
-        
+#endif        
         flightgear::addSentryBreadcrumb("entering main loop", "info");
     }
 
@@ -465,25 +485,6 @@ void fgInitSecureMode()
                               SGPropertyNode::PROTECTED);
 }
 
-static void upper_case_property(const char *name)
-{
-    using namespace simgear;
-    SGPropertyNode *p = fgGetNode(name, false);
-    if (!p) {
-        p = fgGetNode(name, true);
-        p->setStringValue("");
-    } else {
-        props::Type t = p->getType();
-        if (t == props::NONE || t == props::UNSPECIFIED)
-            p->setStringValue("");
-        else
-            assert(t == props::STRING);
-    }
-    SGPropertyChangeListener* muc = new FGMakeUpperCase;
-    globals->addListenerToCleanup(muc);
-    p->addChangeListener(muc);
-}
-
 // this hack is needed to avoid weird viewport sizing within OSG on Windows.
 // still required as of March 2017, sad times.
 // see for example https://sourceforge.net/p/flightgear/codetickets/1958/
@@ -522,7 +523,9 @@ static void rotateOldLogFiles()
     if (!p.exists())
         return;
     SGPath log0Path = homePath / "fgfs_0.log";
-    p.rename(log0Path);
+    if (!p.rename(log0Path)) {
+        std::cerr << "Failed to rename " << p.str() << " to " << log0Path.str() << std::endl;
+    }
 }
 
 static void logToHome(const std::string& pri)
@@ -532,7 +535,7 @@ static void logToHome(const std::string& pri)
     if (!pri.empty()) {
         try {
             fileLogLevel = std::min(fileLogLevel, logstream::priorityFromString(pri));
-        } catch (std::exception& e) {
+        } catch (std::exception& ) {
             // let's not worry about this, and just log at INFO
         }
     }
@@ -546,6 +549,17 @@ static void logToHome(const std::string& pri)
     sglog().logToFile(logPath, SG_ALL, fileLogLevel);
 }
 
+struct SGLogDeltasListener : SGPropertyChangeListener
+{
+    void valueChanged(SGPropertyNode* node) override
+    {
+        std::string value = node->getStringValue();
+        std::cerr << __FILE__ << ":" << __LINE__ << ": sglogdeltas value=" << value << "\n";
+        logDeltaSet(value.c_str());
+    }
+};
+static SGLogDeltasListener s_sglogdeltas_listener;
+
 // Main top level initialization
 int fgMainInit( int argc, char **argv )
 {
@@ -564,14 +578,18 @@ int fgMainInit( int argc, char **argv )
 #endif
 
     // check if the launcher is requested, since it affects config file parsing
-    bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
+    bool showLauncher = flightgear::Options::checkForArgEnable(argc, argv, "launcher");
     // an Info.plist bundle can't define command line arguments, but it can set
     // environment variables. This avoids needed a wrapper shell-script on OS-X.
     showLauncher |= (::getenv("FG_LAUNCHER") != nullptr);
 
+    bool needToShowReadOnlyWarning = (initHomeResult == InitHomeReadOnly);
 #if defined(HAVE_QT)
-    if (showLauncher && (initHomeResult == InitHomeReadOnly)) {
-// show this message early, if we can
+    // honour headless mode, even if Qt is enabled
+    // if we're in headless mode, we fall through to the case
+    // below and call modalMessageBox which logs
+    if (needToShowReadOnlyWarning && !flightgear::isHeadlessMode()) {
+        needToShowReadOnlyWarning = false;
         auto r = flightgear::showLockFileDialog();
         if (r == flightgear::LockFileReset) {
             SG_LOG( SG_GENERAL, SG_MANDATORY_INFO, "Deleting lock file at user request");
@@ -582,7 +600,34 @@ int fgMainInit( int argc, char **argv )
             return EXIT_SUCCESS;
         }
     }
-#endif
+ #endif
+
+    if (needToShowReadOnlyWarning) {
+        // will handle headless mode by logging
+        flightgear::modalMessageBox("Multiple copies of FlightGear",
+            "Another copy of FlightGear is running, so this copy will run read-only.", 
+            "This means aircraft and scenery cannot be downloaded, and settings will not be saved.");
+    }
+
+    {
+        SGPropertyNode* sglogdeltas = globals->get_props()->getNode("/sim/sg-log-deltas", true /*create*/);
+        assert(sglogdeltas);
+        sglogdeltas->addChangeListener(&s_sglogdeltas_listener, false /*initial*/);
+        const char* sglogdeltas_value = getenv("SG_LOG_DELTAS");
+        if (sglogdeltas_value) {
+            sglogdeltas->setStringValue(sglogdeltas_value);
+        }
+    }
+    
+    globals->get_props()->getNode("/sim", true /*create*/)->setAttribute(SGPropertyNode::VALUE_CHANGED_DOWN, true);
+    
+    {
+        SGPropertyNode* active = globals->get_props()->getNode("/sim/property-locking/active", true /*create*/);
+        SGPropertyNode* verbose = globals->get_props()->getNode("/sim/property-locking/verbose", true /*create*/);
+        SGPropertyNode* timing = globals->get_props()->getNode("/sim/property-locking/timing", true /*create*/);
+        SGPropertyNode* parent_listeners = globals->get_props()->getNode("/sim/property-locking/parent_listeners", true /*create*/);
+        SGPropertyLockControl(active, verbose, timing, parent_listeners);
+    }
     
     const bool readOnlyFGHome = fgGetBool("/sim/fghome-readonly");
     if (!readOnlyFGHome) {
@@ -602,9 +647,7 @@ int fgMainInit( int argc, char **argv )
 	SG_LOG( SG_GENERAL, SG_INFO, "Jenkins number/ID " << JENKINS_BUILD_NUMBER << ":"
 			<< JENKINS_BUILD_ID);
 
-#if OSG_VERSION_LESS_THAN(3,4,1)
-    SG_LOG(SG_GENERAL, SG_ALERT, "Minimum supported OpenScenegraph is V3.4.1 - currently using " << osgGetVersion() << " This can cause fatal OSG 'final reference count' errors at runtime");
-#endif
+    flightgear::addSentryTag("osg-version", osgGetVersion());
     
 #ifdef __OpenBSD__
     {
@@ -617,7 +660,7 @@ int fgMainInit( int argc, char **argv )
             SG_LOG( SG_GENERAL, SG_INFO, "This is OpenBSD; getrlimit() failed: " << strerror(errno));
         }
         else {
-            long long   required = 4LL * (1LL<<30);
+            unsigned long long   required = 4ULL * (1ULL<<30);
             if (rlimit.rlim_cur < required) {
                 SG_LOG( SG_GENERAL, SG_POPUP, ""
                         << "Max data segment (" << rlimit.rlim_cur << "bytes) too small.\n"
@@ -635,12 +678,6 @@ int fgMainInit( int argc, char **argv )
     string_list *col = new string_list;
     globals->set_channel_options_list( col );
 
-    fgValidatePath(globals->get_fg_home(), false);  // initialize static variables
-    upper_case_property("/sim/presets/airport-id");
-    upper_case_property("/sim/presets/runway");
-    upper_case_property("/sim/tower/airport-id");
-    upper_case_property("/autopilot/route-manager/input");
-
     if (showLauncher) {
         // to minimise strange interactions when launcher and config files
         // set overlaping options, we disable the default files. Users can
@@ -658,6 +695,7 @@ int fgMainInit( int argc, char **argv )
         return EXIT_SUCCESS;
     }
 
+    bool didUseLauncher = false;
 #if defined(HAVE_QT)
     if (showLauncher) {
         flightgear::addSentryBreadcrumb("starting launcher", "info");
@@ -665,6 +703,7 @@ int fgMainInit( int argc, char **argv )
             return EXIT_SUCCESS;
         }
 
+        didUseLauncher = true;
         flightgear::addSentryBreadcrumb("completed launcher", "info");
     }
 #else
@@ -676,7 +715,10 @@ int fgMainInit( int argc, char **argv )
     fgInitSecureMode();
     fgInitAircraftPaths(false);
 
-    configResult = fgInitAircraft(false);
+    auto errorManager = globals->get_subsystem_mgr()->add<flightgear::ErrorReporter>();
+    errorManager->preinit();
+
+    configResult = fgInitAircraft(false, didUseLauncher);
     if (configResult == flightgear::FG_OPTIONS_ERROR) {
         return EXIT_FAILURE;
     } else if ((configResult == flightgear::FG_OPTIONS_EXIT) ||
@@ -724,7 +766,14 @@ int fgMainInit( int argc, char **argv )
     // Copy the property nodes for the menus added by registered add-ons
     addons::AddonManager::instance()->addAddonMenusToFGMenubar();
 
+    // The GraphicsPreset subsystem is special - it's not added together with
+    // the rest of the subsystems because it should be present before all the
+    // graphics-related stuff is initialized.
+    auto presets = globals->get_subsystem_mgr()->add<flightgear::GraphicsPresets>();
+    presets->applyInitialPreset();
+
     // Initialize the Window/Graphics environment.
+
     fgOSInit(&argc, argv);
     _bootstrap_OSInit++;
 
@@ -740,10 +789,6 @@ int fgMainInit( int argc, char **argv )
     fntInit();
     globals->get_renderer()->preinit();
 
-#if defined(ENABLE_COMPOSITOR)
-    flightgear::addSentryTag("compositor", "yes");
-#endif
-
     if (fgGetBool("/sim/ati-viewport-hack", true)) {
         SG_LOG(SG_GENERAL, SG_WARN, "Enabling ATI/AMD viewport hack");
         flightgear::addSentryTag("ati-viewport-hack", "enabled");
@@ -761,8 +806,12 @@ int fgMainInit( int argc, char **argv )
 
     fgOSCloseWindow();
     fgShutdownHome();
-    
-     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_stopped);
+
+    const bool requestLauncherRestart = fgGetBool("/sim/restart-launcher-on-exit");
+
+#ifdef NASAL_BACKGROUND_GC_THREAD
+    simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_stopped);
+#endif
 
     simgear::clearEffectCache();
 
@@ -775,5 +824,23 @@ int fgMainInit( int argc, char **argv )
     // objects (eg, airports, navaids, runways).
     delete flightgear::NavDataCache::instance();
 
+#if defined(HAVE_QT)
+    if (requestLauncherRestart) {
+        string_list originalArgs;
+        for (int arg = 1; arg < argc; ++arg) {
+            originalArgs.push_back(argv[arg]);
+        }
+
+        if (!showLauncher) {
+            // The user specified the --restart-launcher option but without the
+            // --launcher option. So let's add the --launcher option so that the
+            // launcher actually starts instead of the simulator again.
+            originalArgs.push_back("--launcher");
+        }
+
+        flightgear::addSentryBreadcrumb("Requested to restart launcher", "info");
+        flightgear::startLaunchOnExit(originalArgs);
+    }
+#endif
     return result;
 }
