@@ -1,45 +1,30 @@
 // sentryIntegration.cxx - Interface with Sentry.io crash reporting
 //
-// Copyright (C) 2020 James Turner  james@flightgear.org
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+// SPDX-FileCopyrightText: (C) 2018 James Turner <james@flightgear.org>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
 
 #include "sentryIntegration.hxx"
 
 #include <cstring> // for strcmp
-#include <algorithm>
 
-#include <simgear/debug/LogCallback.hxx>
-#include <simgear/debug/logstream.hxx>
 #include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/debug/LogCallback.hxx>
+#include <simgear/debug/Reporting.hxx>
+#include <simgear/debug/logstream.hxx>
+
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/structure/exception.hxx>
-#include <simgear/io/iostreams/sgstream.hxx>
 
-#include <Include/version.h>
 #include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
 
-#include <build.h>
-
-using namespace std;
+#include <flightgearBuildId.h>
 
 bool doesStringMatchPrefixes(const std::string& s, const std::initializer_list<const char*>& prefixes)
 {
@@ -47,7 +32,7 @@ bool doesStringMatchPrefixes(const std::string& s, const std::initializer_list<c
         return false;
 
 
-    for (auto c  : prefixes) {
+    for (auto c : prefixes) {
         if (s.find(c) == 0)
             return true;
     }
@@ -56,31 +41,19 @@ bool doesStringMatchPrefixes(const std::string& s, const std::initializer_list<c
 }
 
 auto OSG_messageWhitelist = {
-     "PNG lib warning : iCCP: known incorrect sRGB profile",
-     "PNG lib warning : iCCP: profile 'ICC Profile': 1000000h: invalid rendering intent",
-     "osgDB ac3d reader: detected surface with less than 3",
-     "osgDB ac3d reader: detected line with less than 2",
-     "Detected particle system using segment(s) with less than 2 vertices"
-};
-
-auto XML_messageWhitelist = {
-    "Cannot open file",
-    "not well-formed (invalid token)",
-    "mismatched tag",
-    "syntax error",
-    "no element found",
-    "Root element name is",
-    "XML or text declaration not at start of entity",
-    "Failed to open file",
-    "unclosed token"};
+    "PNG lib warning : iCCP: known incorrect sRGB profile",
+    "PNG lib warning : iCCP: profile 'ICC Profile': 1000000h: invalid rendering intent",
+    "osgDB ac3d reader: detected surface with less than 3",
+    "osgDB ac3d reader: detected line with less than 2",
+    "Detected particle system using segment(s) with less than 2 vertices"};
 
 auto exception_messageWhitelist = {
-    "position is invalid, NaNs",    ///< avoid spam when NaNs occur
-    "bad AI flight plan",           ///< adjusting logic to avoid this is tricky
-    "couldn't find shader",         ///< handled seperately
-    "unable to find add-on metadata file",
+    "position is invalid, NaNs", ///< avoid spam when NaNs occur
+    "bad AI flight plan",        ///< adjusting logic to avoid this is tricky
+    "couldn't find shader",      ///< handled separately
+    "(EMEXEC)"                   ///< Emesary log spam
 
-    /// supress noise from user-entered METAR values : we special case
+    /// suppress noise from user-entered METAR values : we special case
     /// when live metar fails to parse
     "metar data bogus",
     "metar data incomplete"
@@ -90,7 +63,7 @@ auto exception_messageWhitelist = {
 #if defined(HAVE_SENTRY) && !defined(BUILDING_TESTSUITE)
 
 static bool static_sentryEnabled = false;
-thread_local bool perThread_reportXMLParseErrors = true;
+static std::string static_sentryUUID;
 
 #include <sentry.h>
 
@@ -103,15 +76,6 @@ void sentryTraceSimgearThrow(const std::string& msg, const std::string& origin,
     if (!static_sentryEnabled)
         return;
 
-// don't report the exceptions raised by easyxml.cxx, if this per-thread
-// flag is set. This avoids frequent errors when the launcher scans
-// directories containing many aircraft of unknown origin/quality
-// if the user tries to fly with one, we'll still get an error then,
-// but that's a real failure point (from the user PoV)
-    if (!perThread_reportXMLParseErrors && doesStringMatchPrefixes(msg, XML_messageWhitelist)) {
-        return;
-    }
-
     if (doesStringMatchPrefixes(msg, exception_messageWhitelist)) {
         return;
     }
@@ -119,9 +83,8 @@ void sentryTraceSimgearThrow(const std::string& msg, const std::string& origin,
     sentry_value_t exc = sentry_value_new_object();
     sentry_value_set_by_key(exc, "type", sentry_value_new_string("Exception"));
 
-    string message = msg;
+    std::string message = msg;
     sentry_value_t info = sentry_value_new_object();
-
     if (!origin.empty()) {
         sentry_value_set_by_key(info, "origin", sentry_value_new_string(origin.c_str()));
     }
@@ -181,7 +144,7 @@ private:
     int _lastLoggedCount = 0;
 };
 
-const auto missingShaderPrefix = string{"Missing shader"};
+const auto missingShaderPrefix = std::string{"Missing shader"};
 
 string_list anon_missingShaderList;
 
@@ -196,7 +159,7 @@ bool isNewMissingShader(const std::string& path)
     return true;
 }
 
-void sentrySimgearReportCallback(const string& msg, const string& more, bool isFatal)
+void sentrySimgearReportCallback(const std::string& msg, const std::string& more, bool isFatal)
 {
     // don't duplicate reports for missing shaders, once per sessions
     // is sufficient
@@ -225,23 +188,24 @@ void sentrySimgearReportCallback(const string& msg, const string& more, bool isF
 
 void sentryReportBadAlloc()
 {
-    sentry_value_t sentryMessage = sentry_value_new_object();
-    sentry_value_set_by_key(sentryMessage, "type", sentry_value_new_string("Fatal Error"));
-    sentry_value_set_by_key(sentryMessage, "formatted", sentry_value_new_string("bad allocation"));
+    if (simgear::ReportBadAllocGuard::isSet()) {
+        sentry_value_t sentryMessage = sentry_value_new_object();
+        sentry_value_set_by_key(sentryMessage, "type", sentry_value_new_string("Fatal Error"));
+        sentry_value_set_by_key(sentryMessage, "formatted", sentry_value_new_string("bad allocation"));
 
-    sentry_value_t event = sentry_value_new_event();
-    sentry_value_set_by_key(event, "message", sentryMessage);
+        sentry_value_t event = sentry_value_new_event();
+        sentry_value_set_by_key(event, "message", sentryMessage);
 
-    sentry_event_value_add_stacktrace(event, nullptr, 0);
-    sentry_capture_event(event);
+        sentry_event_value_add_stacktrace(event, nullptr, 0);
+        sentry_capture_event(event);
+    }
 
-    std::set_new_handler(nullptr);
+    throw std::bad_alloc(); // allow normal processing
 }
 
 } // namespace
 
-namespace flightgear
-{
+namespace flightgear {
 
 bool sentryReportCommand(const SGPropertyNode* args, SGPropertyNode* root)
 {
@@ -253,7 +217,7 @@ bool sentryReportCommand(const SGPropertyNode* args, SGPropertyNode* root)
     sentry_value_t exc = sentry_value_new_object();
     sentry_value_set_by_key(exc, "type", sentry_value_new_string("Report"));
 
-    const string message = args->getStringValue("message");
+    const auto message = args->getStringValue("message");
     sentry_value_set_by_key(exc, "value", sentry_value_new_string(message.c_str()));
 
     sentry_value_t event = sentry_value_new_event();
@@ -282,46 +246,85 @@ bool sentrySendError(const SGPropertyNode* args, SGPropertyNode* root)
     return true;
 }
 
+std::string sentryUserId()
+{
+    if (!static_sentryUUID.empty()) {
+        return static_sentryUUID;
+    }
+
+    const auto uuidPath = fgHomePath() / "sentry_uuid.txt";
+    if (!uuidPath.exists()) {
+        return {};
+    }
+
+    sg_ifstream f(uuidPath);
+    std::getline(f, static_sentryUUID);
+    return static_sentryUUID;
+}
 
 void initSentry()
 {
-    sentry_options_t *options = sentry_options_new();
+    sentry_options_t* options = sentry_options_new();
     // API key is defined in config.h, set in an environment variable prior
     // to running CMake, so it can be customised. Env var at build time is:
     // FLIGHTGEAR_SENTRY_API_KEY
     sentry_options_set_dsn(options, SENTRY_API_KEY);
-    
-    // modified for 2020.3: we don't want to create '-dev' releases
-    // even if people self-compile the branch.
-    sentry_options_set_release(options, "flightgear@" FLIGHTGEAR_VERSION);
+
+    if (strcmp(FG_BUILD_TYPE, "Dev") == 0) {
+        sentry_options_set_environment(options, "dev");
+        sentry_options_set_release(options, "flightgear-dev@" REVISION);
+    } else if (strcmp(FG_BUILD_TYPE, "Nightly") == 0) {
+        sentry_options_set_environment(options, "dev");
+        // we use the build year and month here to cluster nightly builds in Sentry.
+        // the idea being that this is sufficient granularity to notice when an issue
+        // first occurs, and then bisect the commits. We could switch to using
+        // year and week number, but this makes more noise in sentry.
+        sentry_options_set_release(options, "flightgear-nightly@" BUILD_MONTH);
+    } else if (strcmp(FG_BUILD_TYPE, "Release") == 0) {
+        // RC builds are for testing
+        if (strncmp(BUILD_SUFFIX, "rc", 2) == 0) {
+            sentry_options_set_environment(options, "testing");
+        } else {
+            sentry_options_set_environment(options, "production");
+        }
+
+        sentry_options_set_release(options, FLIGHTGEAR_VERSION);
+    } else {
+        // unknown build type
+        sentry_options_set_environment(options, "dev");
+    }
     
     const auto buildString = std::to_string(JENKINS_BUILD_NUMBER);
     sentry_options_set_dist(options, buildString.c_str());
-    
+
+    // for dev / nightly builds, put Sentry in debug mode
+    if (strcmp(FG_BUILD_TYPE, "Release")) {
+        sentry_options_set_debug(options, 1);
+    }
+
     SGPath dataPath = fgHomePath() / "sentry_db";
 #if defined(SG_WINDOWS)
     const auto homePathString = dataPath.wstr();
     sentry_options_set_database_pathw(options, homePathString.c_str());
-    
+
     const auto logPath = (fgHomePath() / "fgfs.log").wstr();
     sentry_options_add_attachmentw(options, logPath.c_str());
 #else
     const auto homePathString = dataPath.utf8Str();
     sentry_options_set_database_path(options, homePathString.c_str());
-    
+
     const auto logPath = (fgHomePath() / "fgfs.log").utf8Str();
     sentry_options_add_attachment(options, logPath.c_str());
 #endif
-    
 
     const auto uuidPath = fgHomePath() / "sentry_uuid.txt";
     bool generateUuid = true;
     std::string uuid;
     if (uuidPath.exists()) {
-        sg_ifstream f(uuidPath);
-        std::getline(f, uuid);
+        sentryUserId(); // will cache into static_sentryUUID as a side-effect
+
         // if we read enough bytes, that this is a valid UUID, then accept it
-        if ( uuid.length() >= 36) {
+        if (static_sentryUUID.length() >= 36) {
             generateUuid = false;
         }
     }
@@ -334,25 +337,28 @@ void initSentry()
         sentry_uuid_as_string(&su, bytes);
         bytes[37] = 0;
 
-        uuid = std::string{bytes};
+        static_sentryUUID = std::string{bytes};
         // write it back to disk for next time
         sg_ofstream f(uuidPath);
-        f << uuid << endl;
+        f << static_sentryUUID << std::endl;
     }
 
-    sentry_init(options);
-    static_sentryEnabled = true;
+    if (sentry_init(options) == 0) {
+        static_sentryEnabled = true;
+        sentry_value_t user = sentry_value_new_object();
+        sentry_value_t userUuidV = sentry_value_new_string(static_sentryUUID.c_str());
+        sentry_value_set_by_key(user, "id", userUuidV);
+        sentry_set_user(user);
 
-    sentry_value_t user = sentry_value_new_object();
-    sentry_value_t userUuidV = sentry_value_new_string(uuid.c_str());
-    sentry_value_set_by_key(user, "id", userUuidV);
-    sentry_set_user(user);
+        sglog().addCallback(new SentryLogCallback);
+        setThrowCallback(sentryTraceSimgearThrow);
+        simgear::setErrorReportCallback(sentrySimgearReportCallback);
 
-    sglog().addCallback(new SentryLogCallback);
-    setThrowCallback(sentryTraceSimgearThrow);
-    simgear::setErrorReportCallback(sentrySimgearReportCallback);
-
-    std::set_new_handler(sentryReportBadAlloc);
+        std::set_new_handler(sentryReportBadAlloc);
+    } else {
+        SG_LOG(SG_GENERAL, SG_WARN, "Failed to init Sentry reporting");
+        static_sentryEnabled = false;
+    }
 }
 
 void delayedSentryInit()
@@ -370,6 +376,10 @@ void delayedSentryInit()
 
     globals->get_commands()->addCommand("sentry-report", &sentryReportCommand);
     globals->get_commands()->addCommand("sentry-exception", &sentrySendError);
+
+    // expose the anonymous user UUID to the property tree, so users
+    // can share it if they wish
+    fgSetString("/sim/crashreport/sentry-user-id", static_sentryUUID);
 }
 
 void shutdownSentry()
@@ -402,7 +412,7 @@ void addSentryTag(const char* tag, const char* value)
 
     if (!tag || !value)
         return;
-    
+
     sentry_set_tag(tag, value);
 }
 
@@ -433,17 +443,17 @@ void sentryReportNasalError(const std::string& msg, const string_list& stack)
     }
     sentry_value_set_by_key(exc, "stack", stackData);
 
-    
+
     sentry_value_t event = sentry_value_new_event();
     sentry_value_set_by_key(event, "exception", exc);
-    
+
     // add the Nasal stack trace data
-    
+
     // capture the C++ stack-trace. Probably not that useful but can't hurt
     sentry_event_value_add_stacktrace(event, nullptr, 0);
-    
+
     sentry_capture_event(event);
-    
+
 #endif
 }
 
@@ -466,13 +476,13 @@ void sentryReportException(const std::string& msg, const std::string& location)
 
     sentry_value_t event = sentry_value_new_event();
     sentry_value_set_by_key(event, "exception", exc);
-    
+
     // capture the C++ stack-trace. Probably not that useful but can't hurt
     sentry_event_value_add_stacktrace(event, nullptr, 0);
     sentry_capture_event(event);
 }
 
-void  sentryReportFatalError(const std::string& msg, const std::string& more)
+void sentryReportFatalError(const std::string& msg, const std::string& more)
 {
     if (!static_sentryEnabled)
         return;
@@ -490,17 +500,12 @@ void  sentryReportFatalError(const std::string& msg, const std::string& more)
 
     sentry_value_t event = sentry_value_new_event();
     sentry_value_set_by_key(event, "message", sentryMessage);
-    
+
     sentry_event_value_add_stacktrace(event, nullptr, 0);
     sentry_capture_event(event);
 }
 
-void sentryThreadReportXMLErrors(bool report)
-{
-    perThread_reportXMLParseErrors = report;
-}
-
-void sentryReportUserError(const std::string& aggregate, const std::string& details)
+void sentryReportUserError(const std::string& aggregate, const std::string& parameter, const std::string& details)
 {
     if (!static_sentryEnabled)
         return;
@@ -513,20 +518,24 @@ void sentryReportUserError(const std::string& aggregate, const std::string& deta
 
     sentry_set_context("what", info);
 
+    auto m = aggregate;
+    if (!parameter.empty()) {
+        m += ":" + parameter;
+    }
+
     sentry_value_t event = sentry_value_new_event();
-    sentry_value_set_by_key(event, "message", sentry_value_new_string(aggregate.c_str()));
+    sentry_value_set_by_key(event, "message", sentry_value_new_string(m.c_str()));
 
     sentry_capture_event(event);
 }
 
-} // of namespace
+} // namespace flightgear
 
 #else
 
 // stubs for non-sentry case
 
-namespace flightgear
-{
+namespace flightgear {
 
 void initSentry()
 {
@@ -545,6 +554,12 @@ bool isSentryEnabled()
     return false;
 }
 
+std::string sentryUserId()
+{
+    return {};
+}
+
+
 void addSentryBreadcrumb(const std::string&, const std::string&)
 {
 }
@@ -562,7 +577,7 @@ void sentryReportNasalError(const std::string&, const string_list&)
 {
 }
 
-void sentryReportException(const std::string&, const  std::string&)
+void sentryReportException(const std::string&, const std::string&)
 {
 }
 
@@ -570,39 +585,24 @@ void sentryReportFatalError(const std::string&, const std::string&)
 {
 }
 
-void sentryThreadReportXMLErrors(bool)
+void sentryReportUserError(const std::string&, const std::string&, const std::string&)
 {
 }
 
-void sentryReportUserError(const std::string&, const std::string&)
-{
-}
-
-} // of namespace
+} // namespace flightgear
 
 #endif
 
 // common helpers
 
-namespace flightgear
-{
+namespace flightgear {
 
 void addSentryTag(const std::string& tag, const std::string& value)
 {
     if (tag.empty() || value.empty())
         return;
-    
+
     addSentryTag(tag.c_str(), value.c_str());
 }
 
-SentryXMLErrorSupression::SentryXMLErrorSupression()
-{
-    sentryThreadReportXMLErrors(false);
-}
-
-SentryXMLErrorSupression::~SentryXMLErrorSupression()
-{
-    sentryThreadReportXMLErrors(true);
-}
-
-} // of namespace flightgear
+} // namespace flightgear
