@@ -1,7 +1,9 @@
-// modelmgr.cxx - manage a collection of 3D models.
-// Written by David Megginson, started 2002.
-//
-// This file is in the Public Domain, and comes with no warranty.
+/*
+ * SPDX-FileName: modelmgr.cxx
+ * SPDX-FileComment: manage a collection of 3D models
+ * SPDX-FileCopyrightText: Written by David Megginson, started 2002.
+ * SPDX-License-Identifier: This file is in the Public Domain, and comes with no warranty.
+ */
 
 #ifdef _MSC_VER
 #  pragma warning( disable: 4355 )
@@ -24,14 +26,62 @@
 
 #include <Main/fg_props.hxx>
 #include <Scenery/scenery.hxx>
+#include <Scenery/marker.hxx>
 
+#include <osg/ProxyNode>
+#include <osgText/String>
 
 #include "modelmgr.hxx"
 
 using namespace simgear;
 
-// OSGFIXME
-// extern SGShadowVolume *shadows;
+namespace {
+
+class CheckInstanceModelLoadedVisitor : public osg::NodeVisitor
+{
+public:
+    CheckInstanceModelLoadedVisitor() :
+        osg::NodeVisitor(osg::NodeVisitor::NODE_VISITOR, osg::NodeVisitor::TRAVERSE_ALL_CHILDREN)
+    {}
+    
+    ~CheckInstanceModelLoadedVisitor() = default;
+
+    void apply(osg::Node& node) override
+    {
+        if (!_loaded)
+            return;
+        traverse(node);
+    }
+    
+    void apply(osg::ProxyNode& node) override
+    {
+        if (!_loaded)
+            return;
+
+        for (unsigned i = 0; i < node.getNumFileNames(); ++i) {
+            if (node.getFileName(i).empty())
+                continue;
+            
+            // Check if this is already loaded.
+            if (i < node.getNumChildren() && node.getChild(i))
+                continue;
+
+            _loaded=false;
+            return;
+        }
+        traverse(node);
+    }
+
+    bool isLoaded() const
+    {
+        return _loaded;
+    }
+
+private:
+    bool _loaded = true;
+};
+
+} // of anonymous namespace
 
 FGModelMgr::FGModelMgr ()
 {
@@ -75,26 +125,46 @@ FGModelMgr::add_model (SGPropertyNode * node)
         SG_LOG(SG_AIRCRAFT, SG_WARN, "add_model called with empty path");
         return;
     }
- 
-  osg::Node *object;
 
-  try {
-      std::string fullPath = simgear::SGModelLib::findDataFile(model_path);
-      if (fullPath.empty()) {
-          SG_LOG(SG_AIRCRAFT, SG_ALERT, "add_model: unable to find model with name '" << model_path << "'");
-          return;
-      }
-      object = SGModelLib::loadDeferredModel(fullPath, globals->get_props());
-  } catch (const sg_throwable& t) {
-    SG_LOG(SG_AIRCRAFT, SG_ALERT, "Error loading " << model_path << ":\n  "
-        << t.getFormattedMessage() << t.getOrigin());
-    return;
+    const std::string internal_model{node->getStringValue("internal-model", "external")};
+
+    osg::ref_ptr<osg::Node> object;
+
+    Instance* instance = new Instance;
+    instance->loaded_node = node->addChild("loaded");
+    instance->loaded_node->setBoolValue(false);
+
+    if (internal_model == "marker") {
+        std::string label{node->getStringValue("marker/text", "MARKER")};
+        float r = node->getFloatValue("marker/color[0]", 1.0f);
+        float g = node->getFloatValue("marker/color[1]", 1.0f);
+        float b = node->getFloatValue("marker/color[2]", 1.0f);
+        osg::Vec4f color(r, g, b, 1.0f);
+        float font_size = node->getFloatValue("marker/size", 1.0f);
+        float pin_height = node->getFloatValue("marker/height", 1000.0f);
+        float tip_height = node->getFloatValue("marker/tip-height", 0.0f);
+        object = fgCreateMarkerNode(osgText::String(label, osgText::String::ENCODING_UTF8), font_size, pin_height, tip_height, color);
+    } else if (internal_model == "external") {
+        try {
+            std::string fullPath = simgear::SGModelLib::findDataFile(model_path);
+            if (fullPath.empty()) {
+                SG_LOG(SG_AIRCRAFT, SG_ALERT, "add_model: unable to find model with name '" << model_path << "'");
+                return;
+            }
+            object = SGModelLib::loadDeferredModel(fullPath, globals->get_props());
+        } catch (const sg_throwable& t) {
+            SG_LOG(SG_AIRCRAFT, SG_ALERT, "Error loading " << model_path << ":\n  " << t.getFormattedMessage() << t.getOrigin());
+            return;
+        }
+  }
+  else {
+      object = new osg::Node;
+      SG_LOG(SG_AIRCRAFT, SG_WARN, "Unsupported internal-model type " << internal_model);
   }
-    
-    const std::string modelName{node->getStringValue("name", model_path.c_str())};
-    SG_LOG(SG_AIRCRAFT, SG_INFO, "Adding model " << modelName);
 
-  Instance * instance = new Instance;
+  const std::string modelName{node->getStringValue("name", model_path.c_str())};
+  SG_LOG(SG_AIRCRAFT, SG_INFO, "Adding model " << modelName);
+
   SGModelPlacement *model = new SGModelPlacement;
   instance->model = model;
   instance->node = node;
@@ -187,11 +257,11 @@ double testNan(double val)
 
     return val;
 }
+} // namespace
 
-struct UpdateFunctor : public std::unary_function<FGModelMgr::Instance*, void>
+void FGModelMgr::update(double dt)
 {
-    void operator()(FGModelMgr::Instance* instance) const
-    {
+    std::for_each(_instances.begin(), _instances.end(), [](FGModelMgr::Instance* instance) {
         SGModelPlacement* model = instance->model;
         double roll, pitch, heading;
         roll = pitch = heading = 0.0;
@@ -214,8 +284,7 @@ struct UpdateFunctor : public std::unary_function<FGModelMgr::Instance*, void>
             if (instance->heading_deg_node != 0)
                 heading = testNan(instance->heading_deg_node->getDoubleValue());
         } catch (const sg_range_exception&) {
-            const char *path = instance->node->getStringValue("path",
-                                                              "unknown");
+            std::string path = instance->node->getStringValue("path", "unknown");
             SG_LOG(SG_AIRCRAFT, SG_INFO, "Instance of model " << path
                    << " has invalid values");
             return;
@@ -231,14 +300,8 @@ struct UpdateFunctor : public std::unary_function<FGModelMgr::Instance*, void>
             model->setHeadingDeg(heading);
 
         instance->model->update();
-    }
-};
-}
-
-void
-FGModelMgr::update (double dt)
-{
-    std::for_each(_instances.begin(), _instances.end(), UpdateFunctor());
+        instance->checkLoaded();
+    });
 }
 
 void
@@ -260,31 +323,55 @@ FGModelMgr::remove_instance (Instance * instance)
     }
 }
 
-
+FGModelMgr::Instance*
+FGModelMgr::findInstanceByNodePath(const std::string& node_path) const
+{
+    if (node_path.empty())
+        return nullptr;
+
+    SGPropertyNode* node = fgGetNode(node_path, false);
+    if (!node)
+        return nullptr;
+    
+    auto it = std::find_if(_instances.begin(), _instances.end(),
+                           [node](const Instance* instance)
+    { return instance->node == node; });
+    
+    if (it == _instances.end()) {
+        return nullptr;
+    }
+    
+    return *it;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGModelMgr::Instance
 ////////////////////////////////////////////////////////////////////////
 
-FGModelMgr::Instance::Instance ()
-  : model(0),
-    node(0),
-    lon_deg_node(0),
-    lat_deg_node(0),
-    elev_ft_node(0),
-    roll_deg_node(0),
-    pitch_deg_node(0),
-    heading_deg_node(0),
-    shadow(false)
-{
-}
-
 FGModelMgr::Instance::~Instance ()
 {
-  delete model;
+    delete model;
 }
 
+bool FGModelMgr::Instance::checkLoaded() const
+{
+    if (!model)
+        return false;
+    
+    if (loaded_node->getBoolValue()) {
+        return true;
+    }
+    
+    CheckInstanceModelLoadedVisitor cilv;
+    model->getSceneGraph()->accept(cilv);
+    const bool loadedNow = cilv.isLoaded();
+    
+    if (loadedNow) {
+        loaded_node->setBoolValue(true);
+    }
+    return loadedNow;
+}
 
-
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGModelMgr::Listener
 ////////////////////////////////////////////////////////////////////////
@@ -292,7 +379,7 @@ FGModelMgr::Instance::~Instance ()
 void
 FGModelMgr::Listener::childAdded(SGPropertyNode * parent, SGPropertyNode * child)
 {
-  if (strcmp(parent->getName(), "model") || strcmp(child->getName(), "load"))
+  if (parent->getNameString() != "model" || child->getNameString() != "load")
     return;
 
   _mgr->add_model(parent);
@@ -301,12 +388,12 @@ FGModelMgr::Listener::childAdded(SGPropertyNode * parent, SGPropertyNode * child
 void
 FGModelMgr::Listener::childRemoved(SGPropertyNode * parent, SGPropertyNode * child)
 {
-  if (strcmp(parent->getName(), "models") || strcmp(child->getName(), "model"))
+  if (parent->getNameString() != "models" || child->getNameString() != "model")
     return;
 
   // search instance by node and remove it from scenegraph
-    std::vector<Instance *>::iterator it = _mgr->_instances.begin();
-    std::vector<Instance *>::iterator end = _mgr->_instances.end();
+  std::vector<Instance *>::iterator it = _mgr->_instances.begin();
+  std::vector<Instance *>::iterator end = _mgr->_instances.end();
 
   for (; it != end; ++it) {
     Instance *instance = *it;
