@@ -82,7 +82,8 @@ const char* restrictionToString(RouteRestriction aRestrict);
 typedef std::vector<FlightPlan::DelegateFactoryRef> FPDelegateFactoryVec;
 static FPDelegateFactoryVec static_delegateFactories;
   
-FlightPlan::FlightPlan() :
+FlightPlan::FlightPlan(bool isRoute) :
+    _isRoute(isRoute),
   _currentIndex(-1),
     _followLegTrackToFix(true),
     _aircraftCategory(ICAO_AIRCRAFT_CATEGORY_C),
@@ -104,7 +105,17 @@ FlightPlan::FlightPlan() :
     }
   }
 }
-  
+
+FlightPlanRef FlightPlan::create()
+{
+    return new FlightPlan(false);
+}
+
+FlightPlanRef FlightPlan::createRoute()
+{
+    return new FlightPlan(true);
+}
+
 FlightPlan::~FlightPlan()
 {
 // clean up delegates
@@ -116,9 +127,12 @@ FlightPlan::~FlightPlan()
     }
 }
   
-FlightPlan* FlightPlan::clone(const string& newIdent) const
+FlightPlanRef FlightPlan::clone(const string& newIdent, bool convertIntoFlightPlan) const
 {
-  FlightPlan* c = new FlightPlan();
+    // this is the only place we allow conversion of a route into an active FP,
+    // by design. Forces people to clone-to-a-flight-plan if they want to
+    // activate a route.
+  FlightPlanRef c = new FlightPlan(convertIntoFlightPlan ? false : _isRoute);
   c->_ident = newIdent.empty() ? _ident : newIdent;
   c->lockDelegates();
   
@@ -165,6 +179,8 @@ FlightPlan* FlightPlan::clone(const string& newIdent) const
   for (int l=0; l < numLegs(); ++l) {
     c->_legs.push_back(_legs[l]->cloneFor(c));
   }
+    
+    c->expandVias();
   c->unlockDelegates();
   return c;
 }
@@ -197,25 +213,55 @@ FlightPlan::LegRef FlightPlan::insertWayptAtIndex(Waypt* aWpt, int aIndex)
   return legAtIndex(index);
 }
   
+static WayptVec copyWaypointsExpandingVias(WayptRef preceeding, const WayptVec& wps)
+{
+    WayptVec result;
+    result.reserve(wps.size());
+    
+    for (auto wp : wps) {
+        if (wp->type() == "via") {
+            Via* via = static_cast<Via*>(wp.get());
+            WayptVec viaPoints = via->expandToWaypoints(preceeding);
+            result.insert(result.end(), viaPoints.begin(), viaPoints.end());
+        } else {
+            // everything else is copied directly
+            result.push_back(wp);
+        }
+    }
+    
+    return result;
+}
+
 void FlightPlan::insertWayptsAtIndex(const WayptVec& wps, int aIndex)
 {
   if (wps.empty()) {
     return;
   }
   
-  int index = aIndex;
-  if ((aIndex == -1) || (aIndex > (int) _legs.size())) {
-    index = _legs.size();
-  }
-  
+    int index = aIndex;
+    if ((aIndex == -1) || (aIndex > (int) _legs.size())) {
+      index = _legs.size();
+    }
+    
+    WayptVec toInsertWps = wps;
+    // catch insert of VIAs here
+    if (!_isRoute && (index > 0)) {
+        const auto pre = _legs.at(index - 1)->waypoint();
+        toInsertWps = copyWaypointsExpandingVias(pre, wps);
+    } else if (index == 0) {
+        if (wps.front()->type() == "via") {
+            SG_LOG(SG_AUTOPILOT, SG_DEV_ALERT, "Inserting a VIA at leg 0 of flight-plan, VIA cannot be expanded");
+        }
+    }
+   
   auto it = _legs.begin() + index;
-  int endIndex = index + wps.size() - 1;
+  int endIndex = index + toInsertWps.size() - 1;
   if (_currentIndex >= endIndex) {
-    _currentIndex += wps.size();
+    _currentIndex += toInsertWps.size();
   }
  
   LegVec newLegs;
-  for (WayptRef wp : wps) {
+  for (WayptRef wp : toInsertWps) {
       newLegs.push_back(LegRef{new Leg(this, wp)});
   }
   
@@ -362,9 +408,16 @@ int FlightPlan::clearWayptsWithFlag(WayptFlag flag)
   unlockDelegates();
   return numDeleted;
 }
-    
+
+bool FlightPlan::isRoute() const
+{
+    return _isRoute;
+}
+
 bool FlightPlan::isActive() const
 {
+    if (_isRoute)
+        return false;
     return (_currentIndex >= 0);
 }
 
@@ -395,6 +448,10 @@ void FlightPlan::sequence()
     
 void FlightPlan::finish()
 {
+    if (_isRoute) {
+        throw sg_exception("Called finish on FlightPlan marked isRoute");
+    }
+    
   if (_currentIndex == -1) {
     return;
   }
@@ -505,6 +562,11 @@ void FlightPlan::setDeparture(FGRunway* rwy)
     _departure = rwy->airport();
     clearSID();
   }
+
+  if (_sid && !_sid->isForRunway(_departureRunway)) {
+      clearSID();
+  }
+
   unlockDelegates();
 }
   
@@ -546,6 +608,10 @@ void FlightPlan::setSID(Transition* trans)
   
 void FlightPlan::clearSID()
 {
+    if (_sid == nullptr) {
+        return;
+    }
+
   lockDelegates();
   _departureChanged = true;
   _sid = nullptr;
@@ -590,7 +656,11 @@ void FlightPlan::setDestination(FGRunway* rwy)
     _destination = rwy->airport();
     clearSTAR();
   }
-  
+
+  if (_star && !_star->isForRunway(_destinationRunway)) {
+      clearSTAR();
+  }
+
   unlockDelegates();
 }
   
@@ -646,6 +716,9 @@ void FlightPlan::setSTAR(Transition* trans)
 
 void FlightPlan::clearSTAR()
 {
+    if (_star == nullptr) {
+        return;
+    }
 
   lockDelegates();
   _arrivalChanged = true;
@@ -760,6 +833,10 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
     d->setIntValue("version", 2);
     
     // general data
+    if (_isRoute) {
+        d->setBoolValue("is-route", true);
+    }
+    
     d->setStringValue("flight-rules", static_icaoFlightRulesCode[static_cast<int>(_flightRules)]);
     d->setStringValue("flight-type", static_icaoFlightTypeCode[static_cast<int>(_flightType)]);
     if (!_callsign.empty()) {
@@ -868,6 +945,10 @@ bool FlightPlan::load(const SGPath& path)
         _departureChanged = true;
         Status = true;
     } else if (loadXmlFormat(path)) { // XML property data
+        if (!_isRoute) {
+            expandVias();
+        }
+        
         // we don't want to re-compute the arrival / departure after
         // a load, since we assume the flight-plan had it specified already
         // especially, the XML might have a SID/STAR embedded, which we don't
@@ -875,7 +956,7 @@ bool FlightPlan::load(const SGPath& path)
         
         // however, we do want to run the normal delegate if no procedure was
         // defined. We'll use the presence of waypoints tagged to decide
-        const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL);
+        const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL) || anyWaypointsWithFlag(this, WPT_APPROACH);
         const bool hasDeparture = anyWaypointsWithFlag(this, WPT_DEPARTURE);
         _arrivalChanged = !hasArrival;
         _departureChanged = !hasDeparture;
@@ -884,6 +965,10 @@ bool FlightPlan::load(const SGPath& path)
         _arrivalChanged = true;
         _departureChanged = true;
         Status = true;
+        
+        if (!_isRoute) {
+            expandVias(); // plain text could in principle contain VIAs
+        }
     }
     
     if (Status == true) {
@@ -929,6 +1014,10 @@ bool FlightPlan::load(std::istream &stream)
         Status = false;
     }
     
+    if (!_isRoute) {
+        expandVias();
+    }
+    
     // we don't want to re-compute the arrival / departure after
      // a load, since we assume the flight-plan had it specified already
      // especially, the XML might have a SID/STAR embedded, which we don't
@@ -1114,7 +1203,6 @@ bool FlightPlan::loadXmlFormat(const SGPath& path)
 void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
 {
     // general info
-
     const auto rules = routeData->getStringValue("flight-rules", "V");
     auto it = std::find(static_icaoFlightRulesCode.begin(),
                         static_icaoFlightRulesCode.end(), rules);
@@ -1130,6 +1218,13 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
     _aircraftType = routeData->getStringValue("aircraft-type");
     _estimatedDuration = routeData->getIntValue("estimated-duration-minutes");
 
+    if (routeData->hasValue("is-route")) {
+        if (_isRoute != routeData->getBoolValue("is-route")) {
+            // this is actually okay, we will expand any VIAs
+            SG_LOG(SG_NAVAID, SG_INFO, "Loading XML marked with 'is-route' into FlightPlan with is-route not set");
+        }
+    }
+    
   // departure nodes
   SGPropertyNode* dep = routeData->getChild("departure");
   if (dep) {
@@ -1147,9 +1242,14 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
           // to parse this value as a SID, and look for a seperate sid_trans
           // value
           const string trans = dep->getStringValue("sid_trans");
-          const auto sid = dep->getStringValue("sid");
-          setSID(_departure->findSIDWithIdent(sid), trans);
-      }
+          const auto sidIdent = dep->getStringValue("sid");
+          auto sid = _departure->findSIDWithIdent(sidIdent);
+          if (!sid) {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown SID:" << sidIdent);
+          } else {
+              setSID(sid, trans);
+          }
+      } // of have a SID identifier
     }
   }
   
@@ -1166,16 +1266,26 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
       if (dst->hasChild("star")) {
           // prior to 2020.2 we would attempt to treat 'star' as a
           // transiiton ID, but this is ambiguous. Look for a seperate value now
-          const auto star = dst->getStringValue("star");
+          const auto starIdent = dst->getStringValue("star");
           const string trans = dst->getStringValue("star_trans");
-          setSTAR(_destination->findSTARWithIdent(star), trans);
+          auto star = _destination->findSTARWithIdent(starIdent);
+
+          if (star) {
+              setSTAR(star, trans);
+          } else {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown STAR:" << starIdent);
+          }
       } // of STAR processing
       
       if (dst->hasChild("approach")) {
           auto app = _destination->findApproachWithIdent(dst->getStringValue("approach"));
           const auto trans = dst->getStringValue("approach_trans");
-          setApproach(app, trans);
-      }
+          if (app) {
+              setApproach(app, trans);
+          } else {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown approach:" << dst->getStringValue("approach"));
+          }
+      } // of have approach in the XML
     }
   }
   
@@ -1222,16 +1332,7 @@ bool FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
             continue;
         }
 
-      LegRef l = new Leg{this, wp};
-      // sync leg restrictions with waypoint ones
-        if (wp->speedRestriction() != RESTRICT_NONE) {
-            l->setSpeed(wp->speedRestriction(), wp->speed());
-        }
-        
-        if (wp->altitudeRestriction() != RESTRICT_NONE) {
-            l->setAltitude(wp->altitudeRestriction(), wp->altitudeFt());
-        }
-        
+        LegRef l = new Leg{this, wp};
         if (wpNode->hasChild("hold-count")) {
             l->setHoldCount(wpNode->getIntValue("hold-count"));
         }
@@ -1304,7 +1405,7 @@ WayptRef FlightPlan::parseVersion1XMLWaypt(SGPropertyNode* aWP)
   
   double altFt = aWP->getDoubleValue("altitude-ft", -9999.9);
   if (altFt > -9990.0) {
-    w->setAltitude(altFt, RESTRICT_AT);
+    w->setAltitude(altFt, RESTRICT_AT, ALTITUDE_FEET);
   }
   
   return w;
@@ -1332,7 +1433,7 @@ bool FlightPlan::loadPlainTextFormat(const SGPath& path)
       if (line.empty() || (line[0] == '#')) {
         continue; // ignore empty/comment lines
       }
-      
+
       // prevent Sentry error 'FLIGHTGEAR-J6', when we try loading XML
       // data here
       if (simgear::strutils::starts_with(line, "<?xml")) {
@@ -1367,6 +1468,27 @@ double FlightPlan::magvarDegAt(const SGGeod& pos) const
   return sgGetMagVar(pos, jd) * SG_RADIANS_TO_DEGREES;
 }
 
+SGGeod FlightPlan::vicinityForInsertIndex(int aIndex) const
+{
+    if (aIndex < 0 || aIndex >= numLegs()) { // appending, not inserting
+        const int n = numLegs();
+        if (n > 0) {
+            // if we have at least one existing leg, use its position
+            // for the search vicinity
+            return pointAlongRoute(n - 1, 0.0);
+        }
+
+        return SGGeod::invalid();
+    }
+
+    // if we're somewhere in the middle of the route compute a search
+    // vicinity halfway between the previous waypoint and the one we are
+    // inserting at, i.e the middle of the leg.
+    // if we're at the beginning, just use zero of course.
+    const double normOffset = (aIndex > 0) ? -0.5 : 0.0;
+    return pointAlongRouteNorm(aIndex, normOffset);
+}
+
 WayptRef FlightPlan::waypointFromString(const string& tgt, const SGGeod& vicinity)
 {
     SGGeod basePosition = vicinity;
@@ -1383,9 +1505,52 @@ WayptRef FlightPlan::waypointFromString(const string& tgt, const SGGeod& vicinit
     return Waypt::createFromString(this, tgt, basePosition);
 }
 
+bool FlightPlan::expandVias()
+{
+    // must be called with the delegats locked, so that
+    // waypointsChanged can be set on finish
+    
+    assert(_delegateLock > 0);
+    bool didChangeAny = false;
+    
+    for (unsigned int i=1; i < _legs.size(); ) {
+      if (_legs[i]->waypoint()->type() == "via") {
+        WayptRef preceeding = _legs[i - 1]->waypoint();
+        Via* via = static_cast<Via*>(_legs[i]->waypoint());
+        WayptVec wps = via->expandToWaypoints(preceeding);
+        
+        // delete the VIA leg
+        auto it = _legs.begin() + i;
+        LegRef l = *it;
+        _legs.erase(it);
+        
+        // create new legs and insert
+          it = _legs.begin() + i;
+        
+        LegVec newLegs;
+        for (WayptRef wp : wps) {
+            newLegs.push_back(LegRef{new Leg(this, wp)});
+        }
+        
+          didChangeAny = true;
+        _legs.insert(it, newLegs.begin(), newLegs.end());
+      } else {
+        ++i; // normal case, no expansion
+      }
+    }
+    
+    return didChangeAny;
+}
+
 void FlightPlan::activate()
 {
-  FGRouteMgr* routeManager = globals->get_subsystem<FGRouteMgr>();
+    if (_isRoute) {
+        // no allowed, clone and make the non-route FP active
+        SG_LOG(SG_NAVAID, SG_DEV_ALERT, "tried to activate an is-route FlightPlan");
+        return;
+    }
+    
+  auto routeManager = globals->get_subsystem<FGRouteMgr>();
   if (routeManager) {
     if (routeManager->flightPlan() != this) {
       SG_LOG(SG_NAVAID, SG_DEBUG, "setting new flight-plan on route-manager");
@@ -1397,32 +1562,7 @@ void FlightPlan::activate()
   
   _currentIndex = 0;
   _currentWaypointChanged = true;
-  
-  for (unsigned int i=1; i < _legs.size(); ) {
-    if (_legs[i]->waypoint()->type() == "via") {
-      WayptRef preceeding = _legs[i - 1]->waypoint();
-      Via* via = static_cast<Via*>(_legs[i]->waypoint());
-      WayptVec wps = via->expandToWaypoints(preceeding);
-      
-      // delete the VIA leg
-      auto it = _legs.begin() + i;
-      LegRef l = *it;
-      _legs.erase(it);
-      
-      // create new legs and insert
-        it = _legs.begin() + i;
-      
-      LegVec newLegs;
-      for (WayptRef wp : wps) {
-          newLegs.push_back(LegRef{new Leg(this, wp)});
-      }
-      
-      _waypointsChanged = true;
-      _legs.insert(it, newLegs.begin(), newLegs.end());
-    } else {
-      ++i; // normal case, no expansion
-    }
-  }
+    _waypointsChanged = expandVias();
   
   for (auto d : _delegates) {
     d->activated();
@@ -1446,9 +1586,12 @@ FlightPlan::Leg* FlightPlan::Leg::cloneFor(FlightPlan* owner) const
 // clone local data
   c->_speed = _speed;
   c->_speedRestrict = _speedRestrict;
-  c->_altitudeFt = _altitudeFt;
+  c->_speedUnits = _speedUnits;
+  c->_altitude = _altitude;
   c->_altRestrict = _altRestrict;
-  
+  c->_altitudeUnits = _altitudeUnits;
+  c->_holdCount = c->_holdCount;
+
   return c;
 }
   
@@ -1465,36 +1608,37 @@ unsigned int FlightPlan::Leg::index() const
   return _parent->findLegIndex(this);
 }
 
-int FlightPlan::Leg::altitudeFt() const
+double FlightPlan::Leg::altitude(RouteUnits aUnits) const
 {
   if (_altRestrict != RESTRICT_NONE) {
-    return _altitudeFt;
+    return convertAltitudeUnits(_altitudeUnits, aUnits, _altitude);
   }
-  
-  return _waypt->altitudeFt();
+
+  return _waypt->altitude(aUnits);
 }
 
-int FlightPlan::Leg::speed() const
+int FlightPlan::Leg::altitudeFt() const
+{
+  return static_cast<int>(altitude(ALTITUDE_FEET));
+}
+
+double FlightPlan::Leg::speed(RouteUnits units) const
 {
   if (_speedRestrict != RESTRICT_NONE) {
-    return _speed;
+    return convertSpeedUnits(_speedUnits, units, altitudeFt(), _speed);
   }
-  
-  return _waypt->speed();
+
+  return _waypt->speed(units);
 }
 
 int FlightPlan::Leg::speedKts() const
 {
-  return speed();
+  return static_cast<int>(speed(SPEED_KNOTS));
 }
   
 double FlightPlan::Leg::speedMach() const
 {
-  if (!isMachRestrict(_speedRestrict)) {
-    return 0.0;
-  }
-  
-  return -(_speed / 100.0);
+  return speed(SPEED_MACH);
 }
 
 RouteRestriction FlightPlan::Leg::altitudeRestriction() const
@@ -1514,21 +1658,30 @@ RouteRestriction FlightPlan::Leg::speedRestriction() const
   
   return _waypt->speedRestriction();
 }
-  
-void FlightPlan::Leg::setSpeed(RouteRestriction ty, double speed)
+
+void FlightPlan::Leg::setSpeed(RouteRestriction ty, double speed, RouteUnits aUnit)
 {
   _speedRestrict = ty;
-  if (isMachRestrict(ty)) {
-    _speed = (speed * -100); 
-  } else {
-    _speed = speed;
+  if (aUnit == DEFAULT_UNITS) {
+    if (isMachRestrict(ty)) {
+      aUnit = SPEED_MACH;
+    } else {
+      // TODO: check for system in metric?
+      aUnit = SPEED_KNOTS;
+    }
   }
+  _speedUnits = aUnit;
+  _speed = speed;
 }
-  
-void FlightPlan::Leg::setAltitude(RouteRestriction ty, int altFt)
+
+void FlightPlan::Leg::setAltitude(RouteRestriction ty, double alt, RouteUnits aUnit)
 {
   _altRestrict = ty;
-  _altitudeFt = altFt;
+  if (aUnit == DEFAULT_UNITS) {
+    aUnit = ALTITUDE_FEET;
+  }
+  _altitudeUnits = aUnit;
+  _altitude = alt;
 }
 
 double FlightPlan::Leg::courseDeg() const
@@ -1604,16 +1757,24 @@ void FlightPlan::Leg::writeToProperties(SGPropertyNode* aProp) const
 {
     if (_speedRestrict != RESTRICT_NONE) {
         aProp->setStringValue("speed-restrict", restrictionToString(_speedRestrict));
-        if (_speedRestrict == SPEED_RESTRICT_MACH) {
-            aProp->setDoubleValue("speed", speedMach());
+        if (_speedUnits == SPEED_MACH) {
+      aProp->setDoubleValue("speed-mach", _speed);
+        } else if (_speedUnits == SPEED_KPH) {
+      aProp->setDoubleValue("speed-kph", _speed);
         } else {
-            aProp->setDoubleValue("speed", _speed);
+      aProp->setDoubleValue("speed", _speed);
         }
     }
   
     if (_altRestrict != RESTRICT_NONE) {
         aProp->setStringValue("alt-restrict", restrictionToString(_altRestrict));
-        aProp->setDoubleValue("altitude-ft", _altitudeFt);
+        if (_altitudeUnits == ALTITUDE_FLIGHTLEVEL) {
+      aProp->setDoubleValue("flight-level", _altitude);
+        } else if (_altitudeUnits == ALTITUDE_METER) {
+      aProp->setDoubleValue("altitude-m", _altitude);
+        } else {
+      aProp->setDoubleValue("altitude-ft", _altitude);
+        }
     }
     
     if (_holdCount > 0) {
@@ -1687,21 +1848,15 @@ void FlightPlan::lockDelegates()
   }
 }
 
-void FlightPlan::unlockDelegates()
+void FlightPlan::runDelegates()
 {
-  assert(_delegateLock > 0);
-  if (_delegateLock > 1) {
-    --_delegateLock;
-    return;
-  }
-  
     if (_didLoadFP) {
         _didLoadFP = false;
         for (auto d : _delegates) {
           d->loaded();
         }
     }
-    
+
   if (_departureChanged) {
     _departureChanged = false;
     for (auto d : _delegates) {
@@ -1737,10 +1892,28 @@ void FlightPlan::unlockDelegates()
       d->currentWaypointChanged();
     }
   }
-  
-  --_delegateLock;
+
+  const auto cascadingUpdate = _departureChanged | _arrivalChanged | _waypointsChanged | _currentWaypointChanged;
+  if (cascadingUpdate) {
+      // one of our callbacks, caused a cascading change, so go around again.
+      // we assume the delegates will converge after one or more cascades, so we don't check for infinite looping here
+      SG_LOG(SG_AUTOPILOT, SG_INFO, "FlightPlan::runDelegates will re-run");
+      runDelegates();
+  }
 }
-  
+
+void FlightPlan::unlockDelegates()
+{
+    assert(_delegateLock > 0);
+    if (_delegateLock > 1) {
+        --_delegateLock;
+        return;
+    }
+
+    runDelegates();
+    --_delegateLock;
+}
+
 void FlightPlan::registerDelegateFactory(DelegateFactoryRef df)
 {
   auto it = std::find(static_delegateFactories.begin(), static_delegateFactories.end(), df);
@@ -1804,6 +1977,16 @@ bool FlightPlan::followLegTrackToFixes() const
     return _followLegTrackToFix;
 }
 
+void FlightPlan::setMaxFlyByTurnAngle(double deg)
+{
+    _maxFlyByTurnAngle = deg;
+}
+
+double FlightPlan::maxFlyByTurnAngle() const
+{
+    return _maxFlyByTurnAngle;
+}
+
 std::string FlightPlan::icaoAircraftCategory() const
 {
     std::string r;
@@ -1984,9 +2167,6 @@ bool FlightPlan::parseICAORouteString(const std::string& routeData)
 
     SG_LOG(SG_AUTOPILOT, SG_INFO, "adding waypoints from string");
     // rebuild legs from waypoints we created
-    for (auto l : _legs) {
-        delete l;
-    }
     _legs.clear();
     insertWayptsAtIndex(enroute, 0);
 
@@ -2164,9 +2344,9 @@ void FlightPlan::forEachLeg(const LegVisitor& lv)
 int FlightPlan::indexOfFirstNonDepartureWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (!(_legs.at(i)->waypoint()->flags() & WPT_DEPARTURE))
-            return i ;
+            return static_cast<int>(i);
     }
     
     // all waypoints are marked as departure
@@ -2176,9 +2356,9 @@ int FlightPlan::indexOfFirstNonDepartureWaypoint() const
 int FlightPlan::indexOfFirstArrivalWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (_legs.at(i)->waypoint()->flags() & WPT_ARRIVAL)
-            return i;
+            return static_cast<int>(i);
     }
     
     // no waypoints are marked as arrival
@@ -2188,9 +2368,9 @@ int FlightPlan::indexOfFirstArrivalWaypoint() const
 int FlightPlan::indexOfFirstApproachWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (_legs.at(i)->waypoint()->flags() & WPT_APPROACH)
-            return i;
+            return static_cast<int>(i);
     }
     
     // no waypoints are marked as arrival
