@@ -212,8 +212,7 @@ TurnInfo turnCenterAndAngleFromExit(const SGGeod& pt, double outHeadingDeg,
     double courseToTC, distanceToTC, az2;
     SGGeodesy::inverse(origin, r.turnCenter, courseToTC, az2, distanceToTC);
     if (distanceToTC < turnRadiusM) {
-        SG_LOG(SG_NAVAID, SG_WARN, "turnCenterAndAngleFromExit: origin point too close to turn center");
-        return r;
+        return r; // invalid
     }
 
     // find additional course angle away from the exit pos to intersect
@@ -368,12 +367,20 @@ public:
       legCourseValid = true;
       return;
     }
+      
+      SGGeod previousPos = previous->pos;
+      // use correct location for runway, otherwise use threshold
+      // and get weird courses
+      if (previous->wpt->type() == "runway") {
+          FGRunway* rwy = static_cast<RunwayWaypt*>(previous->wpt.get())->runway();
+          previousPos = rwy->end();
+      }
     
     if (posValid) {
-      legCourseTrue = SGGeodesy::courseDeg(previous->pos, pos);
+      legCourseTrue = SGGeodesy::courseDeg(previousPos, pos);
       legCourseValid = true;
     } else if (isCourseConstrained()) {
-      double magVar = magVarFor(previous->pos);
+      double magVar = magVarFor(previousPos);
       legCourseTrue = wpt->headingRadialDeg() + magVar;
       legCourseValid = true;
     } // of pos not valid
@@ -397,113 +404,119 @@ public:
         return turnRadius * fabs(angleDeg) * SG_DEGREES_TO_RADIANS;
     }
 
-  void computeTurn(double radiusM, bool constrainLegCourse, WayptData& next)
-  {
-    assert(!skipped);
-    assert(next.legCourseValid);
-      bool isRunway = (wpt->type() == "runway");
+    void computeTurn(double radiusM, bool constrainLegCourse, double maxFlyByTurnAngleDeg, WayptData& next)
+    {
+        assert(!skipped);
+        assert(next.legCourseValid);
+        bool isRunway = (wpt->type() == "runway");
 
-      if (legCourseValid) {
-          if (isRunway) {
-              FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
-              turnExitAngle = next.legCourseTrue - rwy->headingDeg();
-          } else {
-              turnExitAngle = next.legCourseTrue - legCourseTrue;
-          }
-      } else {
-          // happens for first leg
-          if (isRunway) {
-              legCourseValid = true;
-              FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
-              turnExitAngle = next.legCourseTrue - rwy->headingDeg();
-              legCourseTrue = rwy->headingDeg();
-              flyOver = true;
-          } else {
-              legCourseValid = true;
-              legCourseTrue = next.legCourseTrue;
-              turnExitAngle = 0.0;
-              turnExitPos = pos;
-              flyOver = true;
-              return;
-          }
-      }
-
-    SG_NORMALIZE_RANGE(turnExitAngle, -180.0, 180.0);
-    turnRadius = radiusM;
-
-    if (!flyOver && fabs(turnExitAngle) > 120.0) {
-        // flyBy logic blows up for sharp turns - due to the tan() term
-        // heading towards infinity. By converting to flyOver we do something
-        // closer to what was requested.
-        flyOver = true;
-    }
-
-    if (flyOver) {
-        if (isRunway) {
-            FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
-            turnExitCenter = turnCenterOverflight(rwy->end(), rwy->headingDeg(), turnExitAngle, turnRadius);
+        if (legCourseValid) {
+            if (isRunway) {
+                FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
+                turnExitAngle = next.legCourseTrue - rwy->headingDeg();
+            } else {
+                turnExitAngle = next.legCourseTrue - legCourseTrue;
+            }
         } else {
-            turnEntryPos = pos;
-            turnExitCenter = turnCenterOverflight(pos, legCourseTrue, turnExitAngle, turnRadius);
-
+            // happens for first leg
+            if (isRunway) {
+                legCourseValid = true;
+                FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
+                turnExitAngle = next.legCourseTrue - rwy->headingDeg();
+                legCourseTrue = rwy->headingDeg();
+                flyOver = true;
+                // fall through
+            } else {
+                legCourseValid = true;
+                legCourseTrue = next.legCourseTrue;
+                turnExitAngle = 0.0;
+                turnExitPos = pos;
+                flyOver = true;
+                return;
+            }
         }
-      turnExitPos = pointOnExitTurnFromHeading(next.legCourseTrue);
-      
-      if (!next.wpt->flag(WPT_DYNAMIC)) {
-        // distance perpendicular to next leg course, after turning
-        // through turnAngle
-        double xtk = turnRadius * (1 - cos(turnExitAngle * SG_DEGREES_TO_RADIANS));
 
-        if (constrainLegCourse || next.isCourseConstrained()) {
-          // next leg course is constrained. We need to swing back onto the
-          // desired course, using a compensation turn
-          
-          // compensation angle to turn back on course
-          double theta = acos((turnRadius - (xtk * 0.5)) / turnRadius) * SG_RADIANS_TO_DEGREES;
-          theta = copysign(theta, turnExitAngle);
-          turnExitAngle += theta;
-        
-          // move by the distance to compensate
-          double d = turnRadius * 2.0 * sin(theta * SG_DEGREES_TO_RADIANS);
-          turnExitPos = SGGeodesy::direct(turnExitPos, next.legCourseTrue, d);
-          overflightCompensationAngle = -theta;
+        SG_NORMALIZE_RANGE(turnExitAngle, -180.0, 180.0);
+        turnRadius = radiusM;
 
-            // sign of angles will differ, so compute distances seperately
-            turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle) +
-                pathDistanceForTurnAngle(overflightCompensationAngle);
+        if (!flyOver && fabs(turnExitAngle) > maxFlyByTurnAngleDeg) {
+            // flyBy logic blows up for sharp turns - due to the tan() term
+            // heading towards infinity. By converting to flyOver we do something
+            // closer to what was requested. This matches logic in the RNAV
+            // Leg controller
+            flyOver = true;
+        }
+
+        if (flyOver) {
+            if (isRunway) {
+                FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
+                turnExitCenter = turnCenterOverflight(rwy->end(), rwy->headingDeg(), turnExitAngle, turnRadius);
+            } else {
+                turnEntryPos = pos;
+                turnExitCenter = turnCenterOverflight(pos, legCourseTrue, turnExitAngle, turnRadius);
+            }
+            turnExitPos = pointOnExitTurnFromHeading(next.legCourseTrue);
+
+            if (!next.wpt->flag(WPT_DYNAMIC)) {
+                // distance perpendicular to next leg course, after turning
+                // through turnAngle
+                double xtk = turnRadius * (1 - cos(turnExitAngle * SG_DEGREES_TO_RADIANS));
+
+                if (constrainLegCourse || next.isCourseConstrained()) {
+                    // next leg course is constrained. We need to swing back onto the
+                    // desired course, using a compensation turn
+
+                    // compensation angle to turn back on course
+                    double theta = acos((turnRadius - (xtk * 0.5)) / turnRadius) * SG_RADIANS_TO_DEGREES;
+                    theta = copysign(theta, turnExitAngle);
+                    turnExitAngle += theta;
+
+                    double p = copysign(90, turnExitAngle);
+                    const double offsetAngle = legCourseTrue + turnExitAngle - p;
+                    SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
+                                                   offsetAngle,
+                                                   turnRadius * 2.0);
+                    
+       
+                    turnExitPos = SGGeodesy::direct(tc2, next.legCourseTrue + p , turnRadius);
+                    overflightCompensationAngle = -theta;
+
+                    // sign of angles will differ, so compute distances seperately
+                    turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle) +
+                                        pathDistanceForTurnAngle(overflightCompensationAngle);
+                } else {
+                    // next leg course can be adjusted. increase the turn angle
+                    // and modify the next leg's course accordingly.
+
+                    // hypotenuse of triangle, opposite edge has length turnRadius
+                    double distAlongPath = std::min(1.0, sin(fabs(turnExitAngle) * SG_DEGREES_TO_RADIANS)) * turnRadius;
+                    double nextLegDistance = SGGeodesy::distanceM(pos, next.pos) - distAlongPath;
+                    double increaseAngle = atan2(xtk, nextLegDistance) * SG_RADIANS_TO_DEGREES;
+                    increaseAngle = copysign(increaseAngle, turnExitAngle);
+
+                    turnExitAngle += increaseAngle;
+                    turnExitPos = pointOnExitTurnFromHeading(legCourseTrue + turnExitAngle);
+                    // modify next leg course
+                    next.legCourseTrue = SGGeodesy::courseDeg(turnExitPos, next.pos);
+                    turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle);
+                } // of next leg isn't course constrained
+            } else {
+                // next point is dynamic
+                // no compensation needed
+                turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle);
+            }
         } else {
-          // next leg course can be adjusted. increase the turn angle
-          // and modify the next leg's course accordingly.
+            hasEntry = true;
+            turnEntryCenter = turnCenterFlyBy(pos, legCourseTrue, turnExitAngle, turnRadius);
 
-          // hypotenuse of triangle, opposite edge has length turnRadius
-          double distAlongPath = std::min(1.0, sin(fabs(turnExitAngle) * SG_DEGREES_TO_RADIANS)) * turnRadius;
-          double nextLegDistance = SGGeodesy::distanceM(pos, next.pos) - distAlongPath;
-          double increaseAngle = atan2(xtk, nextLegDistance) * SG_RADIANS_TO_DEGREES;
-          increaseAngle = copysign(increaseAngle, turnExitAngle);
+            turnExitAngle = turnExitAngle * 0.5;
+            turnEntryAngle = turnExitAngle;
+            turnExitCenter = turnEntryCenter; // important that these match
 
-          turnExitAngle += increaseAngle;
-          turnExitPos = pointOnExitTurnFromHeading(legCourseTrue + turnExitAngle);
-          // modify next leg course
-          next.legCourseTrue = SGGeodesy::courseDeg(turnExitPos, next.pos);
-          turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle);
-        } // of next leg isn't course constrained
-      } else {
-          // next point is dynamic
-          // no compensation needed
-          turnPathDistanceM = pathDistanceForTurnAngle(turnExitAngle);
-      }
-    } else {
-      hasEntry = true;
-        turnEntryCenter = turnCenterFlyBy(pos, legCourseTrue, turnExitAngle, turnRadius);
-
-        turnExitAngle = turnExitAngle * 0.5;
-        turnEntryAngle = turnExitAngle;
-        turnExitCenter = turnEntryCenter; // important that these match
-
-      turnEntryPos = pointOnEntryTurnFromHeading(legCourseTrue);
-      turnExitPos = pointOnExitTurnFromHeading(next.legCourseTrue);
-      turnPathDistanceM = pathDistanceForTurnAngle(turnEntryAngle);
-    }
+            turnEntryPos = pointOnEntryTurnFromHeading(legCourseTrue);
+            turnExitPos = pointOnExitTurnFromHeading(next.legCourseTrue);
+            turnPathDistanceM = pathDistanceForTurnAngle(turnEntryAngle);
+        }
   }
   
   double turnDistanceM() const
@@ -576,23 +589,28 @@ public:
 
   SGGeod pointAlongExitPath(double distanceM) const
   {
-      double theta = (distanceM / turnRadius) * SG_RADIANS_TO_DEGREES;
-      double p = copysign(90, turnExitAngle);
+      double basicTurnDistance = pathDistanceForTurnAngle(turnExitAngle);
+      
+      if (distanceM > basicTurnDistance) {
+          assert(flyOver);
+          assert(overflightCompensationAngle != 0.0);
+          double p = copysign(90, turnExitAngle);
 
-      if (flyOver && (overflightCompensationAngle != 0.0)) {
-          // figure out if we're in the compensation section
-          if (theta > turnExitAngle) {
-              // compute the compensation turn center - twice the turn radius
-              // from turnCenter
-              SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
-                                             legCourseTrue - overflightCompensationAngle - p,
-                                             turnRadius * 2.0);
-              theta = copysign(theta - turnExitAngle, overflightCompensationAngle);
-              return SGGeodesy::direct(tc2,
-                                       legCourseTrue - overflightCompensationAngle + theta + p, turnRadius);
-          }
+          double distanceAlongCompensationTurn = distanceM - basicTurnDistance;
+          double theta = (distanceAlongCompensationTurn / turnRadius) * SG_RADIANS_TO_DEGREES;
+
+          // compute the compensation turn center - twice the turn radius
+          // from turnCenter
+          SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
+                                         legCourseTrue + turnExitAngle - p,
+                                         turnRadius * 2.0);
+
+          theta = copysign(theta, overflightCompensationAngle);
+          return SGGeodesy::direct(tc2,
+                                   legCourseTrue + turnExitAngle + theta + p, turnRadius);
       }
-
+      
+      double theta = (distanceM / turnRadius) * SG_RADIANS_TO_DEGREES;
       theta = copysign(theta, turnExitAngle);
       double inboundCourse = legCourseTrue + (flyOver ? 0.0 : turnExitAngle);
       return pointOnExitTurnFromHeading(inboundCourse + theta);
@@ -628,87 +646,83 @@ public:
 
     AircraftPerformance perf;
     bool constrainLegCourses;
+    double maxFlyByTurnAngleDeg = 90.0;
 
-  void computeDynamicPosition(int index)
-  {
-    auto previous(previousValidWaypoint(index));
-    if ((previous == waypoints.end()) || !previous->posValid)
+    void computeDynamicPosition(int index)
     {
-      SG_LOG(SG_NAVAID, SG_WARN, "couldn't compute position for dynamic waypoint: no preceeding valid waypoint");
-      return;
-    }
-    
-    WayptRef wpt = waypoints[index].wpt;
-
-    const std::string& ty(wpt->type());
-    if (ty == "hdgToAlt") {
-      HeadingToAltitude* h = (HeadingToAltitude*) wpt.get();
-      
-      double altFt = computeVNAVAltitudeFt(index - 1);
-      double distanceM = perf.distanceNmBetween(altFt, h->altitudeFt()) * SG_NM_TO_METER;
-      double hdg = h->headingDegMagnetic() + magVarFor(previous->pos);
-      waypoints[index].pos = SGGeodesy::direct(previous->turnExitPos, hdg, distanceM);
-      waypoints[index].posValid = true;
-    } else if (ty == "radialIntercept") {
-      // start from previous.turnExit
-      RadialIntercept* i = (RadialIntercept*) wpt.get();
-      
-      SGGeoc prevGc = SGGeoc::fromGeod(previous->turnExitPos);
-      SGGeoc navid = SGGeoc::fromGeod(wpt->position());
-      SGGeoc rGc;
-      double magVar = magVarFor(previous->pos);
-      
-      double radial = i->radialDegMagnetic() + magVar;
-      double track = i->courseDegMagnetic() + magVar;
-      bool ok = geocRadialIntersection(prevGc, track, navid, radial, rGc);
-      if (!ok) {
-        SGGeoc navidAdjusted;
-        // try pulling backward along the radial in case we're too close.
-        // suggests bad procedure construction if this is happening!
-        SGGeodesy::advanceRadM(navid, radial, SG_NM_TO_METER * -10, navidAdjusted);
-
-        // try again
-        ok = geocRadialIntersection(prevGc, track, navidAdjusted, radial, rGc);
-        if (!ok) {
-          SG_LOG(SG_NAVAID, SG_WARN, "couldn't compute interception for radial:"
-               << previous->turnExitPos << " / " << track << "/" << wpt->position()
-               << "/" << radial);
-          waypoints[index].pos = wpt->position(); // horrible fallback
-
-        } else {
-          waypoints[index].pos = SGGeod::fromGeoc(rGc);
+        auto previous(previousValidWaypoint(index));
+        if ((previous == waypoints.end()) || !previous->posValid) {
+            SG_LOG(SG_NAVAID, SG_WARN, "couldn't compute position for dynamic waypoint: no preceeding valid waypoint");
+            return;
+        }
+
+        WayptRef wpt = waypoints[index].wpt;
+
+        const std::string& ty(wpt->type());
+        if (ty == "hdgToAlt") {
+            HeadingToAltitude* h = (HeadingToAltitude*)wpt.get();
+
+            double altFt = computeVNAVAltitudeFt(index - 1);
+            double distanceM = perf.distanceNmBetween(altFt, h->altitudeFt()) * SG_NM_TO_METER;
+            double hdg = h->headingDegMagnetic() + magVarFor(previous->pos);
+            waypoints[index].pos = SGGeodesy::direct(previous->turnExitPos, hdg, distanceM);
+            waypoints[index].posValid = true;
+        } else if (ty == "radialIntercept") {
+            // start from previous.turnExit
+            RadialIntercept* i = (RadialIntercept*)wpt.get();
+
+            SGGeoc prevGc = SGGeoc::fromGeod(previous->turnExitPos);
+            SGGeoc navid = SGGeoc::fromGeod(wpt->position());
+            SGGeoc rGc;
+            double magVar = magVarFor(previous->pos);
+
+            double radial = i->radialDegMagnetic() + magVar;
+            double track = i->courseDegMagnetic() + magVar;
+            bool ok = geocRadialIntersection(prevGc, track, navid, radial, rGc);
+            if (!ok) {
+                // try pulling backward along the radial in case we're too close.
+                // suggests bad procedure construction if this is happening!
+                SGGeoc navidAdjusted = SGGeodesy::advanceDegM(navid, radial, -10 * SG_NM_TO_METER);
+
+                // try again
+                ok = geocRadialIntersection(prevGc, track, navidAdjusted, radial, rGc);
+                if (!ok) {
+                    SG_LOG(SG_NAVAID, SG_WARN, "couldn't compute interception for radial:" << previous->turnExitPos << " / " << track << "/" << wpt->position() << "/" << radial);
+                    waypoints[index].pos = wpt->position(); // horrible fallback
+
+                } else {
+                    waypoints[index].pos = SGGeod::fromGeoc(rGc);
+                }
+            } else {
+                waypoints[index].pos = SGGeod::fromGeoc(rGc);
+            }
+
+            waypoints[index].posValid = true;
+        } else if (ty == "dmeIntercept") {
+            DMEIntercept* di = (DMEIntercept*)wpt.get();
+
+            SGGeoc prevGc = SGGeoc::fromGeod(previous->turnExitPos);
+            SGGeoc navid = SGGeoc::fromGeod(wpt->position());
+            double distRad = di->dmeDistanceNm() * SG_NM_TO_RAD;
+            SGGeoc rGc;
+
+            double crs = di->courseDegMagnetic() + magVarFor(wpt->position());
+            SGGeoc bPt = SGGeodesy::advanceDegM(prevGc, crs, 1e5);
+
+            double dNm = pointsKnownDistanceFromGC(prevGc, bPt, navid, distRad);
+            if (dNm < 0.0) {
+                SG_LOG(SG_NAVAID, SG_WARN, "dmeIntercept failed");
+                waypoints[index].pos = wpt->position(); // horrible fallback
+            } else {
+                waypoints[index].pos = SGGeodesy::direct(previous->turnExitPos, crs, dNm * SG_NM_TO_METER);
+            }
+
+            waypoints[index].posValid = true;
+        } else if (ty == "vectors") {
+            waypoints[index].legCourseTrue = SGGeodesy::courseDeg(previous->turnExitPos, waypoints[index].pos);
+            waypoints[index].legCourseValid = true;
+            // no turn data
         }
-      } else {
-        waypoints[index].pos = SGGeod::fromGeoc(rGc);
-      }
-      
-      waypoints[index].posValid = true;
-    } else if (ty == "dmeIntercept") {
-      DMEIntercept* di = (DMEIntercept*) wpt.get();
-      
-      SGGeoc prevGc = SGGeoc::fromGeod(previous->turnExitPos);
-      SGGeoc navid = SGGeoc::fromGeod(wpt->position());
-      double distRad = di->dmeDistanceNm() * SG_NM_TO_RAD;
-      SGGeoc rGc;
-      
-      SGGeoc bPt;
-      double crs = di->courseDegMagnetic() + magVarFor(wpt->position());
-      SGGeodesy::advanceRadM(prevGc, crs, 100 * SG_NM_TO_RAD, bPt);
-      
-      double dNm = pointsKnownDistanceFromGC(prevGc, bPt, navid, distRad);
-      if (dNm < 0.0) {
-        SG_LOG(SG_NAVAID, SG_WARN, "dmeIntercept failed");
-        waypoints[index].pos = wpt->position(); // horrible fallback
-      } else {
-        waypoints[index].pos = SGGeodesy::direct(previous->turnExitPos, crs, dNm * SG_NM_TO_METER);
-      }
-      
-      waypoints[index].posValid = true;
-    } else if (ty == "vectors") {
-      waypoints[index].legCourseTrue = SGGeodesy::courseDeg(previous->turnExitPos, waypoints[index].pos);
-      waypoints[index].legCourseValid = true;
-      // no turn data
-    }
   }
   
   double computeVNAVAltitudeFt(int index)
@@ -917,7 +931,7 @@ void RoutePath::commonInit()
           nextIt->computeLegCourse(&(d->waypoints[i]), radiusM);
 
           if (nextIt->legCourseValid) {
-              d->waypoints[i].computeTurn(radiusM, d->constrainLegCourses, *nextIt);
+              d->waypoints[i].computeTurn(radiusM, d->constrainLegCourses, d->maxFlyByTurnAngleDeg, *nextIt);
           } else {
             // next waypoint has indeterminate course. Let's create a sharp turn
             // this can happen when the following point is ATC vectors, for example.
@@ -937,50 +951,67 @@ void RoutePath::commonInit()
 
 SGGeodVec RoutePath::pathForIndex(int index) const
 {
+  if ((index < 0) || (index >= static_cast<int>(d->waypoints.size()))) {
+    return {};
+  }
+
   const WayptData& w(d->waypoints[index]);
   const std::string& ty(w.wpt->type());
   SGGeodVec r;
-
-    if (d->waypoints[index].skipped) {
-        return SGGeodVec();
-    }
-
-  if (ty == "vectors") {
-      // ideally we'd show a stippled line to connect the route?
-    return SGGeodVec();
-  }
-
-  if (ty == "discontinuity") {
-    return SGGeodVec(); // no points for a discontinuity of course
-  }
-
-  if (ty == "via") {
-      return pathForVia(static_cast<Via*>(d->waypoints[index].wpt.get()), index);
+  
+  if (d->waypoints[index].skipped) {
+    return {};
   }
   
-    auto prevIt = d->previousValidWaypoint(index);
-    if (prevIt != d->waypoints.end()) {
-        prevIt->turnExitPath(r);
-      
-        SGGeod from = prevIt->turnExitPos,
-            to = w.turnEntryPos;
-      
-      // compute rounding offset, we want to round towards the direction of travel
-      // which depends on the east/west sign of the longitude change
-      double lonDelta = to.getLongitudeDeg() - from.getLongitudeDeg();
-      if (fabs(lonDelta) > 0.5) {
-        interpolateGreatCircle(from, to, r);
-      }
-    } // of have previous waypoint
+  // don't show any path
+  if (w.wpt->flag(WPT_HIDDEN)) {
+    return {};
+  }
 
-    w.turnEntryPath(r);
+  // also hide the path if the prev WP is hidden
+  auto prevIt = d->previousValidWaypoint(index);
+  if (prevIt != d->waypoints.end()) {
+      if (prevIt->wpt->flag(WPT_HIDDEN)) {
+          return {};
+      }
+  }
+
+  if (ty == "vectors") {
+    // ideally we'd show a stippled line to connect the route?
+    return {};
+  }
+  
+  if (ty == "discontinuity") {
+    return {}; // no points for a discontinuity of course
+  }
+  
+  if (ty == "via") {
+    return pathForVia(static_cast<Via*>(d->waypoints[index].wpt.get()), index);
+  }
+
+
+  if (prevIt != d->waypoints.end()) {
+    prevIt->turnExitPath(r);
     
-    // hold is the normal leg and then the hold waypoints as well
-    if (ty== "hold") {
-        const auto h = static_cast<Hold*>(d->waypoints[index].wpt.get());
-        const auto holdPath = pathForHold(h);
-        r.insert(r.end(), holdPath.begin(), holdPath.end());
+    SGGeod from = prevIt->turnExitPos,
+    to = w.turnEntryPos;
+    
+    // compute rounding offset, we want to round towards the direction of travel
+    // which depends on the east/west sign of the longitude change
+    double lonDelta = to.getLongitudeDeg() - from.getLongitudeDeg();
+    if (fabs(lonDelta) > 0.5) {
+      interpolateGreatCircle(from, to, r);
     }
+  } // of have previous waypoint
+  
+  w.turnEntryPath(r);
+  
+  // hold is the normal leg and then the hold waypoints as well
+  if (ty== "hold") {
+    const auto h = static_cast<Hold*>(d->waypoints[index].wpt.get());
+    const auto holdPath = pathForHold(h);
+    r.insert(r.end(), holdPath.begin(), holdPath.end());
+  }
   
   if (ty == "runway") {
     // runways get an extra point, at the end. this is particularly
@@ -1018,6 +1049,10 @@ void RoutePath::interpolateGreatCircle(const SGGeod& aFrom, const SGGeod& aTo, S
 
 SGGeod RoutePath::positionForIndex(int index) const
 {
+  if ((index < 0) || (index >= static_cast<int>(d->waypoints.size()))) {
+    return {};
+  }
+
   return d->waypoints[index].pos;
 }
 
@@ -1085,9 +1120,8 @@ SGGeodVec RoutePath::pathForHold(Hold* hold) const
 
 double RoutePath::computeDistanceForIndex(int index) const
 {
-    if ((index < 0) || (index >= (int) d->waypoints.size())) {
-        throw sg_range_exception("waypt index out of range",
-                                 "RoutePath::computeDistanceForIndex");
+    if ((index < 0) || (index >= static_cast<int>(d->waypoints.size()))) {
+      return 0.0;
     }
 
     auto it = d->waypoints.begin() + index;
@@ -1131,7 +1165,6 @@ double RoutePath::distanceForVia(Via* via, int index) const
     WayptVec enrouteWaypoints = via->expandToWaypoints(prevIt->wpt);
     double dist = 0.0;
 
-    WayptVec::const_iterator it;
     SGGeod legStart = prevIt->wpt->position();
     for (auto wp : enrouteWaypoints) {
         dist += SGGeodesy::distanceM(legStart, wp->position());
@@ -1143,6 +1176,11 @@ double RoutePath::distanceForVia(Via* via, int index) const
 
 double RoutePath::trackForIndex(int index) const
 {
+    const auto sz = static_cast<int>(d->waypoints.size());
+    if ((index < 0) || (index >= sz)) {
+        return 0.0;
+    }
+
     if (d->waypoints[index].skipped)
         return trackForIndex(index - 1);
 
@@ -1155,12 +1193,26 @@ double RoutePath::trackForIndex(int index) const
 
 double RoutePath::distanceForIndex(int index) const
 {
+    const auto sz = static_cast<int>(d->waypoints.size());
+    if ((index < 0) || (index >= sz)) {
+        return 0.0;
+    }
+
   return d->waypoints[index].pathDistanceM;
 }
 
 double RoutePath::distanceBetweenIndices(int from, int to) const
 {
-  return d->distanceBetweenIndices(from, to);
+    if (from < 0) {
+        from = 0;
+    }
+
+    const auto sz = static_cast<int>(d->waypoints.size());
+    if (to >= sz) {
+        to = sz - 1;
+    }
+
+    return d->distanceBetweenIndices(from, to);
 }
 
 SGGeod RoutePath::positionForDistanceFrom(int index, double distanceM) const
