@@ -16,13 +16,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "SceneryPager.hxx"
 #include <algorithm>
 #include <functional>
+#include <simgear/debug/logstream.hxx>
 
 using namespace osg;
 using namespace flightgear;
@@ -40,6 +39,7 @@ SceneryPager::SceneryPager(const SceneryPager& rhs) :
 
 SceneryPager::~SceneryPager()
 {
+    SG_LOG(SG_TERRAIN, SG_INFO, "Destroying scenery pager");
 }
 
 void SceneryPager::clearRequests()
