@@ -2,28 +2,10 @@
 //
 // Written by Curtis Olson, started January 1998.
 //
-// Copyright (C) 1997  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+// SPDX-FileCopyrightText: 1998 Curtis L. Olson
+// SPDX-License-Identifier: GPL-2.0-or-later
 
-
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <algorithm>
 #include <functional>
@@ -36,16 +18,20 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
+#include <simgear/scene/tgdb/VPBTechnique.hxx>
+#include <simgear/scene/tgdb/VPBLineFeatureRenderer.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/scene/material/matlib.hxx>
 
-#include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Model/validateSharedModels.hxx>
+#include <Scripting/NasalModelData.hxx>
+#include <Scripting/NasalSys.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/splash.hxx>
-#include <Scripting/NasalSys.hxx>
-#include <Scripting/NasalModelData.hxx>
 
 #include "scenery.hxx"
 #include "SceneryPager.hxx"
@@ -68,16 +54,16 @@ public:
         _lodBare(fgGetNode("/sim/rendering/static-lod/bare", true))
     {
         _useVBOsProp->addChangeListener(this, true);
-      
+
         _enableCacheProp->addChangeListener(this, true);
         if (_enableCacheProp->getType() == simgear::props::NONE) {
             _enableCacheProp->setBoolValue(true);
         }
-      
+
         if (_pagedLODMaximumProp->getType() == simgear::props::NONE) {
             // not set, use OSG default / environment value variable
-            osg::ref_ptr<osgViewer::Viewer> viewer(globals->get_renderer()->getViewer());
-            int current = viewer->getDatabasePager()->getTargetMaximumNumberOfPageLOD();
+            osg::ref_ptr<osgViewer::View> view(globals->get_renderer()->getView());
+            int current = view->getDatabasePager()->getTargetMaximumNumberOfPageLOD();
             _pagedLODMaximumProp->setIntValue(current);
         }
         _pagedLODMaximumProp->addChangeListener(this, true);
@@ -85,7 +71,7 @@ public:
         _lodBareDelta->addChangeListener(this, true);
         _lodRoughDelta->addChangeListener(this, true);
     }
-    
+
     ~TileManagerListener()
     {
         _useVBOsProp->removeChangeListener(this);
@@ -95,8 +81,8 @@ public:
         _lodBareDelta->removeChangeListener(this);
         _lodRoughDelta->removeChangeListener(this);
     }
-    
-    virtual void valueChanged(SGPropertyNode* prop)
+
+    void valueChanged(SGPropertyNode* prop) override
     {
         if (prop == _useVBOsProp) {
             bool useVBOs = prop->getBoolValue();
@@ -106,18 +92,25 @@ public:
             _manager->_enableCache = prop->getBoolValue();
         } else if (prop == _pagedLODMaximumProp) {
             int                             v = prop->getIntValue();
-            osg::ref_ptr<osgViewer::Viewer> viewer(globals->get_renderer()->getViewer());
-            viewer->getDatabasePager()->setTargetMaximumNumberOfPageLOD(v);
+            osg::ref_ptr<osgViewer::View> view(globals->get_renderer()->getView());
+            if (view) {
+              osgDB::DatabasePager* pager = view->getDatabasePager();
+              if (pager) pager->setTargetMaximumNumberOfPageLOD(v);
+            }
         } else if (prop == _lodDetailed || prop == _lodBareDelta || prop == _lodRoughDelta) {
-            // compatibility with earlier versions; set the static lod ranges appropriately as otherwise (bad) self managed
+            // compatibility with earlier versions; set the static LOD ranges appropriately as otherwise (bad) self managed
             // LOD on scenery with range animations doesn't work.
             // see also /sim/rendering/enable-range-lod-animations - which is false by default in > 2019.2 which also fixes
             // the scenery but in a more efficient way.
             _lodRough->setDoubleValue(_lodDetailed->getDoubleValue() + _lodRoughDelta->getDoubleValue());
             _lodBare->setDoubleValue(_lodRough->getDoubleValue() + _lodBareDelta->getDoubleValue());
         }
+
+        flightgear::addSentryBreadcrumb("Property:" + prop->getNameString() + " is now " +
+                                            prop->getStringValue(),
+                                        "info");
     }
-    
+
 private:
     FGTileMgr* _manager;
     SGPropertyNode_ptr _useVBOsProp,
@@ -135,8 +128,6 @@ FGTileMgr::FGTileMgr():
     state( Start ),
     last_state( Running ),
     scheduled_visibility(100.0),
-    _terra_sync(NULL),
-    _listener(NULL),
     _visibilityMeters(fgGetNode("/environment/visibility-m", true)),
     _lodDetailed(fgGetNode("/sim/rendering/static-lod/detailed", true)),
     _lodRoughDelta(fgGetNode("/sim/rendering/static-lod/rough-delta", true)),
@@ -145,14 +136,13 @@ FGTileMgr::FGTileMgr():
     _scenery_loaded(fgGetNode("/sim/sceneryloaded", true)),
     _scenery_override(fgGetNode("/sim/sceneryloaded-override", true)),
     _pager(FGScenery::getPagerSingleton()),
-    _enableCache(true)
+    _enableCache(true),
+    _use_vpb(false)
 {
 }
 
 
-FGTileMgr::~FGTileMgr()
-{
-}
+FGTileMgr::~FGTileMgr() = default;
 
 // Initialize the Tile Manager subsystem
 void FGTileMgr::init()
@@ -162,8 +152,7 @@ void FGTileMgr::init()
 
 void FGTileMgr::shutdown()
 {
-    delete _listener;
-    _listener = NULL;
+    _listener.reset();
 
     FGScenery* scenery = globals->get_scenery();
     if (scenery && scenery->get_terrain_branch()) {
@@ -178,15 +167,15 @@ void FGTileMgr::shutdown()
 void FGTileMgr::reinit()
 {
     SG_LOG( SG_TERRAIN, SG_INFO, "Initializing Tile Manager subsystem." );
-    _terra_sync = static_cast<simgear::SGTerraSync*> (globals->get_subsystem("terrasync"));
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
 
-  // drops the previous options reference
+    // drops the previous options reference
     _options = new simgear::SGReaderWriterOptions;
-    _listener = new TileManagerListener(this);
-    
+    _listener.reset(new TileManagerListener(this));
+
     materialLibChanged();
     _options->setPropertyNode(globals->get_props());
-    
+
     osgDB::FilePathList &fp = _options->getDatabasePathList();
     const PathList &sc = globals->get_fg_scenery();
     fp.clear();
@@ -194,21 +183,26 @@ void FGTileMgr::reinit()
         fp.push_back(it->utf8Str());
     }
     _options->setPluginStringData("SimGear::FG_ROOT", globals->get_fg_root().utf8Str());
-    
-    if (_terra_sync) {
-      _options->setPluginStringData("SimGear::TERRASYNC_ROOT", globals->get_terrasync_dir().utf8Str());
+
+    if (terraSync) {
+        _options->setPluginStringData("SimGear::TERRASYNC_ROOT", globals->get_terrasync_dir().utf8Str());
     }
-    
+
     if (!_disableNasalHooks->getBoolValue())
       _options->setModelData(new FGNasalModelDataProxy);
 
     double detailed = fgGetDouble("/sim/rendering/static-lod/detailed", SG_OBJECT_RANGE_DETAILED);
     double rough    = fgGetDouble("/sim/rendering/static-lod/rough-delta", SG_OBJECT_RANGE_ROUGH) + detailed;
     double bare     = fgGetDouble("/sim/rendering/static-lod/bare", SG_OBJECT_RANGE_BARE) + rough;
-        
+    double tile_min_expiry = fgGetDouble("/sim/rendering/plod-minimum-expiry-time-secs", SG_TILE_MIN_EXPIRY);
+    flightgear::addSentryBreadcrumb("PLod-minimum-expiry time=" + std::to_string(tile_min_expiry), "info");
+
+    _use_vpb = fgGetBool("/scenery/use-vpb");
+
     _options->setPluginStringData("SimGear::LOD_RANGE_BARE", std::to_string(bare));
     _options->setPluginStringData("SimGear::LOD_RANGE_ROUGH", std::to_string(rough));
     _options->setPluginStringData("SimGear::LOD_RANGE_DETAILED", std::to_string(detailed));
+    _options->setPluginStringData("SimGear::PAGED_LOD_EXPIRY", std::to_string(tile_min_expiry));
 
     string_list scenerySuffixes;
     for (auto node : fgGetNode("/sim/rendering/", true)->getChildren("scenery-path-suffix")) {
@@ -219,11 +213,14 @@ void FGTileMgr::reinit()
 
     if (scenerySuffixes.empty()) {
         // if preferences didn't load, use some default
-        scenerySuffixes = {"Objects", "Terrain"}; // defaut values
+        scenerySuffixes = {"Objects", "Terrain"}; // default values
     }
 
+    if (terraSync) {
+        terraSync->setSceneryPathSuffixes(scenerySuffixes);
+    }
     _options->setSceneryPathSuffixes(scenerySuffixes);
-  
+
     if (state != Start)
     {
       // protect against multiple scenery reloads and properly reset flags,
@@ -233,25 +230,25 @@ void FGTileMgr::reinit()
         return;
       }
     }
-  
+
     _scenery_loaded->setBoolValue(false);
     fgSetDouble("/sim/startup/splash-alpha", 1.0);
-    
+
     materialLibChanged();
 
     // remove all old scenery nodes from scenegraph and clear cache
     osg::Group* group = globals->get_scenery()->get_terrain_branch();
     group->removeChildren(0, group->getNumChildren());
     tile_cache.init();
-    
+
     // clear OSG cache, except on initial start-up
     if (state != Start)
     {
         osgDB::Registry::instance()->clearObjectCache();
     }
-    
+
     state = Inited;
-    
+
     previous_bucket.make_bad();
     current_bucket.make_bad();
     scheduled_visibility = 100.0;
@@ -270,13 +267,12 @@ void FGTileMgr::materialLibChanged()
 bool FGTileMgr::sched_tile( const SGBucket& b, double priority, bool current_view, double duration)
 {
     // see if tile already exists in the cache
-    TileEntry *t = tile_cache.get_tile( b );
+    STGTileEntry *t = tile_cache.get_stg_tile( b );
     if (!t)
     {
         // create a new entry
-        t = new TileEntry( b );
-        SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new tile entry for:" << b );
-
+        t = new STGTileEntry( b );
+        SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new STG tile entry for:" << b );
 
         // insert the tile into the cache, update will generate load request
         if ( tile_cache.insert_tile( t ) )
@@ -298,6 +294,36 @@ bool FGTileMgr::sched_tile( const SGBucket& b, double priority, bool current_vie
     // update tile's properties
     tile_cache.request_tile(t,priority,current_view,duration);
 
+    if (_use_vpb) {
+        VPBTileEntry *v = tile_cache.get_vpb_tile( b );
+
+        if (!v)
+        {
+            // create a new entry
+            v = new VPBTileEntry( b, _options );
+            SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new VPB tile entry for:" << b );
+
+            // insert the tile into the cache, update will generate load request
+            if ( tile_cache.insert_tile( v ) )
+            {
+                // Attach to scene graph
+                v->addToSceneGraph(globals->get_scenery()->get_terrain_branch());
+            } else {
+                // insert failed (cache full with no available entries to
+                // delete.)  Try again later
+                delete v;
+                return false;
+            }
+
+            SG_LOG( SG_TERRAIN, SG_DEBUG, "  New tile cache size " << (int)tile_cache.get_size() );
+        }
+
+        // update tile's properties.  We ensure VPB tiles have maximum priority - priority is calculated as
+        // _negative_ the square of the distance from the viewer to the tile.
+        // so by multiplying by 0.1 we increase the number towards 0.
+        tile_cache.request_tile(v,priority * 0.1,current_view,duration);
+    }
+
     return t->is_loaded();
 }
 
@@ -319,7 +345,7 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
             "scheduling needed tiles for " << curr_bucket
            << ", tile-width-m:" << tile_width << ", tile-height-m:" << tile_height);
 
-    
+
     // cout << "tile width = " << tile_width << "  tile_height = "
     //      << tile_height << endl;
     // starting with 2018.3 we will use deltas rather than absolutes as it is more intuitive for the user
@@ -339,17 +365,18 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
     // cout << "max cache size = " << tile_cache.get_max_cache_size()
     //      << " current cache size = " << tile_cache.get_size() << endl;
 
-    // clear flags of all tiles belonging to the previous view set 
+    // clear flags of all tiles belonging to the previous view set
     tile_cache.clear_current_view();
 
     // update timestamps, so all tiles scheduled now are *newer* than any tile previously loaded
     osg::FrameStamp* framestamp
-            = globals->get_renderer()->getViewer()->getFrameStamp();
+            = globals->get_renderer()->getFrameStamp();
     tile_cache.set_current_time(framestamp->getReferenceTime());
 
     SGBucket b;
 
     int x, y;
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
 
     /* schedule all tiles, use distance-based loading priority,
      * so tiles are loaded in innermost-to-outermost sequence. */
@@ -361,12 +388,12 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
             if (!b.isValid()) {
                 continue;
             }
-            
+
             float priority = (-1.0) * (x*x+y*y);
             sched_tile( b, priority, true, 0.0 );
-            
-            if (_terra_sync) {
-                _terra_sync->scheduleTile(b);
+
+            if (terraSync) {
+                terraSync->scheduleTile(b);
             }
         }
     }
@@ -378,14 +405,13 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
  */
 void FGTileMgr::update_queues(bool& isDownloadingScenery)
 {
-    osg::FrameStamp* framestamp
-        = globals->get_renderer()->getViewer()->getFrameStamp();
+    osg::FrameStamp* framestamp = globals->get_renderer()->getFrameStamp();
     double current_time = framestamp->getReferenceTime();
     double vis = _visibilityMeters->getDoubleValue();
     TileEntry *e;
     int loading=0;
     int sz=0;
-    
+
     tile_cache.set_current_time( current_time );
     tile_cache.reset_traversal();
 
@@ -396,9 +422,9 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
         {
             // Prepare the ssg nodes corresponding to each tile.
             // Set the ssg transform and update it's range selector
-            // based on current visibilty
+            // based on current visibility
             e->prep_ssg_node(vis);
-            
+
             if (!e->is_loaded()) {
                 bool nonExpiredOrCurrent = !e->is_expired(current_time) || e->is_current_view();
                 bool downloading = isTileDirSyncing(e->tileFileName);
@@ -429,7 +455,7 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
       dropTiles = true;
       drop_count = sz; // no limit on tiles to drop
     }
-  
+
     if (dropTiles)
     {
         long drop_index = _enableCache ? tile_cache.get_drop_tile() :
@@ -441,14 +467,19 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
             SG_LOG(SG_TERRAIN, SG_DEBUG, "Dropping:" << old->get_tile_bucket());
 
             tile_cache.clear_entry(drop_index);
-            
+
+            if (_use_vpb) {
+                // Clear out any VPB data - e.g. roads
+                simgear::VPBLineFeatureRenderer::unloadFeatures(old->get_tile_bucket());
+            }
+
             osg::ref_ptr<osg::Object> subgraph = old->getNode();
             old->removeFromSceneGraph();
             delete old;
             // zeros out subgraph ref_ptr, so subgraph is owned by
             // the pager and will be deleted in the pager thread.
             _pager->queueDeleteRequest(subgraph);
-          
+
             if (!_enableCache)
                 drop_index = tile_cache.get_first_expired_tile();
             // limit tiles dropped to drop_count
@@ -477,14 +508,20 @@ void FGTileMgr::update(double)
         bool fdmInited = fgGetBool("sim/fdm-initialized");
         bool positionFinalized = fgGetBool("sim/position-finalized");
         bool sceneryOverride = _scenery_override->getBoolValue();
-        
-        
+
+
     // we are done if final position is set and the scenery & FDM are done.
     // scenery-override can ignore the last two, but not position finalization.
         if (positionFinalized && (sceneryOverride || (isSceneryLoaded() && fdmInited)))
         {
             _scenery_loaded->setBoolValue(true);
             fgSplashProgress("");
+
+            // by this time Models/ should be in-sync, so run the check now.
+            // we could do this in a listener on the property, but we already have
+            // 'special' knowledge of Models/ in isTileDirSyncing so this feels
+            // reasonable.
+            flightgear::validateSharedModels();
         }
         else
         {
@@ -495,7 +532,7 @@ void FGTileMgr::update(double)
             } else {
                 fgSplashProgress("loading-scenery");
             }
-            
+
             // be nice to loader threads while waiting for initial scenery, reduce to 20fps
             SGTimeStamp::sleepForMSec(50);
         }
@@ -535,7 +572,7 @@ void FGTileMgr::schedule_tiles_at(const SGGeod& location, double range_m)
             scheduled_visibility = range_m;
             schedule_needed(current_bucket, range_m);
         }
-        
+
         // save bucket
         previous_bucket = current_bucket;
     } else if ( state == Start || state == Inited ) {
@@ -562,7 +599,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
 
     SGBucket bucket(position);
     available = sched_tile( bucket, priority, false, duration );
-  
+
     if ((!available)&&(duration==0.0)) {
         SG_LOG( SG_TERRAIN, SG_DEBUG, "schedule_scenery: Scheduling tile at bucket:" << bucket << " return false" );
         return false;
@@ -576,7 +613,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
     double tile_r = 0.5*sqrt(tile_width*tile_width + tile_height*tile_height);
     double max_dist = tile_r + range_m;
     double max_dist2 = max_dist*max_dist;
-    
+
     int xrange = (int)fabs(range_m / tile_width) + 1;
     int yrange = (int)fabs(range_m / tile_height) + 1;
 
@@ -591,7 +628,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
                 if (!b.isValid()) {
                     continue;
                 }
-                
+
                 double distance2 = distSqr(cartPos, SGVec3d::fromGeod(b.get_center()));
                 // Do not ask if it is just the next tile but way out of range.
                 if (distance2 <= max_dist2)
@@ -619,14 +656,20 @@ bool FGTileMgr::isSceneryLoaded()
 
 bool FGTileMgr::isTileDirSyncing(const std::string& tileFileName) const
 {
-    if (!_terra_sync) {
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
+    if (!terraSync) {
         return false;
     }
-    
+
+    // if Models is syncing, also wait for it, since otherwise
+    // we get load errors
+    if (terraSync->isDataDirPending("Models")) {
+        return true;
+    }
+
     std::string nameWithoutExtension = tileFileName.substr(0, tileFileName.size() - 4);
     long int bucketIndex = simgear::strutils::to_int(nameWithoutExtension);
     SGBucket bucket(bucketIndex);
-    
-    return _terra_sync->isTileDirPending(bucket.gen_base_path());
-}
 
+    return terraSync->isTileDirPending(bucket.gen_base_path());
+}
