@@ -173,8 +173,8 @@ View* View::createFromProperties(SGPropertyNode_ptr config, int view_index)
 
     View* v = nullptr;
     // supporting two types "lookat" = 1 and "lookfrom" = 0
-    const char *type = config->getParent()->getStringValue("type");
-    if (!strcmp(type, "lookat")) {
+    std::string type = config->getParent()->getStringValue("type");
+    if (type == "lookat") {
         bool at_model = config->getBoolValue("at-model");
         int at_model_index = config->getIntValue("at-model-idx");
 
@@ -748,6 +748,15 @@ static void getViewOffsets(
         offset.z() -= ViewPropertyEvaluator::getDoubleValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/z-offset-m)");
         offset_m += offset;
     }
+
+    /* Handle headtracking :
+    Add offsets coming from the headtracker if there is one. The headtracker offsets have to be added ; they
+    cannot replace /sim/currentview/config/?-offset-m otherwise the offsets of each view are lost, making the
+    headtracking position unusable. */
+    offset_m.x() += ViewPropertyEvaluator::getDoubleValue("(/sim/viewer/camera-offset-x)");
+    offset_m.y() += ViewPropertyEvaluator::getDoubleValue("(/sim/viewer/camera-offset-y)");
+    offset_m.z() += ViewPropertyEvaluator::getDoubleValue("(/sim/viewer/camera-offset-z)");
+
 }
 
 // recalculate for LookFrom view type...
