@@ -1,22 +1,6 @@
-/*
- * Copyright (C) 2020 James Turner
- *
- * This file is part of the program FlightGear.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
 
+// SPDX-FileCopyrightText: (C) 2020 James Turner
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "testGC.hxx"
 
@@ -38,13 +22,13 @@ void NasalGCTests::setUp()
     fgInitAllowedPaths();
     auto nasalNode = globals->get_props()->getNode("nasal", true);
 
-    globals->add_subsystem("prop-interpolator", new FGInterpolator, SGSubsystemMgr::INIT);
+    globals->get_subsystem_mgr()->add<FGInterpolator>();
 
     globals->get_subsystem_mgr()->bind();
     globals->get_subsystem_mgr()->init();
     
     global_nasalMinimalInit = true;
-    globals->add_new_subsystem<FGNasalSys>(SGSubsystemMgr::INIT);
+    globals->get_subsystem_mgr()->add<FGNasalSys>();
     
     globals->get_subsystem_mgr()->postinit();
 }
@@ -53,6 +37,7 @@ void NasalGCTests::setUp()
 // Clean up after each test.
 void NasalGCTests::tearDown()
 {
+    global_nasalMinimalInit = false;
     FGTestApi::tearDown::shutdownTestGlobals();
 }
 
